@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -27,14 +27,12 @@
 #include "plugin_hooks.h"
 #include "plugin_common.h"
 #include "amqp_plugin.h"
-#include "ip_flow.h"
-#include "classifier.h"
-#include "crc32.c"
-#ifdef WITH_JANSSON
-#include <jansson.h>
-#else
+#ifndef WITH_JANSSON
 #error "--enable-rabbitmq requires --enable-jansson"
 #endif
+#ifdef WITH_AVRO
+#include <avro.h>
+#endif
 
 /* Functions */
 void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr) 
@@ -51,6 +49,7 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   struct plugins_list_entry *plugin_data = ((struct channels_list_entry *)ptr)->plugin;
   int datasize = ((struct channels_list_entry *)ptr)->datasize;
   u_int32_t bufsz = ((struct channels_list_entry *)ptr)->bufsize;
+  pid_t core_pid = ((struct channels_list_entry *)ptr)->core_pid;
   struct networks_file_data nfd;
 
   unsigned char *rgptr;
@@ -71,13 +70,43 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   P_set_signals();
   P_init_default_values();
   P_config_checks();
-  pipebuf = (unsigned char *) Malloc(config.buffer_size);
+  pipebuf = (unsigned char *) pm_malloc(config.buffer_size);
   memset(pipebuf, 0, config.buffer_size);
 
   timeout = config.sql_refresh_time*1000;
 
   if (!config.sql_user) config.sql_user = rabbitmq_user;
   if (!config.sql_passwd) config.sql_passwd = rabbitmq_pwd;
+  if (!config.message_broker_output) config.message_broker_output = PRINT_OUTPUT_JSON;
+
+  if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+#ifdef WITH_AVRO
+    Log(LOG_INFO, "INFO ( %s/%s ): AVRO: building schema.\n", config.name, config.type);
+    avro_acct_schema = build_avro_schema(config.what_to_count, config.what_to_count_2);
+
+    if (config.avro_schema_output_file) {
+      FILE *avro_fp = open_output_file(config.avro_schema_output_file, "w", TRUE);
+      avro_writer_t avro_schema_writer = avro_writer_file(avro_fp);
+
+      if (avro_schema_to_json(avro_acct_schema, avro_schema_writer)) {
+        Log(LOG_ERR, "ERROR ( %s/%s ): AVRO: unable to dump schema: %s\n",
+            config.name, config.type, avro_strerror());
+        exit_plugin(EXIT_FAILURE);
+      }
+
+      close_output_file(avro_fp);
+    }
+
+    if (!config.avro_buffer_size) config.avro_buffer_size = 4096;
+
+    avro_buf = malloc(config.avro_buffer_size);
+
+    if (!avro_buf) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (avro_buf). Exiting ..\n", config.name, config.type);
+      exit_plugin(EXIT_FAILURE);
+    }
+#endif
+  }
 
   if ((config.sql_table && strchr(config.sql_table, '$')) && config.sql_multi_values) {
     Log(LOG_ERR, "ERROR ( %s/%s ): dynamic 'amqp_routing_key' is not compatible with 'amqp_multi_values'. Exiting.\n", config.name, config.type);
@@ -127,7 +156,7 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   if (config.pipe_amqp) {
     plugin_pipe_amqp_compile_check();
     pipe_fd = plugin_pipe_amqp_connect_to_consume(amqp_host, plugin_data);
-    amqp_timeout = plugin_pipe_amqp_set_poll_timeout(amqp_host, pipe_fd);
+    amqp_timeout = plugin_pipe_set_retry_timeout(&amqp_host->btimers, pipe_fd);
   }
   else setnonblocking(pipe_fd);
 
@@ -182,9 +211,9 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     if (config.pipe_amqp && pipe_fd == ERR) {
       if (timeout == amqp_timeout) {
         pipe_fd = plugin_pipe_amqp_connect_to_consume(amqp_host, plugin_data);
-        amqp_timeout = plugin_pipe_amqp_set_poll_timeout(amqp_host, pipe_fd);
+        amqp_timeout = plugin_pipe_set_retry_timeout(&amqp_host->btimers, pipe_fd);
       }
-      else amqp_timeout = plugin_pipe_amqp_calc_poll_timeout_diff(amqp_host, idata.now);
+      else amqp_timeout = plugin_pipe_calc_retry_timeout_diff(&amqp_host->btimers, idata.now);
     }
 
     switch (ret) {
@@ -214,12 +243,13 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
           else {
             rg_err_count++;
             if (config.debug || (rg_err_count > MAX_RG_COUNT_ERR)) {
-              Log(LOG_ERR, "ERROR ( %s/%s ): We are missing data.\n", config.name, config.type);
-              Log(LOG_ERR, "If you see this message once in a while, discard it. Otherwise some solutions follow:\n");
-              Log(LOG_ERR, "- increase shared memory size, 'plugin_pipe_size'; now: '%u'.\n", config.pipe_size);
-              Log(LOG_ERR, "- increase buffer size, 'plugin_buffer_size'; now: '%u'.\n", config.buffer_size);
-              Log(LOG_ERR, "- increase system maximum socket size.\n\n");
+              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%llu plugin_pipe_size=%llu).\n",
+			config.name, config.type, config.buffer_size, config.pipe_size);
+              Log(LOG_WARNING, "WARN ( %s/%s ): Increase values or look for plugin_buffer_size, plugin_pipe_size in CONFIG-KEYS document.\n\n",
+			config.name, config.type);
             }
+
+	    rg->ptr = (rg->base + status->last_buf_off);
             seq = ((struct ch_buf_hdr *)rg->ptr)->seq;
           }
         }
@@ -233,15 +263,20 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
         if (ret) pipe_fd = ERR;
 
         seq = ((struct ch_buf_hdr *)pipebuf)->seq;
-        amqp_timeout = plugin_pipe_amqp_set_poll_timeout(amqp_host, pipe_fd);
+        amqp_timeout = plugin_pipe_set_retry_timeout(&amqp_host->btimers, pipe_fd);
       }
 
       /* lazy refresh time handling */ 
       if (idata.now > refresh_deadline) P_cache_handle_flush_event(&pt);
 
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
-      Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received seq=%u num_entries=%u\n", config.name, config.type, seq, ((struct ch_buf_hdr *)pipebuf)->num);
 
+      if (config.debug_internal_msg)
+        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received cpid=%u len=%llu seq=%u num_entries=%u\n",
+                config.name, config.type, core_pid, ((struct ch_buf_hdr *)pipebuf)->len,
+                seq, ((struct ch_buf_hdr *)pipebuf)->num);
+
+      if (!config.pipe_check_core_pid || ((struct ch_buf_hdr *)pipebuf)->core_pid == core_pid) {
       while (((struct ch_buf_hdr *)pipebuf)->num > 0) {
         for (num = 0; primptrs_funcs[num]; num++)
           (*primptrs_funcs[num])((u_char *)data, &extras, &prim_ptrs);
@@ -269,6 +304,7 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
           data = (struct pkt_data *) dataptr;
 	}
       }
+      }
 
       if (!config.pipe_amqp) goto read_data;
     }
@@ -298,6 +334,11 @@ void amqp_cache_purge(struct chained_cache *queue[], int index)
   json_t *array = json_array();
 #endif
 
+#ifdef WITH_AVRO
+  avro_writer_t avro_writer;
+  int avro_buffer_full = FALSE;
+#endif
+
   /* setting some defaults */
   if (!config.sql_host) config.sql_host = default_amqp_host;
   if (!config.sql_db) config.sql_db = default_amqp_exchange;
@@ -349,7 +390,31 @@ void amqp_cache_purge(struct chained_cache *queue[], int index)
   Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - START (PID: %u) ***\n", config.name, config.type, writer_pid);
   start = time(NULL);
 
+  if (config.print_markers) {
+    if (config.message_broker_output & PRINT_OUTPUT_JSON) {
+      void *json_obj;
+      char *json_str;
+
+      json_obj = compose_purge_init_json(writer_pid);
+      if (json_obj) json_str = compose_json_str(json_obj);
+      if (json_str) {
+        Log(LOG_DEBUG, "DEBUG ( %s/%s ): %s\n\n", config.name, config.type, json_str);
+        ret = p_amqp_publish_string(&amqpp_amqp_host, json_str);
+
+        free(json_str);
+        json_str = NULL;
+      }
+    }
+  }
+
+#ifdef WITH_AVRO
+  if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+    avro_writer = avro_writer_memory(avro_buf, config.avro_buffer_size);
+  }
+#endif
+
   for (j = 0; j < index; j++) {
+    void *json_obj;
     char *json_str;
 
     if (queue[j]->valid != PRINT_CACHE_COMMITTED) continue;
@@ -372,81 +437,182 @@ void amqp_cache_purge(struct chained_cache *queue[], int index)
 
     if (queue[j]->valid == PRINT_CACHE_FREE) continue;
 
-    json_str = compose_json(config.what_to_count, config.what_to_count_2, queue[j]->flow_type,
-                         &queue[j]->primitives, pbgp, pnat, pmpls, pcust, pvlen, queue[j]->bytes_counter,
-			 queue[j]->packet_counter, queue[j]->flow_counter, queue[j]->tcp_flags,
-			 &queue[j]->basetime, queue[j]->stitch);
+    if (config.message_broker_output & PRINT_OUTPUT_JSON) {
+      json_obj = compose_json(config.what_to_count, config.what_to_count_2, queue[j]->flow_type,
+                           &queue[j]->primitives, pbgp, pnat, pmpls, pcust, pvlen, queue[j]->bytes_counter,
+                           queue[j]->packet_counter, queue[j]->flow_counter, queue[j]->tcp_flags,
+                           &queue[j]->basetime, queue[j]->stitch);
 
+      json_str = compose_json_str(json_obj);
+    }
+    else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+#ifdef WITH_AVRO
+      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_schema);
+      avro_value_t avro_value = compose_avro(config.what_to_count, config.what_to_count_2, queue[j]->flow_type,
+                           &queue[j]->primitives, pbgp, pnat, pmpls, pcust, pvlen, queue[j]->bytes_counter,
+                           queue[j]->packet_counter, queue[j]->flow_counter, queue[j]->tcp_flags,
+                           &queue[j]->basetime, queue[j]->stitch, avro_iface);
+      size_t avro_value_size;
+
+      avro_value_sizeof(&avro_value, &avro_value_size);
+
+      if (avro_value_size > config.avro_buffer_size) {
+        Log(LOG_ERR, "ERROR ( %s/%s ): AVRO; insufficient buffer size (avro_buffer_size=%u)\n",
+            config.name, config.type, config.avro_buffer_size);
+        Log(LOG_ERR, "ERROR ( %s/%s ): AVRO: increase value or look for avro_buffer_size in CONFIG-KEYS document.\n\n",
+            config.name, config.type);
+        exit_plugin(EXIT_FAILURE);
+      }
+      else if (avro_value_size >= (config.avro_buffer_size - avro_writer_tell(avro_writer))) {
+        avro_buffer_full = TRUE;
+        j--;
+      }
+      else if (avro_value_write(avro_writer, &avro_value)) {
+        Log(LOG_ERR, "ERROR ( %s/%s ): ARVO: unable to write value: %s\n",
+            config.name, config.type, avro_strerror());
+        exit_plugin(EXIT_FAILURE);
+      }
+      else {
+        mv_num++;
+      }
+
+      avro_value_decref(&avro_value);
+      avro_value_iface_decref(avro_iface);
+#else
+      if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_avro(): AVRO object not created due to missing --enable-avro\n", config.name, config.type);
+#endif
+    }
+
+    if (config.message_broker_output & PRINT_OUTPUT_JSON) {
 #ifdef WITH_JANSSON
-    if (json_str && config.sql_multi_values) {
-      json_t *elem = NULL;
-      char *tmp_str = json_str;
-      int do_free = FALSE;
+      if (json_str && config.sql_multi_values) {
+        json_t *elem = NULL;
+        char *tmp_str = json_str;
+        int do_free = FALSE;
 
-      if (json_array_size(array) >= config.sql_multi_values) {
-	json_str = json_dumps(array, 0);
-	json_array_clear(array);
+        if (json_array_size(array) >= config.sql_multi_values) {
+	  json_str = json_dumps(array, JSON_PRESERVE_ORDER);
+          json_array_clear(array);
+          mv_num_save = mv_num;
+          mv_num = 0;
+        }
+        else do_free = TRUE;
+
+        elem = json_loads(tmp_str, 0, NULL);
+        json_array_append_new(array, elem);
+        mv_num++;
+
+        if (do_free) {
+          free(json_str);
+          json_str = NULL;
+        }
+      }
+#endif
+
+      if (json_str) {
+        if (is_routing_key_dyn) {
+          P_handle_table_dyn_strings(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, queue[j]);
+          p_amqp_set_routing_key(&amqpp_amqp_host, dyn_amqp_routing_key);
+        }
+
+        if (config.amqp_routing_key_rr) {
+          P_handle_table_dyn_rr(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, &amqpp_amqp_host.rk_rr);
+          p_amqp_set_routing_key(&amqpp_amqp_host, dyn_amqp_routing_key);
+        }
+
+        Log(LOG_DEBUG, "DEBUG ( %s/%s ): %s\n\n", config.name, config.type, json_str);
+        ret = p_amqp_publish_string(&amqpp_amqp_host, json_str);
+        free(json_str);
+        json_str = NULL;
+
+        if (!ret) {
+          if (!config.sql_multi_values) qn++;
+          else qn += mv_num_save;
+        }
+        else break;
+      }
+    }
+    else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+#ifdef WITH_AVRO
+      if (!config.sql_multi_values || (mv_num >= config.sql_multi_values) || avro_buffer_full) {
+        if (is_routing_key_dyn) {
+          P_handle_table_dyn_strings(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, queue[j]);
+          p_amqp_set_routing_key(&amqpp_amqp_host, dyn_amqp_routing_key);
+        }
+
+        if (config.amqp_routing_key_rr) {
+          P_handle_table_dyn_rr(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, &amqpp_amqp_host.rk_rr);
+          p_amqp_set_routing_key(&amqpp_amqp_host, dyn_amqp_routing_key);
+        }
+
+        ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_writer_tell(avro_writer));
+        avro_writer_reset(avro_writer);
+        avro_buffer_full = FALSE;
         mv_num_save = mv_num;
         mv_num = 0;
+
+        if (!ret) qn += mv_num_save;
+        else break;
       }
-      else do_free = TRUE;
+#endif
+    }
+  }
 
-      elem = json_loads(tmp_str, 0, NULL);
-      json_array_append_new(array, elem);
-      mv_num++;
+  if (config.sql_multi_values) {
+    if (config.message_broker_output & PRINT_OUTPUT_JSON) {
+#ifdef WITH_JANSSON
+      if (json_array_size(array)) {
+        char *json_str;
+
+        json_str = json_dumps(array, JSON_PRESERVE_ORDER);
+        json_array_clear(array);
+        json_decref(array);
+
+        if (json_str) {
+          /* no handling of dyn routing keys here: not compatible */
+          Log(LOG_DEBUG, "DEBUG ( %s/%s ): %s\n\n", config.name, config.type, json_str);
+          ret = p_amqp_publish_string(&amqpp_amqp_host, json_str);
+          free(json_str);
+          json_str = NULL;
+
+          if (!ret) qn += mv_num;
+	}
+      }
+#endif
+    }
+    else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+#ifdef WITH_AVRO
+      if (avro_writer_tell(avro_writer)) {
+        ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_writer_tell(avro_writer));
+        avro_writer_free(avro_writer);
+
+        if (!ret) qn += mv_num;
+      }
+#endif
+    }
+  }
+
+  duration = time(NULL)-start;
+
+  if (config.print_markers) {
+    if (config.message_broker_output & PRINT_OUTPUT_JSON) {
+      void *json_obj;
+      char *json_str;
+
+      json_obj = compose_purge_close_json(writer_pid, qn, saved_index, duration);
+      if (json_obj) json_str = compose_json_str(json_obj);
+      if (json_str) {
+        Log(LOG_DEBUG, "DEBUG ( %s/%s ): %s\n\n", config.name, config.type, json_str);
+        ret = p_amqp_publish_string(&amqpp_amqp_host, json_str);
 
-      if (do_free) {
         free(json_str);
         json_str = NULL;
       }
     }
-#endif
-
-    if (json_str) {
-      if (is_routing_key_dyn) {
-	amqp_handle_routing_key_dyn_strings(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, queue[j]);
-	p_amqp_set_routing_key(&amqpp_amqp_host, dyn_amqp_routing_key);
-      }
-
-      if (config.amqp_routing_key_rr) {
-        p_amqp_handle_routing_key_dyn_rr(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, &amqpp_amqp_host.rk_rr);
-	p_amqp_set_routing_key(&amqpp_amqp_host, dyn_amqp_routing_key);
-      }
-
-      ret = p_amqp_publish_string(&amqpp_amqp_host, json_str);
-      free(json_str);
-      json_str = NULL;
-
-      if (!ret) {
-	if (!config.sql_multi_values) qn++;
-	else qn += mv_num_save;
-      }
-      else break;
-    }
   }
 
-#ifdef WITH_JANSSON
-  if (config.sql_multi_values && json_array_size(array)) {
-    char *json_str;
-
-    json_str = json_dumps(array, 0);
-    json_array_clear(array);
-    json_decref(array);
-
-    if (json_str) {
-      /* no handling of dyn routing keys here: not compatible */
-      ret = p_amqp_publish_string(&amqpp_amqp_host, json_str);
-      free(json_str);
-      json_str = NULL;
-
-      if (!ret) qn += mv_num;
-    }
-  }
-#endif
-
   p_amqp_close(&amqpp_amqp_host, FALSE);
 
-  duration = time(NULL)-start;
   Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: %u/%u, ET: %u) ***\n",
 		config.name, config.type, writer_pid, qn, saved_index, duration);
 
@@ -454,83 +620,3 @@ void amqp_cache_purge(struct chained_cache *queue[], int index)
 
   if (empty_pcust) free(empty_pcust);
 }
-
-void amqp_handle_routing_key_dyn_strings(char *new, int newlen, char *old, struct chained_cache *elem)
-{
-  int oldlen, ptr_len;
-  char peer_src_ip_string[] = "$peer_src_ip", post_tag_string[] = "$post_tag";
-  char pre_tag_string[] = "$pre_tag";
-  char *ptr_start, *ptr_end;
-
-  oldlen = strlen(old);
-  if (oldlen <= newlen) strcpy(new, old);
-  else {
-    strncpy(new, old, newlen);
-    return;
-  }
-
-  if (!strchr(new, '$')) return;
-  ptr_start = strstr(new, peer_src_ip_string);
-  if (ptr_start) {
-    char ip_address[INET6_ADDRSTRLEN];
-    char buf[newlen];
-    int len;
-
-    if (!elem || !elem->pbgp) goto out_peer_src_ip; 
-
-    len = strlen(ptr_start);
-    ptr_end = ptr_start;
-    ptr_len = strlen(peer_src_ip_string);
-    ptr_end += ptr_len;
-    len -= ptr_len;
-
-    addr_to_str(ip_address, &elem->pbgp->peer_src_ip);
-    snprintf(buf, newlen, "%s", ip_address);
-    strncat(buf, ptr_end, len);
-
-    len = strlen(buf);
-    *ptr_start = '\0';
-    strncat(new, buf, len);
-  }
-  out_peer_src_ip:
-
-  if (!strchr(new, '$')) return;
-  ptr_start = strstr(new, post_tag_string);
-  if (ptr_start) {
-    char buf[newlen];
-    int len;
-
-    len = strlen(ptr_start);
-    ptr_end = ptr_start;
-    ptr_len = strlen(post_tag_string);
-    ptr_end += ptr_len;
-    len -= ptr_len;
-
-    snprintf(buf, newlen, "%u", config.post_tag);
-    strncat(buf, ptr_end, len);
-
-    len = strlen(buf);
-    *ptr_start = '\0';
-    strncat(new, buf, len);
-  }
-
-  if (!strchr(new, '$')) return;
-  ptr_start = strstr(new, pre_tag_string);
-  if (ptr_start) {
-    char buf[newlen];
-    int len;
-
-    len = strlen(ptr_start);
-    ptr_end = ptr_start;
-    ptr_len = strlen(pre_tag_string);
-    ptr_end += ptr_len;
-    len -= ptr_len;
-
-    snprintf(buf, newlen, "%u", elem->primitives.tag);
-    strncat(buf, ptr_end, len);
-
-    len = strlen(buf);
-    *ptr_start = '\0';
-    strncat(new, buf, len);
-  }
-}
