@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2014 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -24,6 +24,8 @@
 #include <amqp_tcp_socket.h>
 #include <sys/poll.h>
 
+/* defines */
+
 /* structures */
 
 /* prototypes */
@@ -34,7 +36,6 @@
 #endif
 EXT void amqp_plugin(int, struct configuration *, void *);
 EXT void amqp_cache_purge(struct chained_cache *[], int);
-EXT void amqp_handle_routing_key_dyn_strings(char *, int, char *, struct chained_cache *);
 
 /* global vars */
 EXT void (*insert_func)(struct primitives_ptrs *, struct insert_data *); /* pointer to INSERT function */
@@ -47,4 +48,9 @@ EXT int qq_ptr, pp_size, pb_size, pn_size, pm_size, dbc_size, quit;
 EXT time_t refresh_deadline;
 
 EXT struct timeval sbasetime;
+
+#ifdef WITH_AVRO
+EXT char *avro_buf;
+EXT avro_schema_t avro_acct_schema;
+#endif
 #undef EXT
