@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -19,12 +19,9 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include "bgp_hash.h"
 #include "bgp_prefix.h"
 #include "bgp_packet.h"
 #include "bgp_table.h"
-#include "bgp_community.h"
-#include "bgp_ecommunity.h"
 #include "bgp_logdump.h"
 
 #ifndef _BGP_H_
@@ -80,19 +77,83 @@
    nfacctd_bgp_stdcomm_pattern, nfacctd_bgp_extcomm_pattern */
 #define MAX_BGP_COMM_PATTERNS 16
 
-/* requires as_t */
-#include "bgp_aspath.h"
-
 /* structures */
+struct bgp_dump_event {
+  struct timeval tstamp;
+  char tstamp_str[SRVBUFLEN];
+  int period;
+};
+
+struct bgp_rt_structs {
+  struct hash *attrhash;
+  struct hash *ashash;
+  struct hash *comhash;
+  struct hash *ecomhash;
+  struct bgp_table *rib[AFI_MAX][SAFI_MAX];
+};
+
+struct bgp_misc_structs {
+  struct bgp_peer_log *peers_log;
+  u_int64_t log_seq;
+  struct timeval log_tstamp;
+  char log_tstamp_str[SRVBUFLEN];
+  struct bgp_dump_event dump;
+  char *peer_str; /* "bmp_router", "peer_src_ip", "peer_ip", etc. */
+  char *log_str; /* BGP, BMP, thread, daemon, etc. */
+  int is_thread;
+
+#if defined WITH_RABBITMQ
+  struct p_amqp_host *msglog_amqp_host;
+#endif
+#if defined WITH_KAFKA
+  struct p_kafka_host *msglog_kafka_host;
+#endif
+  
+  int max_peers;
+  char *neighbors_file;
+  char *dump_file;
+  char *dump_amqp_routing_key;
+  int dump_amqp_routing_key_rr;
+  char *dump_kafka_topic;
+  int dump_kafka_topic_rr;
+  char *msglog_file;
+  int msglog_output;
+  char *msglog_amqp_routing_key;
+  int msglog_amqp_routing_key_rr;
+  char *msglog_kafka_topic;
+  int msglog_kafka_topic_rr;
+  void (*bgp_peer_log_msg_extras)(struct bgp_peer *, int, void *);
+
+  int table_peer_buckets;
+  int table_per_peer_buckets;
+  int table_attr_hash_buckets;
+  int table_per_peer_hash;
+  u_int32_t (*route_info_modulo)(struct bgp_peer *, path_id_t *);
+  struct bgp_peer *(*bgp_lookup_find_peer)(struct sockaddr *, struct xflow_status_entry *, u_int16_t, int);
+  int (*bgp_lookup_node_match_cmp)(struct bgp_info *, struct node_match_cmp_term2 *);
+
+  int msglog_backend_methods;
+  int dump_backend_methods;
+};
+
+struct bgp_peer_stats {
+    u_int32_t packets; /* Datagrams received */
+    u_int32_t packet_bytes; /* Bytes read off the socket */
+    u_int32_t msg_bytes; /* Bytes in the decoded messages */
+    u_int32_t msg_errors; /* Errors detected in message content */
+    time_t last_check; /* Timestamp when stats were last checked */
+};
+
 struct bgp_peer_buf {
   char *base;
-  int len;
-  int truncated_len;
+  u_int32_t len;
+  u_int32_t truncated_len;
 };
 
 struct bgp_peer {
   int fd;
   int lock;
+  int type; /* ie. BGP vs BMP */
   u_int8_t status;
   as_t myas;
   as_t as;
@@ -106,9 +167,29 @@ struct bgp_peer {
   char *cap_4as;
   u_int8_t cap_add_paths;
   u_int32_t msglen;
+  struct bgp_peer_stats stats;
   struct bgp_peer_buf buf;
   struct bgp_peer_log *log;
-  void *bmp_se; /* struct bmp_dump_se_ll */
+
+  /*
+     bmp_peer.self.bmp_se:		pointer to struct bmp_dump_se_ll
+     bmp_peer.bgp_peers[n].bmp_se:	backpointer to parent struct bmp_peer
+  */
+  void *bmp_se;
+};
+
+/* these includes require definition of bgp_rt_structs and bgp_peer */
+#include "bgp_aspath.h"
+#include "bgp_community.h"
+#include "bgp_ecommunity.h"
+/* this include requires definition of bgp_peer */
+#include "bgp_hash.h"
+
+struct bgp_peer_batch {
+  int num;
+  int num_current;
+  time_t base_stamp;
+  int interval;
 };
 
 struct bgp_nlri {
@@ -129,8 +210,8 @@ struct bgp_attr {
   u_int32_t med;
   u_int32_t local_pref;
   struct {
-	u_int32_t as;
-	u_char ttl;
+    u_int32_t as;
+    u_char ttl;
   } pathlimit;
   u_char origin;
 };
@@ -140,6 +221,10 @@ struct bgp_comm_range {
   u_int32_t last;
 };
 
+#include "bgp_msg.h"
+#include "bgp_lookup.h"
+#include "bgp_util.h"
+
 /* prototypes */
 #if (!defined __BGP_C)
 #define EXT extern
@@ -148,75 +233,25 @@ struct bgp_comm_range {
 #endif
 EXT void nfacctd_bgp_wrapper();
 EXT void skinny_bgp_daemon();
-EXT int bgp_marker_check(struct bgp_header *, int);
-EXT int bgp_keepalive_msg(char *);
-EXT int bgp_open_msg(char *, char *, int, struct bgp_peer *);
-EXT int bgp_update_msg(struct bgp_peer *, char *);
-EXT int bgp_attr_parse(struct bgp_peer *, struct bgp_attr *, char *, int, struct bgp_nlri *, struct bgp_nlri *);
-EXT int bgp_attr_parse_community(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, u_int8_t);
-EXT int bgp_attr_parse_ecommunity(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, u_int8_t);
-EXT int bgp_attr_parse_aspath(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, u_int8_t);
-EXT int bgp_attr_parse_as4path(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, u_int8_t, struct aspath **);
-EXT int bgp_attr_parse_nexthop(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, u_int8_t);
-EXT int bgp_attr_parse_med(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, u_char);
-EXT int bgp_attr_parse_local_pref(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, u_char);
-EXT int bgp_attr_parse_origin(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, u_char);
-EXT int bgp_attr_parse_mp_reach(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, struct bgp_nlri *);
-EXT int bgp_attr_parse_mp_unreach(struct bgp_peer *, u_int16_t, struct bgp_attr *, char *, struct bgp_nlri *);
-EXT int bgp_nlri_parse(struct bgp_peer *, void *, struct bgp_nlri *);
-EXT int bgp_process_update(struct bgp_peer *, struct prefix *, void *, afi_t, safi_t, rd_t *, path_id_t *, char *);
-EXT int bgp_process_withdraw(struct bgp_peer *, struct prefix *, void *, afi_t, safi_t, rd_t *, path_id_t *, char *);
-EXT int bgp_afi2family(int);
-EXT int bgp_rd2str(char *, rd_t *);
-EXT int bgp_str2rd(rd_t *, char *);
-EXT struct bgp_info_extra *bgp_info_extra_new();
-EXT void bgp_info_extra_free(struct bgp_info_extra **);
-EXT struct bgp_info_extra *bgp_info_extra_get(struct bgp_info *);
-EXT struct bgp_info *bgp_info_new();
-EXT void bgp_info_add(struct bgp_node *, struct bgp_info *, u_int32_t);
-EXT void bgp_info_delete(struct bgp_node *, struct bgp_info *, u_int32_t);
-EXT void bgp_info_free(struct bgp_info *);
-EXT void bgp_attr_init();
-EXT struct bgp_attr *bgp_attr_intern(struct bgp_attr *);
-EXT void bgp_attr_unintern (struct bgp_attr *);
-EXT void *bgp_attr_hash_alloc (void *);
-EXT int bgp_peer_init(struct bgp_peer *);
-EXT void bgp_peer_close(struct bgp_peer *, int);
-EXT void bgp_peer_info_delete(struct bgp_peer *);
-EXT int bgp_attr_munge_as4path(struct bgp_peer *, struct bgp_attr *, struct aspath *);
-EXT void load_comm_patterns(char **, char **, char **);
-EXT void load_peer_src_as_comm_ranges(char *, char *);
-EXT void evaluate_comm_patterns(char *, char *, char **, int);
-EXT as_t evaluate_last_asn(struct aspath *);
-EXT as_t evaluate_first_asn(char *);
-EXT void bgp_srcdst_lookup(struct packet_ptrs *);
-EXT void bgp_follow_nexthop_lookup(struct packet_ptrs *);
-EXT void write_neighbors_file(char *);
-EXT void process_bgp_md5_file(int, struct bgp_md5_table *);
-EXT u_int32_t bgp_route_info_modulo_pathid(struct bgp_peer *, path_id_t *);
-
-EXT unsigned int attrhash_key_make(void *);
-EXT int attrhash_cmp(const void *, const void *);
-EXT void attrhash_init(struct hash **);
-
-EXT void cache_to_pkt_bgp_primitives(struct pkt_bgp_primitives *, struct cache_bgp_primitives *);
-EXT void pkt_to_cache_bgp_primitives(struct cache_bgp_primitives *, struct pkt_bgp_primitives *, pm_cfgreg_t);
-EXT void free_cache_bgp_primitives(struct cache_bgp_primitives **);
-EXT void bgp_config_checks(struct configuration *);
+EXT void bgp_prepare_thread();
+EXT void bgp_prepare_daemon();
+#undef EXT
 
 /* global variables */
+#if (!defined __BGP_C)
+#define EXT extern
+#else
+#define EXT
+#endif
 EXT struct bgp_peer *peers;
-EXT struct hash *attrhash;
-EXT struct hash *ashash;
-EXT struct hash *comhash;
-EXT struct hash *ecomhash;
 EXT char *std_comm_patterns[MAX_BGP_COMM_PATTERNS];
 EXT char *ext_comm_patterns[MAX_BGP_COMM_PATTERNS];
 EXT char *std_comm_patterns_to_asn[MAX_BGP_COMM_PATTERNS];
 EXT struct bgp_comm_range peer_src_as_ifrange; 
 EXT struct bgp_comm_range peer_src_as_asrange; 
-EXT struct bgp_table *rib[AFI_MAX][SAFI_MAX];
 EXT u_int32_t (*bgp_route_info_modulo)(struct bgp_peer *, path_id_t *);
 
+EXT struct bgp_rt_structs inter_domain_routing_dbs[FUNC_TYPE_MAX], *bgp_routing_db;
+EXT struct bgp_misc_structs inter_domain_misc_dbs[FUNC_TYPE_MAX], *bgp_misc_db;
 #undef EXT
 #endif 
