@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /* 
@@ -28,20 +28,26 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 
 #include "pmacct.h"
 #include "bgp.h"
-#include "bgp_community.h"
 
 /* Hash of community attribute. */
 // struct hash *comhash;
 
 /* Allocate a new communities value.  */
 static struct community *
-community_new (void)
+community_new (struct bgp_peer *peer)
 {
+  struct bgp_misc_structs *bms;
   void *tmp;
 
+  if (!peer) return NULL;
+
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return NULL;
+
   tmp = malloc(sizeof (struct community));
   if (!tmp) {
-    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (community_new). Exiting ..\n", config.name);
+    Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (community_new). Exiting ..\n", config.name, bms->log_str);
     exit_all(1);
   }
   memset(tmp, 0, sizeof (struct community));
@@ -60,15 +66,23 @@ community_free (struct community *com)
 
 /* Add one community value to the community. */
 static void
-community_add_val (struct community *com, u_int32_t val)
+community_add_val (struct bgp_peer *peer, struct community *com, u_int32_t val)
 {
+  struct bgp_misc_structs *bms;
+
+  if (!peer) return;
+
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return;
+
   com->size++;
   if (com->val)
     com->val = realloc(com->val, com_length (com));
   else {
     com->val = malloc(com_length (com));
     if (!com->val) {
-      Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (community_add_val). Exiting ..\n", config.name);
+      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (community_add_val). Exiting ..\n", config.name, bms->log_str);
       exit_all(1);
     }
   }
@@ -175,7 +189,7 @@ community_val_get (struct community *com, int i)
 
 /* Sort and uniq given community. */
 struct community *
-community_uniq_sort (struct community *com)
+community_uniq_sort (struct bgp_peer *peer, struct community *com)
 {
   int i;
   struct community *new;
@@ -184,14 +198,14 @@ community_uniq_sort (struct community *com)
   if (! com)
     return NULL;
   
-  new = community_new ();;
+  new = community_new (peer);
   
   for (i = 0; i < com->size; i++)
     {
       val = community_val_get (com, i);
 
       if (! community_include (new, val))
-	community_add_val (new, val);
+	community_add_val (peer, new, val);
     }
 
   qsort (new->val, new->size, sizeof (u_int32_t), community_compare);
@@ -210,8 +224,9 @@ community_uniq_sort (struct community *com)
 
    For other values, "AS:VAL" format is used.  */
 static char *
-community_com2str  (struct community *com)
+community_com2str  (struct bgp_peer *peer, struct community *com)
 {
+  struct bgp_misc_structs *bms;
   int i;
   char *str;
   char *pnt;
@@ -221,15 +236,18 @@ community_com2str  (struct community *com)
   u_int16_t as;
   u_int16_t val;
 
-  if (!com)
-    return NULL;
+  if (!com || !peer) return NULL;
+
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return NULL;
   
   /* When communities attribute is empty.  */
   if (com->size == 0)
     {
       str = malloc(1);
       if (!str) {
-	Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (community_com2str). Exiting ..\n", config.name);
+	Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (community_com2str). Exiting ..\n", config.name, bms->log_str);
 	exit_all(1);
       }
       str[0] = '\0';
@@ -268,7 +286,7 @@ community_com2str  (struct community *com)
   /* Allocate memory.  */
   str = pnt = malloc(len);
   if (!str) {
-    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (community_com2str). Exiting ..\n", config.name);
+    Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (community_com2str). Exiting ..\n", config.name, bms->log_str);
     exit_all(1);
   }
   first = 1;
@@ -317,15 +335,22 @@ community_com2str  (struct community *com)
 
 /* Intern communities attribute.  */
 struct community *
-community_intern (struct community *com)
+community_intern (struct bgp_peer *peer, struct community *com)
 {
+  struct bgp_rt_structs *inter_domain_routing_db;
   struct community *find;
 
+  if (!peer) return NULL;
+
+  inter_domain_routing_db = bgp_select_routing_db(peer->type);
+
+  if (!inter_domain_routing_db) return NULL;
+
   /* Assert this community structure is not interned. */
   assert (com->refcnt == 0);
 
   /* Lookup community hash. */
-  find = (struct community *) hash_get (comhash, com, hash_alloc_intern);
+  find = (struct community *) hash_get(peer, inter_domain_routing_db->comhash, com, hash_alloc_intern);
 
   /* Arguemnt com is allocated temporary.  So when it is not used in
      hash, it should be freed.  */
@@ -337,34 +362,40 @@ community_intern (struct community *com)
 
   /* Make string.  */
   if (! find->str)
-    find->str = community_com2str (find);
+    find->str = community_com2str (peer, find);
 
   return find;
 }
 
 /* Free community attribute. */
 void
-community_unintern (struct community *com)
+community_unintern (struct bgp_peer *peer, struct community *com)
 {
+  struct bgp_rt_structs *inter_domain_routing_db;
   struct community *ret;
 
+  if (!peer) return;
+  
+  inter_domain_routing_db = bgp_select_routing_db(peer->type);
+
+  if (!inter_domain_routing_db) return;
+
   if (com->refcnt)
     com->refcnt--;
 
   /* Pull off from hash.  */
-  if (com->refcnt == 0)
-    {
-      /* Community value com must exist in hash. */
-      ret = (struct community *) hash_release (comhash, com);
-      assert (ret != NULL);
+  if (com->refcnt == 0) {
+    /* Community value com must exist in hash. */
+    ret = (struct community *) hash_release(inter_domain_routing_db->comhash, com);
+    assert (ret != NULL);
 
-      community_free (com);
-    }
+    community_free (com);
+  }
 }
 
 /* Create new community attribute. */
 struct community *
-community_parse (u_int32_t *pnt, u_short length)
+community_parse (struct bgp_peer *peer, u_int32_t *pnt, u_short length)
 {
   struct community tmp;
   struct community *new;
@@ -377,46 +408,20 @@ community_parse (u_int32_t *pnt, u_short length)
   tmp.size = length / 4;
   tmp.val = pnt;
 
-  new = community_uniq_sort (&tmp);
+  new = community_uniq_sort (peer, &tmp);
 
-  return community_intern (new);
-}
-
-struct community *
-community_dup (struct community *com)
-{
-  struct community *new;
-
-  new = malloc(sizeof (struct community));
-  if (!new) {
-    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (community_dup). Exiting ..\n", config.name);
-    exit_all(1);
-  }
-
-  new->size = com->size;
-  if (new->size)
-    {
-      new->val = malloc(com->size * 4);
-      if (!new->val) {
-        Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (community_dup). Exiting ..\n", config.name);
-        exit_all(1);
-      }
-      memcpy (new->val, com->val, com->size * 4);
-    }
-  else
-    new->val = NULL;
-  return new;
+  return community_intern (peer, new);
 }
 
 /* Retrun string representation of communities attribute. */
 char *
-community_str (struct community *com)
+community_str (struct bgp_peer *peer, struct community *com)
 {
   if (!com)
     return NULL;
   
   if (! com->str)
-    com->str = community_com2str (com);
+    com->str = community_com2str (peer, com);
   return com->str;
 }
 
@@ -483,26 +488,6 @@ community_cmp (const struct community *com1, const struct community *com2)
   return 0;
 }
 
-/* Add com2 to the end of com1. */
-struct community *
-community_merge (struct community *com1, struct community *com2)
-{
-  if (com1->val)
-    com1->val = realloc(com1->val, (com1->size + com2->size) * 4);
-  else {
-    com1->val = malloc((com1->size + com2->size) * 4);
-    if (!com1->val) {
-      Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (community_merge). Exiting ..\n", config.name);
-      exit_all(1);
-    }
-  }
-
-  memcpy (com1->val + com1->size, com2->val, com2->size * 4);
-  com1->size += com2->size;
-
-  return com1;
-}
-
 /* Community token enum. */
 enum community_token
 {
@@ -611,64 +596,10 @@ community_gettoken (const char *buf, enum community_token *token,
   return NULL;
 }
 
-/* convert string to community structure */
-struct community *
-community_str2com (const char *str)
-{
-  struct community *com = NULL;
-  struct community *com_sort = NULL;
-  u_int32_t val = 0;
-  enum community_token token = community_token_unknown;
-
-  do 
-    {
-      str = community_gettoken (str, &token, &val);
-      
-      switch (token)
-	{
-	case community_token_val:
-	case community_token_no_export:
-	case community_token_no_advertise:
-	case community_token_local_as:
-	  if (com == NULL)
-	    com = community_new();
-	  community_add_val (com, val);
-	  break;
-	case community_token_unknown:
-	default:
-	  if (com)
-	    community_free (com);
-	  return NULL;
-	}
-    } while (str);
-  
-  if (! com)
-    return NULL;
-
-  com_sort = community_uniq_sort (com);
-  community_free (com);
-
-  return com_sort;
-}
-
-/* Return communities hash entry count.  */
-unsigned long
-community_count ()
-{
-  return comhash->count;
-}
-
-/* Return communities hash.  */
-struct hash *
-community_hash (void)
-{
-  return comhash;
-}
-
 /* Initialize comminity related hash. */
 void
-community_init (struct hash **loc_comhash)
+community_init (int buckets, struct hash **loc_comhash)
 {
-  (*loc_comhash) = hash_create ((unsigned int (*) (void *))community_hash_make,
+  (*loc_comhash) = hash_create (buckets, (unsigned int (*) (void *))community_hash_make,
 			 (int (*) (const void *, const void *))community_cmp);
 }
