@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /* 
@@ -30,10 +30,14 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "pmacct.h"
 #include "bgp.h"
 
-static void bgp_node_delete (struct bgp_node *);
-static void bgp_node_free_aggressive (struct bgp_node *, safi_t);
-static void bgp_table_free (struct bgp_table *);
-
+static void bgp_node_delete (struct bgp_peer *, struct bgp_node *);
+static struct bgp_node *bgp_node_create (struct bgp_peer *);
+static struct bgp_node *bgp_node_set (struct bgp_peer *, struct bgp_table *, struct prefix *);
+static void bgp_node_free (struct bgp_node *);
+static void route_common (struct prefix *, struct prefix *, struct prefix *);
+static int check_bit (u_char *, u_char);
+static void set_link (struct bgp_node *, struct bgp_node *);
+
 struct bgp_table *
 bgp_table_init (afi_t afi, safi_t safi)
 {
@@ -43,12 +47,11 @@ bgp_table_init (afi_t afi, safi_t safi)
   if (rt) {
     memset (rt, 0, sizeof (struct bgp_table));
 
-    rt->type = BGP_TABLE_MAIN;
     rt->afi = afi;
     rt->safi = safi;
   }
   else {
-    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (bgp_table_init). Exiting ..\n", config.name);
+    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (bgp_table_init). Exiting ..\n", config.name); // XXX
     exit_all(1);
   }
   
@@ -56,16 +59,23 @@ bgp_table_init (afi_t afi, safi_t safi)
 }
 
 static struct bgp_node *
-bgp_node_create ()
+bgp_node_create (struct bgp_peer *peer)
 {
+  struct bgp_misc_structs *bms;
   struct bgp_node *rn;
 
+  if (!peer) return NULL;
+
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return NULL;
+
   rn = (struct bgp_node *) malloc (sizeof (struct bgp_node));
   if (rn) {
     memset (rn, 0, sizeof (struct bgp_node));
 
-    rn->info = (void **) malloc(sizeof(struct bgp_info *) * (config.bgp_table_peer_buckets * config.bgp_table_per_peer_buckets));
-    if (rn->info) memset (rn->info, 0, sizeof(struct bgp_info *) * (config.bgp_table_peer_buckets * config.bgp_table_per_peer_buckets));
+    rn->info = (void **) malloc(sizeof(struct bgp_info *) * (bms->table_peer_buckets * bms->table_per_peer_buckets));
+    if (rn->info) memset (rn->info, 0, sizeof(struct bgp_info *) * (bms->table_peer_buckets * bms->table_per_peer_buckets));
     else goto malloc_failed;
   }
   else goto malloc_failed;
@@ -73,17 +83,18 @@ bgp_node_create ()
   return rn;
 
   malloc_failed:
-  Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (bgp_node_create). Exiting ..\n", config.name);
+  Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (bgp_node_create). Exiting ..\n", config.name, bms->log_str);
   exit_all(1);
 }
 
 /* Allocate new route node with prefix set. */
 static struct bgp_node *
-bgp_node_set (struct bgp_table *table, struct prefix *prefix)
+bgp_node_set (struct bgp_peer *peer, struct bgp_table *table, struct prefix *prefix)
 {
+  struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
   struct bgp_node *node;
   
-  node = bgp_node_create ();
+  node = bgp_node_create (peer);
 
   prefix_copy (&node->p, prefix);
   node->table = table;
@@ -99,84 +110,6 @@ bgp_node_free (struct bgp_node *node)
   free (node);
 }
 
-/* Free route node aggressively: also attributes and info;
-   should be meant to be invoked only by bgp_table_free() */
-static void
-bgp_node_free_aggressive (struct bgp_node *node, safi_t safi)
-{
-  struct bgp_info *ri, *next;
-  u_int32_t ri_idx;
-
-  /* XXX: this should be moved further outside */
-  if (config.nfacctd_bgp_msglog_file || config.nfacctd_bgp_msglog_amqp_routing_key)
-    gettimeofday(&log_tstamp, NULL);
-
-  for (ri_idx = 0; ri_idx < (config.bgp_table_peer_buckets * config.bgp_table_per_peer_buckets); ri_idx++) {
-    for (ri = node->info[ri_idx]; ri; ri = next) {
-      if (config.nfacctd_bgp_msglog_file || config.nfacctd_bgp_msglog_amqp_routing_key) {
-        char event_type[] = "delete";
-
-        bgp_peer_log_msg(node, ri, safi, event_type, config.nfacctd_bgp_msglog_output);
-      }
-
-      next = ri->next;
-      bgp_info_free(ri);
-    }
-  }
-
-  free (node->info);
-  free (node);
-}
-
-/* Free route table. */
-static void
-bgp_table_free (struct bgp_table *rt)
-{
-  struct bgp_node *tmp_node;
-  struct bgp_node *node;
- 
-  if (rt == NULL)
-    return;
-
-  node = rt->top;
-
-  while (node)
-    {
-      if (node->l_left)
-	{
-	  node = node->l_left;
-	  continue;
-	}
-
-      if (node->l_right)
-	{
-	  node = node->l_right;
-	  continue;
-	}
-
-      tmp_node = node;
-      node = node->parent;
-
-      if (node != NULL)
-	{
-	  if (node->l_left == tmp_node)
-	    node->l_left = NULL;
-	  else
-	    node->l_right = NULL;
-
-	  bgp_node_free_aggressive (tmp_node, rt->safi);
-	}
-      else
-	{
-	  bgp_node_free_aggressive (tmp_node, rt->safi);
-	  break;
-	}
-    }
- 
-  free (rt);
-  return;
-}
-
 /* Utility mask array. */
 static u_char maskbit[] = 
 {
@@ -256,7 +189,7 @@ set_link (struct bgp_node *node, struct bgp_node *new)
 
 /* Lock node. */
 struct bgp_node *
-bgp_lock_node (struct bgp_node *node)
+bgp_lock_node (struct bgp_peer *peer, struct bgp_node *node)
 {
   node->lock++;
   return node;
@@ -264,47 +197,72 @@ bgp_lock_node (struct bgp_node *node)
 
 /* Unlock node. */
 void
-bgp_unlock_node (struct bgp_node *node)
+bgp_unlock_node (struct bgp_peer *peer, struct bgp_node *node)
 {
   node->lock--;
 
   if (node->lock == 0)
-    bgp_node_delete (node);
+    bgp_node_delete (peer, node);
 }
 
 /* Find matched prefix. */
-struct bgp_node *
-bgp_node_match (const struct bgp_table *table, struct prefix *p, struct bgp_peer *peer)
+void
+bgp_node_match (const struct bgp_table *table, struct prefix *p, struct bgp_peer *peer,
+		u_int32_t (*modulo_func)(struct bgp_peer *, path_id_t *),
+		int (*cmp_func)(struct bgp_info *, struct node_match_cmp_term2 *),
+		struct node_match_cmp_term2 *nmct2,
+		struct bgp_node **result_node, struct bgp_info **result_info)
 {
-  struct bgp_node *node;
-  struct bgp_node *matched;
-  struct bgp_info *info;
-  u_int32_t modulo = bgp_route_info_modulo(peer, NULL);
+  struct bgp_misc_structs *bms;
+  struct bgp_node *node, *matched_node;
+  struct bgp_info *info, *matched_info;
+  u_int32_t modulo, modulo_idx, local_modulo, modulo_max;
 
-  matched = NULL;
+  if (!table || !peer || !modulo_func || !cmp_func) return;
+
+  bms = bgp_select_misc_db(peer->type);
+  if (!bms) return;
+
+  modulo = modulo_func(peer, NULL);
+  if (bms->table_per_peer_hash == BGP_ASPATH_HASH_PATHID) modulo_max = bms->table_per_peer_buckets;
+  else modulo_max = 1;
+
+  matched_node = NULL;
+  matched_info = NULL;
   node = table->top;
 
-  /* Walk down tree.  If there is matched route then store it to
-     matched. */
+  /* Walk down tree.  If there is matched route then store it to matched. */
   while (node && node->p.prefixlen <= p->prefixlen && prefix_match(&node->p, p)) {
-    for (info = node->info[modulo]; info; info = info->next) {
-      if (info->peer == peer) {
-	matched = node;
-        break;
+    for (local_modulo = modulo, modulo_idx = 0; modulo_idx < modulo_max; local_modulo++, modulo_idx++) {
+      for (info = node->info[local_modulo]; info; info = info->next) {
+	if (!cmp_func(info, nmct2)) {
+	  matched_node = node;
+	  matched_info = info;
+	  break;
+	}
       }
     }
 
     node = node->link[check_bit(&p->u.prefix, node->p.prefixlen)];
   }
 
-  /* If matched route found, return it. */
-  if (matched) return bgp_lock_node (matched);
-
-  return NULL;
+  if (matched_node) {
+    (*result_node) = matched_node;
+    (*result_info) = matched_info;
+    bgp_lock_node (NULL /* XXX */, matched_node);
+  }
+  else {
+    (*result_node) = NULL;
+    (*result_info) = NULL;
+  }
 }
 
-struct bgp_node *
-bgp_node_match_ipv4 (const struct bgp_table *table, struct in_addr *addr, struct bgp_peer *peer)
+void
+bgp_node_match_ipv4 (const struct bgp_table *table, struct in_addr *addr, struct bgp_peer *peer,
+		     u_int32_t (*modulo_func)(struct bgp_peer *, path_id_t *),
+		     int (*cmp_func)(struct bgp_info *, struct node_match_cmp_term2 *),
+		     struct node_match_cmp_term2 *nmct2,
+		     struct bgp_node **result_node, struct bgp_info **result_info)
 {
   struct prefix_ipv4 p;
 
@@ -313,12 +271,16 @@ bgp_node_match_ipv4 (const struct bgp_table *table, struct in_addr *addr, struct
   p.prefixlen = IPV4_MAX_PREFIXLEN;
   p.prefix = *addr;
 
-  return bgp_node_match (table, (struct prefix *) &p, peer);
+  bgp_node_match (table, (struct prefix *) &p, peer, modulo_func, cmp_func, nmct2, result_node, result_info);
 }
 
 #ifdef ENABLE_IPV6
-struct bgp_node *
-bgp_node_match_ipv6 (const struct bgp_table *table, struct in6_addr *addr, struct bgp_peer *peer)
+void
+bgp_node_match_ipv6 (const struct bgp_table *table, struct in6_addr *addr, struct bgp_peer *peer,
+		     u_int32_t (*modulo_func)(struct bgp_peer *, path_id_t *),
+		     int (*cmp_func)(struct bgp_info *, struct node_match_cmp_term2 *),
+		     struct node_match_cmp_term2 *nmct2,
+		     struct bgp_node **result_node, struct bgp_info **result_info)
 {
   struct prefix_ipv6 p;
 
@@ -327,14 +289,15 @@ bgp_node_match_ipv6 (const struct bgp_table *table, struct in6_addr *addr, struc
   p.prefixlen = IPV6_MAX_PREFIXLEN;
   p.prefix = *addr;
 
-  return bgp_node_match (table, (struct prefix *) &p, peer);
+  bgp_node_match (table, (struct prefix *) &p, peer, modulo_func, cmp_func, nmct2, result_node, result_info);
 }
 #endif /* ENABLE_IPV6 */
 
 /* Add node to routing table. */
 struct bgp_node *
-bgp_node_get (struct bgp_table *const table, struct prefix *p)
+bgp_node_get (struct bgp_peer *peer, struct bgp_table *const table, struct prefix *p)
 {
+  struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
   struct bgp_node *new;
   struct bgp_node *node;
   struct bgp_node *match;
@@ -346,7 +309,7 @@ bgp_node_get (struct bgp_table *const table, struct prefix *p)
     {
       if (node->p.prefixlen == p->prefixlen)
 	{
-	  bgp_lock_node (node);
+	  bgp_lock_node (peer, node);
 	  return node;
 	}
       match = node;
@@ -355,7 +318,7 @@ bgp_node_get (struct bgp_table *const table, struct prefix *p)
 
   if (node == NULL)
     {
-      new = bgp_node_set (table, p);
+      new = bgp_node_set (peer, table, p);
       if (match)
 	set_link (match, new);
       else
@@ -363,7 +326,7 @@ bgp_node_get (struct bgp_table *const table, struct prefix *p)
     }
   else
     {
-      new = bgp_node_create ();
+      new = bgp_node_create (peer);
       route_common (&node->p, p, &new->p);
       new->p.family = p->family;
       new->table = table;
@@ -377,27 +340,29 @@ bgp_node_get (struct bgp_table *const table, struct prefix *p)
       if (new->p.prefixlen != p->prefixlen)
 	{
 	  match = new;
-	  new = bgp_node_set (table, p);
+	  new = bgp_node_set (peer, table, p);
 	  set_link (match, new);
 	  table->count++;
 	}
     }
   table->count++;
-  bgp_lock_node (new);
+  bgp_lock_node (peer, new);
   
   return new;
 }
 
 /* Delete node from the routing table. */
 static void
-bgp_node_delete (struct bgp_node *node)
+bgp_node_delete (struct bgp_peer *peer, struct bgp_node *node)
 {
+  struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
   struct bgp_node *child;
   struct bgp_node *parent;
   u_int32_t ri_idx;
 
   assert (node->lock == 0);
-  for (ri_idx = 0; ri_idx < (config.bgp_table_peer_buckets * config.bgp_table_per_peer_buckets); ri_idx++)
+
+  for (ri_idx = 0; ri_idx < (bms->table_peer_buckets * bms->table_per_peer_buckets); ri_idx++)
     assert (node->info[ri_idx] == NULL);
 
   if (node->l_left && node->l_right)
@@ -429,26 +394,30 @@ bgp_node_delete (struct bgp_node *node)
 
   /* If parent node is stub then delete it also. */
   if (parent && parent->lock == 0)
-    bgp_node_delete (parent);
+    bgp_node_delete (peer, parent);
 }
 
 /* Get fist node and lock it.  This function is useful when one want
    to lookup all the node exist in the routing table. */
 struct bgp_node *
-bgp_table_top (const struct bgp_table *const table)
+bgp_table_top (struct bgp_peer *peer, const struct bgp_table *const table)
 {
-  /* If there is no node in the routing table return NULL. */
-  if (table->top == NULL)
-    return NULL;
+  if (table) {
+    /* If there is no node in the routing table return NULL. */
+    if (table->top == NULL)
+      return NULL;
 
-  /* Lock the top node and return it. */
-  bgp_lock_node (table->top);
-  return table->top;
+    /* Lock the top node and return it. */
+    bgp_lock_node (peer, table->top);
+    return table->top;
+  }
+  
+  return NULL;
 }
 
 /* Unlock current node and lock next node then return it. */
 struct bgp_node *
-bgp_route_next (struct bgp_node *node)
+bgp_route_next (struct bgp_peer *peer, struct bgp_node *node)
 {
   struct bgp_node *next;
   struct bgp_node *start;
@@ -459,15 +428,15 @@ bgp_route_next (struct bgp_node *node)
   if (node->l_left)
     {
       next = node->l_left;
-      bgp_lock_node (next);
-      bgp_unlock_node (node);
+      bgp_lock_node (peer, next);
+      bgp_unlock_node (peer, node);
       return next;
     }
   if (node->l_right)
     {
       next = node->l_right;
-      bgp_lock_node (next);
-      bgp_unlock_node (node);
+      bgp_lock_node (peer, next);
+      bgp_unlock_node (peer, node);
       return next;
     }
 
@@ -477,19 +446,19 @@ bgp_route_next (struct bgp_node *node)
       if (node->parent->l_left == node && node->parent->l_right)
 	{
 	  next = node->parent->l_right;
-	  bgp_lock_node (next);
-	  bgp_unlock_node (start);
+	  bgp_lock_node (peer, next);
+	  bgp_unlock_node (peer, start);
 	  return next;
 	}
       node = node->parent;
     }
-  bgp_unlock_node (start);
+  bgp_unlock_node (peer, start);
   return NULL;
 }
 
 /* Unlock current node and lock next node until limit. */
 struct bgp_node *
-bgp_route_next_until (struct bgp_node *node, struct bgp_node *limit)
+bgp_route_next_until (struct bgp_peer *peer, struct bgp_node *node, struct bgp_node *limit)
 {
   struct bgp_node *next;
   struct bgp_node *start;
@@ -500,15 +469,15 @@ bgp_route_next_until (struct bgp_node *node, struct bgp_node *limit)
   if (node->l_left)
     {
       next = node->l_left;
-      bgp_lock_node (next);
-      bgp_unlock_node (node);
+      bgp_lock_node (peer, next);
+      bgp_unlock_node (peer, node);
       return next;
     }
   if (node->l_right)
     {
       next = node->l_right;
-      bgp_lock_node (next);
-      bgp_unlock_node (node);
+      bgp_lock_node (peer, next);
+      bgp_unlock_node (peer, node);
       return next;
     }
 
@@ -518,18 +487,12 @@ bgp_route_next_until (struct bgp_node *node, struct bgp_node *limit)
       if (node->parent->l_left == node && node->parent->l_right)
 	{
 	  next = node->parent->l_right;
-	  bgp_lock_node (next);
-	  bgp_unlock_node (start);
+	  bgp_lock_node (peer, next);
+	  bgp_unlock_node (peer, start);
 	  return next;
 	}
       node = node->parent;
     }
-  bgp_unlock_node (start);
+  bgp_unlock_node (peer, start);
   return NULL;
 }
-
-unsigned long
-bgp_table_count (const struct bgp_table *table)
-{
-  return table->count;
-}
