@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2014 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /* 
@@ -33,16 +33,8 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 typedef u_int16_t afi_t;
 typedef u_int8_t safi_t;
 
-typedef enum
-{
-  BGP_TABLE_MAIN,
-  BGP_TABLE_RSCLIENT,
-} bgp_table_t;
-
 struct bgp_table
 {
-  bgp_table_t type;
-  
   /* afi/safi of this table */
   afi_t afi;
   safi_t safi;
@@ -98,6 +90,13 @@ struct bgp_info
   struct bgp_info_extra *extra;
 };
 
+struct node_match_cmp_term2 {
+  struct bgp_peer *peer;
+  safi_t safi;
+  rd_t *rd;
+  struct host_addr *peer_dst_ip;
+};
+
 /* Prototypes */
 #if (!defined __BGP_TABLE_C)
 #define EXT extern
@@ -105,19 +104,28 @@ struct bgp_info
 #define EXT
 #endif
 EXT struct bgp_table *bgp_table_init (afi_t, safi_t);
-EXT void bgp_table_finish (struct bgp_table **);
-EXT void bgp_unlock_node (struct bgp_node *node);
-EXT struct bgp_node *bgp_table_top (const struct bgp_table *const);
-EXT struct bgp_node *bgp_route_next (struct bgp_node *);
-EXT struct bgp_node *bgp_route_next_until (struct bgp_node *, struct bgp_node *);
-EXT struct bgp_node *bgp_node_get (struct bgp_table *const, struct prefix *);
-EXT struct bgp_node *bgp_lock_node (struct bgp_node *node);
-EXT struct bgp_node *bgp_node_match (const struct bgp_table *, struct prefix *, struct bgp_peer *);
-EXT struct bgp_node *bgp_node_match_ipv4 (const struct bgp_table *, struct in_addr *, struct bgp_peer *);
+EXT void bgp_unlock_node (struct bgp_peer *, struct bgp_node *node);
+EXT struct bgp_node *bgp_table_top (struct bgp_peer *, const struct bgp_table *const);
+EXT struct bgp_node *bgp_route_next (struct bgp_peer *, struct bgp_node *);
+EXT struct bgp_node *bgp_route_next_until (struct bgp_peer *, struct bgp_node *, struct bgp_node *);
+EXT struct bgp_node *bgp_node_get (struct bgp_peer *, struct bgp_table *const, struct prefix *);
+EXT struct bgp_node *bgp_lock_node (struct bgp_peer *, struct bgp_node *node);
+EXT void bgp_node_match (const struct bgp_table *, struct prefix *, struct bgp_peer *,
+			 u_int32_t (*modulo_func)(struct bgp_peer *, path_id_t *),
+			 int (*cmp_func)(struct bgp_info *, struct node_match_cmp_term2 *),
+			 struct node_match_cmp_term2 *,
+			 struct bgp_node **result_node, struct bgp_info **result_info);
+EXT void bgp_node_match_ipv4 (const struct bgp_table *, struct in_addr *, struct bgp_peer *,
+			      u_int32_t (*modulo_func)(struct bgp_peer *, path_id_t *),
+			      int (*cmp_func)(struct bgp_info *, struct node_match_cmp_term2 *),
+			      struct node_match_cmp_term2 *,
+			      struct bgp_node **result_node, struct bgp_info **result_info);
 #ifdef ENABLE_IPV6
-EXT struct bgp_node *bgp_node_match_ipv6 (const struct bgp_table *, struct in6_addr *, struct bgp_peer *);
+EXT void bgp_node_match_ipv6 (const struct bgp_table *, struct in6_addr *, struct bgp_peer *,
+			      u_int32_t (*modulo_func)(struct bgp_peer *, path_id_t *),
+			      int (*cmp_func)(struct bgp_info *, struct node_match_cmp_term2 *),
+			      struct node_match_cmp_term2 *,
+			      struct bgp_node **result_node, struct bgp_info **result_info);
 #endif /* ENABLE_IPV6 */
-EXT unsigned long bgp_table_count (const struct bgp_table *const);
-
 #undef EXT
 #endif 
