@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -27,7 +27,10 @@
 #endif
 
 EXT int parse_truefalse(char *);
+EXT int parse_truefalse_nonzero(char *);
+EXT int validate_truefalse(int);
 EXT int cfg_key_debug(char *, char *, char *);
+EXT int cfg_key_debug_internal_msg(char *, char *, char *);
 EXT int cfg_key_syslog(char *, char *, char *);
 EXT int cfg_key_logfile(char *, char *, char *);
 EXT int cfg_key_pidfile(char *, char *, char *);
@@ -70,8 +73,6 @@ EXT int cfg_key_sql_optimize_clauses(char *, char *, char *);
 EXT int cfg_key_sql_history(char *, char *, char *);
 EXT int cfg_key_sql_history_offset(char *, char *, char *);
 EXT int cfg_key_sql_history_roundoff(char *, char *, char *);
-EXT int cfg_key_sql_history_since_epoch(char *, char *, char *);
-EXT int cfg_key_sql_recovery_logfile(char *, char *, char *);
 EXT int cfg_key_sql_recovery_backup_host(char *, char *, char *);
 EXT int cfg_key_sql_max_writers(char *, char *, char *);
 EXT int cfg_key_sql_trigger_exec(char *, char *, char *);
@@ -86,15 +87,23 @@ EXT int cfg_key_sql_locking_style(char *, char *, char *);
 EXT int cfg_key_sql_use_copy(char *, char *, char *);
 EXT int cfg_key_sql_delimiter(char *, char *, char *);
 EXT int cfg_key_timestamps_secs(char *, char *, char *);
+EXT int cfg_key_timestamps_since_epoch(char *, char *, char *);
 EXT int cfg_key_mongo_insert_batch(char *, char *, char *);
+EXT int cfg_key_message_broker_output(char *, char *, char *);
+EXT int cfg_key_avro_buffer_size(char *, char *, char *);
+EXT int cfg_key_avro_schema_output_file(char *, char *, char *);
 EXT int cfg_key_amqp_exchange_type(char *, char *, char *);
 EXT int cfg_key_amqp_persistent_msg(char *, char *, char *);
 EXT int cfg_key_amqp_frame_max(char *, char *, char *);
 EXT int cfg_key_amqp_heartbeat_interval(char *, char *, char *);
 EXT int cfg_key_amqp_vhost(char *, char *, char *);
 EXT int cfg_key_amqp_routing_key_rr(char *, char *, char *);
+EXT int cfg_key_kafka_broker_port(char *, char *, char *);
+EXT int cfg_key_kafka_partition(char *, char *, char *);
+EXT int cfg_key_kafka_partition_key(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_size(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_backlog(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_check_core_pid(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_amqp(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_amqp_user(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_amqp_passwd(char *, char *, char *);
@@ -103,6 +112,12 @@ EXT int cfg_key_plugin_pipe_amqp_host(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_amqp_vhost(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_amqp_routing_key(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_amqp_retry(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_kafka(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_kafka_broker_host(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_kafka_broker_port(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_kafka_topic(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_kafka_partition(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_kafka_retry(char *, char *, char *);
 EXT int cfg_key_plugin_buffer_size(char *, char *, char *);
 EXT int cfg_key_networks_mask(char *, char *, char *);
 EXT int cfg_key_networks_file(char *, char *, char *);
@@ -115,6 +130,7 @@ EXT int cfg_key_print_markers(char *, char *, char *);
 EXT int cfg_key_print_output(char *, char *, char *);
 EXT int cfg_key_print_output_file(char *, char *, char *);
 EXT int cfg_key_print_output_file_append(char *, char *, char *);
+EXT int cfg_key_print_output_lock_file(char *, char *, char *);
 EXT int cfg_key_print_output_separator(char *, char *, char *);
 EXT int cfg_key_print_latest_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_port(char *, char *, char *);
@@ -138,9 +154,79 @@ EXT int cfg_key_pmacctd_conntrack_buffer_size(char *, char *, char *);
 EXT int cfg_key_pmacctd_flow_lifetime(char *, char *, char *);
 EXT int cfg_key_pmacctd_flow_tcp_lifetime(char *, char *, char *);
 EXT int cfg_key_pmacctd_ext_sampling_rate(char *, char *, char *);
+EXT int cfg_key_pmacctd_nonroot(char *, char *, char *);
 EXT int cfg_key_sfacctd_renormalize(char *, char *, char *);
 EXT int cfg_key_sfacctd_counter_output(char *, char *, char *);
 EXT int cfg_key_sfacctd_counter_file(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_host(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_vhost(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_user(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_passwd(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_exchange(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_exchange_type(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_routing_key(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_persistent_msg(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_frame_max(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_heartbeat_interval(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_amqp_retry(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_kafka_broker_host(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_kafka_broker_port(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_kafka_topic(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_kafka_partition(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_kafka_partition_key(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_kafka_retry(char *, char *, char *);
+EXT int cfg_key_telemetry_daemon(char *, char *, char *);
+EXT int cfg_key_telemetry_port_tcp(char *, char *, char *);
+EXT int cfg_key_telemetry_port_udp(char *, char *, char *);
+EXT int cfg_key_telemetry_ip(char *, char *, char *);
+EXT int cfg_key_telemetry_decoder(char *, char *, char *);
+EXT int cfg_key_telemetry_max_peers(char *, char *, char *);
+EXT int cfg_key_telemetry_udp_timeout(char *, char *, char *);
+EXT int cfg_key_telemetry_allow_file(char *, char *, char *);
+EXT int cfg_key_telemetry_pipe_size(char *, char *, char *);
+EXT int cfg_key_telemetry_ip_precedence(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_output(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_file(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_host(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_vhost(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_user(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_passwd(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_exchange(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_exchange_type(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_routing_key(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_routing_key_rr(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_persistent_msg(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_frame_max(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_heartbeat_interval(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_amqp_retry(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_kafka_broker_host(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_kafka_broker_port(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_kafka_topic(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_kafka_topic_rr(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_kafka_partition(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_kafka_partition_key(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_kafka_retry(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_output(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_file(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_latest_file(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_refresh_time(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_host(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_vhost(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_user(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_passwd(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_exchange(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_exchange_type(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_routing_key(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_routing_key_rr(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_persistent_msg(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_frame_max(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_amqp_heartbeat_interval(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_kafka_broker_host(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_kafka_broker_port(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_kafka_topic(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_kafka_topic_rr(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_kafka_partition(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_kafka_partition_key(char *, char *, char *);
 EXT int cfg_key_pcap_savefile(char *, char *, char *);
 EXT int cfg_key_maps_refresh(char *, char *, char *);
 EXT int cfg_key_maps_index(char *, char *, char *);
@@ -192,6 +278,13 @@ EXT int cfg_key_nfacctd_bgp_msglog_amqp_persistent_msg(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_msglog_amqp_frame_max(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_msglog_amqp_heartbeat_interval(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_msglog_amqp_retry(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_kafka_broker_host(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_kafka_broker_port(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_kafka_topic(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_kafka_topic_rr(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_kafka_partition(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_kafka_partition_key(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_kafka_retry(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_max_peers(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_ip(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_id(char *, char *, char *);
@@ -215,6 +308,7 @@ EXT int cfg_key_nfacctd_bgp_src_med_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_to_agent_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_follow_default(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_follow_nexthop(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_follow_nexthop_external(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_neighbors_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_md5_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_peer_buckets(char *, char *, char *);
@@ -223,6 +317,7 @@ EXT int cfg_key_nfacctd_bgp_table_attr_hash_buckets(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_per_peer_hash(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_output(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_file(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_latest_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_refresh_time(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_amqp_host(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_amqp_vhost(char *, char *, char *);
@@ -235,6 +330,12 @@ EXT int cfg_key_nfacctd_bgp_table_dump_amqp_routing_key_rr(char *, char *, char
 EXT int cfg_key_nfacctd_bgp_table_dump_amqp_persistent_msg(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_amqp_frame_max(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_amqp_heartbeat_interval(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_kafka_broker_host(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_kafka_broker_port(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_kafka_topic(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_kafka_topic_rr(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_kafka_partition(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_kafka_partition_key(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_batch(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_batch_interval(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_pipe_size(char *, char *, char *);
@@ -247,7 +348,6 @@ EXT int cfg_key_nfacctd_bmp_allow_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_ip_precedence(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_batch(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_batch_interval(char *, char *, char *);
-EXT int cfg_key_nfacctd_bmp_neighbors_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_msglog_output(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_msglog_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_msglog_amqp_host(char *, char *, char *);
@@ -262,12 +362,20 @@ EXT int cfg_key_nfacctd_bmp_msglog_amqp_persistent_msg(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_msglog_amqp_frame_max(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_msglog_amqp_heartbeat_interval(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_msglog_amqp_retry(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_msglog_kafka_broker_host(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_msglog_kafka_broker_port(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_msglog_kafka_topic(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_msglog_kafka_topic_rr(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_msglog_kafka_partition(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_msglog_kafka_partition_key(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_msglog_kafka_retry(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_table_peer_buckets(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_table_per_peer_buckets(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_table_attr_hash_buckets(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_table_per_peer_hash(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_output(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_file(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_dump_latest_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_refresh_time(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_amqp_host(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_amqp_vhost(char *, char *, char *);
@@ -280,6 +388,12 @@ EXT int cfg_key_nfacctd_bmp_dump_amqp_routing_key_rr(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_amqp_persistent_msg(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_amqp_frame_max(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_amqp_heartbeat_interval(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_dump_kafka_broker_host(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_dump_kafka_broker_port(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_dump_kafka_topic(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_dump_kafka_topic_rr(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_dump_kafka_partition(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_dump_kafka_partition_key(char *, char *, char *);
 EXT int cfg_key_nfacctd_flow_to_rd_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis_ip(char *, char *, char *);
@@ -294,9 +408,12 @@ EXT int cfg_key_geoip_ipv6_file(char *, char *, char *);
 EXT int cfg_key_geoipv2_file(char *, char *, char *);
 EXT int cfg_key_uacctd_group(char *, char *, char *);
 EXT int cfg_key_uacctd_nl_size(char *, char *, char *);
+EXT int cfg_key_uacctd_threshold(char *, char *, char *);
 EXT int cfg_key_tunnel_0(char *, char *, char *);
 EXT int cfg_key_pkt_len_distrib_bins(char *, char *, char *);
 EXT int cfg_key_tmp_net_own_field(char *, char *, char *);
+EXT int cfg_key_tmp_asa_bi_flow(char *, char *, char *);
+EXT int cfg_key_tmp_comms_same_field(char *, char *, char *);
 
 EXT void parse_time(char *, char *, int *, int *);
 EXT void cfg_set_aggregate(char *, u_int64_t [], u_int64_t, char *);
