@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2014 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -38,7 +38,8 @@ struct acc {
   unsigned int signature;
   u_int8_t reset_flag;
   struct timeval rstamp;	/* classifiers: reset timestamp */
-  struct cache_bgp_primitives *cbgp;
+  struct pkt_bgp_primitives *pbgp;
+  struct cache_legacy_bgp_primitives *clbgp;
   struct pkt_nat_primitives *pnat;
   struct pkt_mpls_primitives *pmpls;
   char *pcust;
@@ -73,14 +74,15 @@ struct query_header {
 };
 
 struct query_entry {
-  pm_cfgreg_t what_to_count;		/* aggregation */
-  pm_cfgreg_t what_to_count_2;		/* aggregation */
-  struct pkt_primitives data;		/* actual data */
-  struct pkt_bgp_primitives pbgp;	/* extended BGP data */
-  struct pkt_nat_primitives pnat;	/* extended NAT + timestamp data */
-  struct pkt_mpls_primitives pmpls;	/* extended MPLS data */
-  char *pcust;				/* custom-defined data */
-  struct pkt_vlen_hdr_primitives *pvlen;/* variable-length data */
+  pm_cfgreg_t what_to_count;			/* aggregation */
+  pm_cfgreg_t what_to_count_2;			/* aggregation */
+  struct pkt_primitives data;			/* actual data */
+  struct pkt_bgp_primitives pbgp;		/* extended BGP data */
+  struct pkt_legacy_bgp_primitives plbgp;	/* extended BGP data */
+  struct pkt_nat_primitives pnat;		/* extended NAT + timestamp data */
+  struct pkt_mpls_primitives pmpls;		/* extended MPLS data */
+  char *pcust;					/* custom-defined data */
+  struct pkt_vlen_hdr_primitives *pvlen;	/* variable-length data */
 };
 
 struct reply_buffer {
@@ -147,9 +149,9 @@ EXT void set_reset_flag(struct acc *);
 EXT void reset_counters(struct acc *);
 EXT int build_query_server(char *);
 EXT void process_query_data(int, unsigned char *, int, struct extra_primitives *, int, int);
-EXT void mask_elem(struct pkt_primitives *, struct pkt_bgp_primitives *, struct pkt_nat_primitives *,
-			struct pkt_mpls_primitives *, struct acc *, u_int64_t, u_int64_t,
-			struct extra_primitives *);
+EXT void mask_elem(struct pkt_primitives *, struct pkt_bgp_primitives *, struct pkt_legacy_bgp_primitives *,
+			struct pkt_nat_primitives *, struct pkt_mpls_primitives *, struct acc *, u_int64_t,
+			u_int64_t, struct extra_primitives *);
 EXT void enQueue_elem(int, struct reply_buffer *, void *, int, int);
 EXT void Accumulate_Counters(struct pkt_data *, struct acc *);
 EXT int test_zero_elem(struct acc *);
@@ -176,7 +178,7 @@ EXT void free_extra_allocs();
 #else
 #define EXT
 #endif
-EXT void (*insert_func)(struct primitives_ptrs *); /* pointer to INSERT function */
+EXT void (*imt_insert_func)(struct primitives_ptrs *); /* pointer to INSERT function */
 EXT unsigned char *mpd;  /* memory pool descriptors table */
 EXT unsigned char *a;  /* accounting in-memory table */
 EXT struct memory_pool_desc *current_pool; /* pointer to currently used memory pool */
