@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2014 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -230,7 +230,7 @@ struct my_icmphdr
     u_int32_t   gateway;        /* gateway address */
     struct
     {
-      u_int16_t __unused;
+      u_int16_t _unused;
       u_int16_t mtu;
     } frag;                     /* path mtu discovery */
   } un;
@@ -363,8 +363,8 @@ struct packet_ptrs {
   struct class_st cst; /* classifiers: class status */
   u_int8_t shadow; /* 0=the packet is being distributed for the 1st time
 		      1=the packet is being distributed for the 2nd+ time */
-  u_int16_t ifindex_in;  /* input ifindex; only used by ULOG for the time being */
-  u_int16_t ifindex_out; /* output ifindex; only used by ULOG for the time being */
+  u_int32_t ifindex_in;  /* input ifindex; only used by NFLOG for the time being */
+  u_int32_t ifindex_out; /* output ifindex; only used by NFLOG for the time being */
   u_int8_t tun_stack; /* tunnelling stack */
   u_int8_t tun_layer; /* tunnelling layer count */
   u_int32_t sample_type; /* sFlow sample type */
@@ -433,6 +433,8 @@ struct pkt_primitives {
   pm_class_t class;
   u_int32_t sampling_rate;
   u_int16_t pkt_len_distrib;
+  u_int32_t export_proto_seqno;
+  u_int16_t export_proto_version;
 };
 
 struct pkt_data {
@@ -460,8 +462,8 @@ struct pkt_payload {
   struct host_addr dst_ip;
   as_t src_as;
   as_t dst_as;
-  u_int16_t ifindex_in;
-  u_int16_t ifindex_out;
+  u_int32_t ifindex_in;
+  u_int32_t ifindex_out;
   u_int8_t src_nmask;
   u_int8_t dst_nmask;
   u_int16_t vlan;
@@ -472,7 +474,7 @@ struct pkt_payload {
 struct pkt_vlen_hdr_primitives {
   u_int16_t tot_len;
   u_int16_t num;
-};
+} __attribute__ ((packed));
 
 // XXX: eventually deprecate pkt_extras
 struct pkt_extras {
@@ -487,7 +489,7 @@ struct pkt_msg {
   struct sockaddr agent;
   u_int32_t seqno;
   u_int16_t len;
-  u_char payload[PKT_MSG_SIZE];
+  char *payload;
   pm_id_t tag;
   pm_id_t tag2;
   u_int16_t pad;
@@ -505,6 +507,7 @@ struct pkt_stitching {
 
 struct extra_primitives {
   u_int16_t off_pkt_bgp_primitives;
+  u_int16_t off_pkt_lbgp_primitives;
   u_int16_t off_pkt_nat_primitives;
   u_int16_t off_pkt_mpls_primitives;
   u_int16_t off_custom_primitives;
@@ -515,6 +518,7 @@ struct extra_primitives {
 struct primitives_ptrs {
   struct pkt_data *data;
   struct pkt_bgp_primitives *pbgp;
+  struct pkt_legacy_bgp_primitives *plbgp;
   struct pkt_nat_primitives *pnat;
   struct pkt_mpls_primitives *pmpls;
   char *pcust;
@@ -531,19 +535,22 @@ struct pkt_bgp_primitives {
   as_t peer_dst_as;
   struct host_addr peer_src_ip;
   struct host_addr peer_dst_ip;
-  char std_comms[MAX_BGP_STD_COMMS];
-  char ext_comms[MAX_BGP_EXT_COMMS];
-  char as_path[MAX_BGP_ASPATH];
   u_int32_t local_pref;
   u_int32_t med;
-  char src_std_comms[MAX_BGP_STD_COMMS];
-  char src_ext_comms[MAX_BGP_EXT_COMMS];
-  char src_as_path[MAX_BGP_ASPATH];
   u_int32_t src_local_pref;
   u_int32_t src_med;
   rd_t mpls_vpn_rd;
 };
 
+struct pkt_legacy_bgp_primitives {
+  char std_comms[MAX_BGP_STD_COMMS];
+  char ext_comms[MAX_BGP_EXT_COMMS];
+  char as_path[MAX_BGP_ASPATH];
+  char src_std_comms[MAX_BGP_STD_COMMS];
+  char src_ext_comms[MAX_BGP_EXT_COMMS];
+  char src_as_path[MAX_BGP_ASPATH];
+};
+
 struct pkt_nat_primitives {
   struct host_addr post_nat_src_ip;
   struct host_addr post_nat_dst_ip;
@@ -552,6 +559,7 @@ struct pkt_nat_primitives {
   u_int8_t nat_event;
   struct timeval timestamp_start; /* XXX: clean-up: to be moved in a separate structure */
   struct timeval timestamp_end; /* XXX: clean-up: to be moved in a separate structure */
+  struct timeval timestamp_arrival; /* XXX: clean-up: to be moved in a separate structure */
 };
 
 struct pkt_mpls_primitives {
@@ -560,23 +568,14 @@ struct pkt_mpls_primitives {
   u_int8_t mpls_stack_depth;
 };
 
-/* same as above but pointers in place of strings */
-struct cache_bgp_primitives {
-  as_t peer_src_as;
-  as_t peer_dst_as;
-  struct host_addr peer_src_ip;
-  struct host_addr peer_dst_ip;
+/* same as pkt_legacy_bgp_primitives but pointers in place of strings */
+struct cache_legacy_bgp_primitives {
   char *std_comms;
   char *ext_comms;
   char *as_path;
-  u_int32_t local_pref;
-  u_int32_t med;
   char *src_std_comms;
   char *src_ext_comms;
   char *src_as_path;
-  u_int32_t src_local_pref;
-  u_int32_t src_med;
-  rd_t mpls_vpn_rd;
 };
 /* END: BGP section */
 
@@ -627,4 +626,11 @@ struct tunnel_entry {
   tunnel_configurator tc;
 };
 
-struct tunnel_handler tunnel_registry[TUNNEL_REGISTRY_STACKS][TUNNEL_REGISTRY_ENTRIES];
+/* global variables */
+#if (!defined __PMACCTD_C) && (!defined __NFACCTD_C) && (!defined __SFACCTD_C) && (!defined __UACCTD_C) && (!defined __PMTELEMETRYD_C) && (!defined __PMBGPD_C) && (!defined __PMBMPD_C)
+#define EXT extern
+#else
+#define EXT
+#endif
+EXT struct tunnel_handler tunnel_registry[TUNNEL_REGISTRY_STACKS][TUNNEL_REGISTRY_ENTRIES];
+#undef EXT
