@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -155,13 +155,15 @@ struct template_cache_entry *insert_template(struct template_hdr_v9 *hdr, struct
       ptr->list[count].type = TPL_TYPE_LEGACY;
     }
     else {
-      struct utpl_field *ext_db_ptr = ext_db_get_next_ie(ptr, type);
+      u_int8_t repeat_id = 0;
+      struct utpl_field *ext_db_ptr = ext_db_get_next_ie(ptr, type, &repeat_id);
 
       if (ext_db_ptr) {
 	if (pen) ext_db_ptr->pen = ntohl(*pen);
 	ext_db_ptr->type = type;
 	ext_db_ptr->off = ptr->len;
 	ext_db_ptr->tpl_len = ntohs(field->len);
+	ext_db_ptr->repeat_id = repeat_id;
 
         if (ptr->vlen) ext_db_ptr->off = 0;
 
@@ -191,7 +193,7 @@ struct template_cache_entry *insert_template(struct template_hdr_v9 *hdr, struct
   if (prevptr) prevptr->next = ptr;
   else tpl_cache.c[modulo] = ptr;
 
-  log_template_footer(ptr->len, version);
+  log_template_footer(ptr, ptr->len, version);
 
   return ptr;
 }
@@ -264,13 +266,15 @@ struct template_cache_entry *refresh_template(struct template_hdr_v9 *hdr, struc
       tpl->list[count].type = TPL_TYPE_LEGACY;
     }
     else {
-      struct utpl_field *ext_db_ptr = ext_db_get_next_ie(tpl, type);
+      u_int8_t repeat_id = 0;
+      struct utpl_field *ext_db_ptr = ext_db_get_next_ie(tpl, type, &repeat_id);
 
       if (ext_db_ptr) {
         if (pen) ext_db_ptr->pen = ntohl(*pen);
         ext_db_ptr->type = type;
         ext_db_ptr->off = tpl->len;
         ext_db_ptr->tpl_len = ntohs(field->len);
+	ext_db_ptr->repeat_id = repeat_id;
 
         if (tpl->vlen) ext_db_ptr->off = 0;
 
@@ -297,7 +301,7 @@ struct template_cache_entry *refresh_template(struct template_hdr_v9 *hdr, struc
     field++;
   }
 
-  log_template_footer(tpl->len, version);
+  log_template_footer(tpl, tpl->len, version);
 
   return tpl;
 }
@@ -316,12 +320,12 @@ void log_template_header(struct template_cache_entry *tpl, struct packet_ptrs *p
   Log(LOG_DEBUG, "DEBUG ( %s/core ): NfV%u template ID   : %u\n", config.name, version, ntohs(tpl->template_id));
 
   if ( tpl->template_type == 0 || tpl->template_type == 2 ) {
-    Log(LOG_DEBUG, "DEBUG ( %s/core ): -----------------------------------------------------\n", config.name);
-    Log(LOG_DEBUG, "DEBUG ( %s/core ): |    pen     |     field type     | offset |  size  |\n", config.name);
+    Log(LOG_DEBUG, "DEBUG ( %s/core ): -------------------------------------------------------------\n", config.name);
+    Log(LOG_DEBUG, "DEBUG ( %s/core ): |    pen     |         field type         | offset |  size  |\n", config.name);
   }
   else {
-    Log(LOG_DEBUG, "DEBUG ( %s/core ): ----------------------------------------\n", config.name);
-    Log(LOG_DEBUG, "DEBUG ( %s/core ): |     field type     | offset |  size  |\n", config.name);
+    Log(LOG_DEBUG, "DEBUG ( %s/core ): ------------------------------------------------\n", config.name);
+    Log(LOG_DEBUG, "DEBUG ( %s/core ): |         field type         | offset |  size  |\n", config.name);
   }
 }
 
@@ -330,36 +334,40 @@ void log_template_field(u_int8_t vlen, u_int32_t *pen, u_int16_t type, u_int16_t
   if (!pen) {
     if (type <= MAX_TPL_DESC_LIST && strlen(tpl_desc_list[type])) { 
       if (!off && vlen)
-        Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18s | %6s | %6u |\n", config.name, 0, tpl_desc_list[type], "tbd", len);
+        Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18s [%-5u] | %6s | %6u |\n", config.name, 0, tpl_desc_list[type], type, "tbd", len);
       else
-        Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18s | %6u | %6u |\n", config.name, 0, tpl_desc_list[type], off, len);
+        Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18s [%-5u] | %6u | %6u |\n", config.name, 0, tpl_desc_list[type], type, off, len);
     }
     else {
       if (!off && vlen)
-        Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18u | %6s | %6u |\n", config.name, 0, type, "tbd", len);
+        Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18u [%-5u] | %6s | %6u |\n", config.name, 0, type, type, "tbd", len);
       else
-        Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18u | %6u | %6u |\n", config.name, 0, type, off, len);
+        Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18u [%-5u] | %6u | %6u |\n", config.name, 0, type, type, off, len);
     }
   }
   else {
     if (!off && vlen) 
-      Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18u | %6s | %6u |\n", config.name, ntohl(*pen), type, "tbd", len);
+      Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18u [%-5u] | %6s | %6u |\n", config.name, ntohl(*pen), type, type, "tbd", len);
     else 
-      Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18u | %6u | %6u |\n", config.name, ntohl(*pen), type, off, len);
+      Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-10u | %-18u [%-5u] | %6u | %6u |\n", config.name, ntohl(*pen), type, type, off, len);
   }
 }
 
 void log_opt_template_field(u_int16_t type, u_int16_t off, u_int16_t len, u_int8_t version)
 {
   if (type <= MAX_OPT_TPL_DESC_LIST && strlen(opt_tpl_desc_list[type]))
-    Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-18s | %6u | %6u |\n", config.name, opt_tpl_desc_list[type], off, len);
+    Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-18s [%-5u] | %6u | %6u |\n", config.name, opt_tpl_desc_list[type], type, off, len);
   else
-    Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-18u | %6u | %6u |\n", config.name, type, off, len);
+    Log(LOG_DEBUG, "DEBUG ( %s/core ): | %-18u [%-5u] | %6u | %6u |\n", config.name, type, type, off, len);
 }
 
-void log_template_footer(u_int16_t size, u_int8_t version)
+void log_template_footer(struct template_cache_entry *tpl, u_int16_t size, u_int8_t version)
 {
-  Log(LOG_DEBUG, "DEBUG ( %s/core ): -----------------------------------------------------\n", config.name);
+  if ( tpl->template_type == 0 || tpl->template_type == 2 )
+    Log(LOG_DEBUG, "DEBUG ( %s/core ): -------------------------------------------------------------\n", config.name);
+  else 
+    Log(LOG_DEBUG, "DEBUG ( %s/core ): ------------------------------------------------\n", config.name);
+
   if (!size)
     Log(LOG_DEBUG, "DEBUG ( %s/core ): Netflow V9/IPFIX record size : %s\n", config.name, "tbd");
   else 
@@ -447,7 +455,7 @@ struct template_cache_entry *insert_opt_template(void *hdr, struct packet_ptrs *
   if (prevptr) prevptr->next = ptr;
   else tpl_cache.c[modulo] = ptr;
 
-  log_template_footer(ptr->len, version);
+  log_template_footer(ptr, ptr->len, version);
 
   return ptr;
 }
@@ -517,7 +525,7 @@ struct template_cache_entry *refresh_opt_template(void *hdr, struct template_cac
     off += NfTplFieldV9Sz;
   }
 
-  log_template_footer(tpl->len, version);
+  log_template_footer(tpl, tpl->len, version);
 
   return tpl;
 }
@@ -585,14 +593,15 @@ u_int8_t get_ipfix_vlen(char *base, u_int16_t *len)
   return ret;
 }
 
-struct utpl_field *ext_db_get_ie(struct template_cache_entry *ptr, u_int32_t pen, u_int16_t type)
+struct utpl_field *ext_db_get_ie(struct template_cache_entry *ptr, u_int32_t pen, u_int16_t type, u_int8_t repeat_id)
 {
   u_int16_t ie_idx, ext_db_modulo = (type%TPL_EXT_DB_ENTRIES);
   struct utpl_field *ext_db_ptr = NULL;
 
   for (ie_idx = 0; ie_idx < IES_PER_TPL_EXT_DB_ENTRY; ie_idx++) {
     if (ptr->ext_db[ext_db_modulo].ie[ie_idx].type == type &&
-	ptr->ext_db[ext_db_modulo].ie[ie_idx].pen == pen) {
+	ptr->ext_db[ext_db_modulo].ie[ie_idx].pen == pen &&
+	ptr->ext_db[ext_db_modulo].ie[ie_idx].repeat_id == repeat_id) {
       ext_db_ptr = &ptr->ext_db[ext_db_modulo].ie[ie_idx];
       break;
     }
@@ -601,12 +610,16 @@ struct utpl_field *ext_db_get_ie(struct template_cache_entry *ptr, u_int32_t pen
   return ext_db_ptr;
 }
 
-struct utpl_field *ext_db_get_next_ie(struct template_cache_entry *ptr, u_int16_t type)
+struct utpl_field *ext_db_get_next_ie(struct template_cache_entry *ptr, u_int16_t type, u_int8_t *repeat_id)
 {
   u_int16_t ie_idx, ext_db_modulo = (type%TPL_EXT_DB_ENTRIES);
   struct utpl_field *ext_db_ptr = NULL;
 
+  (*repeat_id) = 0;
+
   for (ie_idx = 0; ie_idx < IES_PER_TPL_EXT_DB_ENTRY; ie_idx++) {
+    if (ptr->ext_db[ext_db_modulo].ie[ie_idx].type == type) (*repeat_id)++;
+
     if (ptr->ext_db[ext_db_modulo].ie[ie_idx].type == 0) {
       ext_db_ptr = &ptr->ext_db[ext_db_modulo].ie[ie_idx];
       break;
