@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -65,11 +65,15 @@ void pcap_cb(u_char *user, const struct pcap_pkthdr *pkthdr, const u_char *buf)
         }
         if (config.nfacctd_bgp) {
           BTA_find_id((struct id_table *)pptrs.bta_table, &pptrs, &pptrs.bta, &pptrs.bta2);
-          bgp_srcdst_lookup(&pptrs);
+          bgp_srcdst_lookup(&pptrs, FUNC_TYPE_BGP);
         }
         if (config.nfacctd_bgp_peer_as_src_map) PM_find_id((struct id_table *)pptrs.bpas_table, &pptrs, &pptrs.bpas, NULL);
         if (config.nfacctd_bgp_src_local_pref_map) PM_find_id((struct id_table *)pptrs.blp_table, &pptrs, &pptrs.blp, NULL);
         if (config.nfacctd_bgp_src_med_map) PM_find_id((struct id_table *)pptrs.bmed_table, &pptrs, &pptrs.bmed, NULL);
+        if (config.nfacctd_bmp) {
+          BTA_find_id((struct id_table *)pptrs.bta_table, &pptrs, &pptrs.bta, &pptrs.bta2);
+	  bmp_srcdst_lookup(&pptrs);
+	}
 
 	set_index_pkt_ptrs(&pptrs);
         exec_plugins(&pptrs, &req);
@@ -390,6 +394,7 @@ void compute_once()
   PpayloadSz = sizeof(struct pkt_payload);
   PextrasSz = sizeof(struct pkt_extras);
   PbgpSz = sizeof(struct pkt_bgp_primitives);
+  PlbgpSz = sizeof(struct pkt_legacy_bgp_primitives);
   PnatSz = sizeof(struct pkt_nat_primitives);
   PmplsSz = sizeof(struct pkt_mpls_primitives);
   PvhdrSz = sizeof(struct pkt_vlen_hdr_primitives);
@@ -482,8 +487,6 @@ int gtp_tunnel_func(register struct packet_ptrs *pptrs)
     ret = 0; trial = 0;
 
     while (!ret && trial < MAX_GTP_TRIALS) {
-      off++; ptr++; trial++;
-
       pptrs->iph_ptr = ptr;
       pptrs->tlh_ptr = NULL; pptrs->payload_ptr = NULL;
       pptrs->l4_proto = 0; pptrs->tcp_flags = 0;
@@ -530,6 +533,9 @@ int gtp_tunnel_func(register struct packet_ptrs *pptrs)
         ret = FALSE;
 	break;
       }
+
+      /* next loop increment */
+      off++; ptr++; trial++;
     }
   }
   else {
