@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -62,6 +62,7 @@ EXT void in_iface_handler(struct channels_list_entry *, struct packet_ptrs *, ch
 EXT void out_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void sampling_rate_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void timestamp_start_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void timestamp_arrival_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void custom_primitives_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void sfprobe_payload_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void tee_payload_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -107,6 +108,9 @@ EXT void NF_mpls_stack_depth_handler(struct channels_list_entry *, struct packet
 EXT void NF_mpls_vpn_rd_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_timestamp_start_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_timestamp_end_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_timestamp_arrival_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_sequence_number_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_version_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_custom_primitives_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_counters_renormalize_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_counters_map_renormalize_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -157,6 +161,9 @@ EXT void SF_out_iface_handler(struct channels_list_entry *, struct packet_ptrs *
 EXT void SF_sampling_rate_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_sampling_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_timestamp_start_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void SF_timestamp_arrival_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void SF_sequence_number_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void SF_version_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_bgp_peer_src_as_fromext_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_bgp_peer_src_as_fromstd_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_as_path_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
