@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -41,7 +41,7 @@ void load_plugins(struct plugin_requests *req)
   u_int64_t buf_pipe_ratio_sz = 0, pipe_idx = 0;
   int snd_buflen = 0, rcv_buflen = 0, socklen = 0, target_buflen = 0, ret;
 
-  int nfprobe_id = 0, min_sz = 0;
+  int nfprobe_id = 0, min_sz = 0, extra_sz = 0;
   struct plugins_list_entry *list = plugins_list;
   int l = sizeof(list->cfg.pipe_size), offset = 0;
   struct channels_list_entry *chptr = NULL;
@@ -58,25 +58,39 @@ void load_plugins(struct plugin_requests *req)
       }
 
       min_sz = ChBufHdrSz;
+      list->cfg.buffer_immediate = FALSE;
       if (list->cfg.data_type & PIPE_TYPE_METADATA) min_sz += PdataSz; 
       if (list->cfg.data_type & PIPE_TYPE_PAYLOAD) {
 	if (list->cfg.acct_type == ACCT_PM && list->cfg.snaplen) min_sz += (PpayloadSz+list->cfg.snaplen); 
 	else min_sz += (PpayloadSz+DEFAULT_PLOAD_SIZE); 
       }
       if (list->cfg.data_type & PIPE_TYPE_EXTRAS) min_sz += PextrasSz; 
-      if (list->cfg.data_type & PIPE_TYPE_MSG) min_sz += PmsgSz; 
+      if (list->cfg.data_type & PIPE_TYPE_MSG) {
+	min_sz += PmsgSz; 
+        if (!list->cfg.buffer_size) {
+          extra_sz = PKT_MSG_SIZE; 
+          list->cfg.buffer_immediate = TRUE;
+        }
+      }
       if (list->cfg.data_type & PIPE_TYPE_BGP) min_sz += sizeof(struct pkt_bgp_primitives);
+      if (list->cfg.data_type & PIPE_TYPE_LBGP) min_sz += sizeof(struct pkt_legacy_bgp_primitives);
       if (list->cfg.data_type & PIPE_TYPE_NAT) min_sz += sizeof(struct pkt_nat_primitives);
       if (list->cfg.data_type & PIPE_TYPE_MPLS) min_sz += sizeof(struct pkt_mpls_primitives);
       if (list->cfg.cpptrs.len) min_sz += list->cfg.cpptrs.len;
-      if (list->cfg.data_type & PIPE_TYPE_VLEN) min_sz += sizeof(struct pkt_vlen_hdr_primitives);
+      if (list->cfg.data_type & PIPE_TYPE_VLEN) {
+	min_sz += sizeof(struct pkt_vlen_hdr_primitives);
+	if (!list->cfg.buffer_size) {
+	  extra_sz = 1024; /* wild shot: 1Kb added for the actual variable-length data */
+	  list->cfg.buffer_immediate = TRUE;
+	}
+      }
 
       /* If nothing is supplied, let's hint some working default values */
       if (!list->cfg.pipe_size || !list->cfg.buffer_size) {
         if (!list->cfg.pipe_size) list->cfg.pipe_size = 4096000; /* 4Mb */
         if (!list->cfg.buffer_size) {
 	  if (list->cfg.pcap_savefile) list->cfg.buffer_size = 10240; /* 10Kb */
-	  else list->cfg.buffer_size = MIN(min_sz, 10240);
+	  else list->cfg.buffer_size = MIN((min_sz + extra_sz), 10240);
 	}
       }
 
@@ -174,6 +188,11 @@ void load_plugins(struct plugin_requests *req)
 	offset += sizeof(struct pkt_bgp_primitives);
       }
       else chptr->extras.off_pkt_bgp_primitives = 0; 
+      if (list->cfg.data_type & PIPE_TYPE_LBGP) {
+        chptr->extras.off_pkt_lbgp_primitives = offset;
+        offset += sizeof(struct pkt_legacy_bgp_primitives);
+      }
+      else chptr->extras.off_pkt_lbgp_primitives = 0;
       if (list->cfg.data_type & PIPE_TYPE_NAT) {
         chptr->extras.off_pkt_nat_primitives = offset;
         offset += sizeof(struct pkt_nat_primitives);
@@ -265,8 +284,18 @@ void load_plugins(struct plugin_requests *req)
             ptm_global = FALSE;
         }
 
+	if (list->cfg.type_id == PLUGIN_ID_TEE) {
+	  req->ptm_c.load_ptm_plugin = list->cfg.type_id;
+	  req->ptm_c.load_ptm_res = FALSE;
+	}
+
         load_pre_tag_map(config.acct_type, list->cfg.pre_tag_map, &list->cfg.ptm, req, &list->cfg.ptm_alloc,
                          list->cfg.maps_entries, list->cfg.maps_row_len);
+
+	if (list->cfg.type_id == PLUGIN_ID_TEE) {
+	  list->cfg.ptm_complex = req->ptm_c.load_ptm_res;
+	  if (req->ptm_c.load_ptm_res) req->ptm_c.exec_ptm_dissect = TRUE;
+	}
       }
 
       list = list->next;
@@ -285,7 +314,7 @@ void load_plugins(struct plugin_requests *req)
   /* AMQP handling, if required */
 #ifdef WITH_RABBITMQ
   {
-    int ret, index;
+    int ret, index, index2;
 
     for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
       chptr = &channels_list[index];
@@ -296,6 +325,73 @@ void load_plugins(struct plugin_requests *req)
         ret = p_amqp_connect_to_publish(&chptr->amqp_host);
         if (ret) plugin_pipe_amqp_sleeper_start(chptr);
       }
+
+      /* reset core process pipe AMQP routing key */
+      if (list->type.id == PLUGIN_ID_CORE) list->cfg.pipe_amqp_routing_key = NULL;
+    }
+
+    for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
+      struct plugins_list_entry *list2 = plugins_list;
+      struct channels_list_entry *chptr2 = NULL;
+
+      chptr = &channels_list[index];
+      list = chptr->plugin;
+
+      for (index2 = index; channels_list[index2].aggregation || channels_list[index2].aggregation_2; index2++) {
+        chptr2 = &channels_list[index2];
+        list2 = chptr2->plugin;
+
+	if (index2 > index && list->cfg.pipe_amqp_exchange && list->cfg.pipe_amqp_routing_key) {
+	  if (!strcmp(list->cfg.pipe_amqp_exchange, list2->cfg.pipe_amqp_exchange) &&
+	      !strcmp(list->cfg.pipe_amqp_routing_key, list2->cfg.pipe_amqp_routing_key)) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): Duplicated plugin_pipe_amqp_exchange, plugin_pipe_amqp_routing_key: %s, %s\nExiting.\n",
+		list->name, list->type.string, list->cfg.pipe_amqp_exchange, list->cfg.pipe_amqp_routing_key);
+	    exit(1);
+	  }
+        }
+      }
+    }
+  }
+#endif
+
+  /* Kafka handling, if required */
+#ifdef WITH_KAFKA
+  {
+    int ret, index, index2;
+
+    for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
+      chptr = &channels_list[index];
+      list = chptr->plugin;
+
+      /* XXX: no sleeper thread, trusting librdkafka */
+      if (list->cfg.pipe_kafka) ret = plugin_pipe_kafka_init_host(&chptr->kafka_host, list, TRUE);
+
+      /* reset core process pipe Kafka topic */
+      if (list->type.id == PLUGIN_ID_CORE) list->cfg.pipe_kafka_topic = NULL;
+    }
+
+    for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
+      struct plugins_list_entry *list2 = plugins_list;
+      struct channels_list_entry *chptr2 = NULL;
+
+      chptr = &channels_list[index];
+      list = chptr->plugin;
+
+      for (index2 = index; channels_list[index2].aggregation || channels_list[index2].aggregation_2; index2++) {
+        chptr2 = &channels_list[index2];
+        list2 = chptr2->plugin;
+
+        if (index2 > index && list->cfg.pipe_kafka_broker_host && list->cfg.pipe_kafka_topic) {
+          if (!strcmp(list->cfg.pipe_kafka_broker_host, list2->cfg.pipe_kafka_broker_host) &&
+              list->cfg.pipe_kafka_broker_port == list2->cfg.pipe_kafka_broker_port &&
+              !strcmp(list->cfg.pipe_kafka_topic, list2->cfg.pipe_kafka_topic) /* && XXX: topic partition too? */ ) {
+            Log(LOG_ERR, "ERROR ( %s/%s ): Duplicated plugin_pipe_kafka_broker_*, plugin_pipe_kafka_topic: %s, %s, %s\nExiting.\n",
+                list->name, list->type.string, list->cfg.pipe_kafka_broker_host, list->cfg.pipe_kafka_broker_port,
+		list->cfg.pipe_kafka_topic);
+            exit(1);
+          }
+        }
+      }
     }
   }
 #endif
@@ -307,7 +403,7 @@ void exec_plugins(struct packet_ptrs *pptrs, struct plugin_requests *req)
   pm_id_t saved_tag = 0, saved_tag2 = 0;
   pt_label_t saved_label;
 
-  int num, ret, fixed_size, already_reprocessed = 0;
+  int num, ret, fixed_size;
   u_int32_t savedptr;
   char *bptr;
   int index, got_tags = FALSE;
@@ -326,7 +422,14 @@ void exec_plugins(struct packet_ptrs *pptrs, struct plugin_requests *req)
   for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
     struct plugins_list_entry *p = channels_list[index].plugin;
 
+    channels_list[index].already_reprocessed = FALSE;
+
     if (p->cfg.pre_tag_map && find_id_func) {
+      if (p->cfg.type_id == PLUGIN_ID_TEE) {
+	if ((req->ptm_c.exec_ptm_res && !p->cfg.ptm_complex) || (!req->ptm_c.exec_ptm_res && p->cfg.ptm_complex)) 
+	  continue;
+      }
+
       if (p->cfg.ptm_global && got_tags) {
         pptrs->tag = saved_tag;
         pptrs->tag2 = saved_tag2;
@@ -386,13 +489,14 @@ reprocess:
 
       if (channels_list[index].reprocess) {
         /* Let's check if we have an issue with the buffer size */
-        if (already_reprocessed) {
+        if (channels_list[index].already_reprocessed) {
           struct plugins_list_entry *list = channels_list[index].plugin;
 
           Log(LOG_ERR, "ERROR ( %s/%s ): plugin_buffer_size is too short.\n", list->name, list->type.string);
           exit_all(1);
         }
-        already_reprocessed = TRUE;
+
+        channels_list[index].already_reprocessed = TRUE;
 
 	/* Let's cheat the size in order to send out the current buffer */
 	fixed_size = channels_list[index].plugin->cfg.pipe_size;
@@ -402,22 +506,47 @@ reprocess:
         channels_list[index].bufptr += (fixed_size + channels_list[index].var_size);
       }
 
-      if ((channels_list[index].bufptr+fixed_size) > channels_list[index].bufend ||
-	  channels_list[index].hdr.num == INT_MAX) {
+      if (((channels_list[index].bufptr + fixed_size) > channels_list[index].bufend) ||
+	  (channels_list[index].hdr.num == INT_MAX) || channels_list[index].buffer_immediate) {
 	channels_list[index].hdr.seq++;
 	channels_list[index].hdr.seq %= MAX_SEQNUM;
 
 	/* let's commit the buffer we just finished writing */
+	((struct ch_buf_hdr *)channels_list[index].rg.ptr)->len = channels_list[index].bufptr;
 	((struct ch_buf_hdr *)channels_list[index].rg.ptr)->seq = channels_list[index].hdr.seq;
 	((struct ch_buf_hdr *)channels_list[index].rg.ptr)->num = channels_list[index].hdr.num;
+	((struct ch_buf_hdr *)channels_list[index].rg.ptr)->core_pid = channels_list[index].core_pid;
 
-        if (config.debug) {
+	channels_list[index].status->last_buf_off = (u_int64_t)(channels_list[index].rg.ptr - channels_list[index].rg.base);
+
+        if (config.debug_internal_msg) {
 	  struct plugins_list_entry *list = channels_list[index].plugin;
-	  Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer released seq=%u num_entries=%u\n", list->name, list->type.string,
-		channels_list[index].hdr.seq, channels_list[index].hdr.num);
+	  Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer released cpid=%u len=%llu seq=%u num_entries=%u off=%llu\n",
+		list->name, list->type.string, channels_list[index].core_pid, channels_list[index].bufptr,
+		channels_list[index].hdr.seq, channels_list[index].hdr.num, channels_list[index].status->last_buf_off);
 	}
 
-	if (!channels_list[index].plugin->cfg.pipe_amqp) {
+	/* sending the buffer to the AMQP broker */
+	if (channels_list[index].plugin->cfg.pipe_amqp) {
+#ifdef WITH_RABBITMQ
+          struct channels_list_entry *chptr = &channels_list[index];
+
+          plugin_pipe_amqp_sleeper_stop(chptr);
+	  if (!chptr->amqp_host_sleep) ret = p_amqp_publish_binary(&chptr->amqp_host, chptr->rg.ptr, chptr->bufsize);
+	  else ret = FALSE;
+          if (ret) plugin_pipe_amqp_sleeper_start(chptr);
+#endif
+	}
+	/* sending the buffer to the Kafka broker */
+	else if (channels_list[index].plugin->cfg.pipe_kafka) {
+#ifdef WITH_KAFKA
+          struct channels_list_entry *chptr = &channels_list[index];
+
+	  /* XXX: no sleeper thread, trusting librdkafka */
+	  ret = p_kafka_produce_data(&chptr->kafka_host, chptr->rg.ptr, chptr->bufsize);
+#endif
+	}
+	else {
 	  if (channels_list[index].status->wakeup) {
 	    channels_list[index].status->backlog++;
 	  
@@ -433,16 +562,6 @@ reprocess:
 	    }
 	  }
 	}
-	/* sending the buffer to the AMQP broker */
-#ifdef WITH_RABBITMQ
-	else {
-	  struct channels_list_entry *chptr = &channels_list[index];
-
-	  plugin_pipe_amqp_sleeper_stop(chptr);
-	  ret = p_amqp_publish_binary(&chptr->amqp_host, chptr->rg.ptr, chptr->bufsize);
-	  if (ret) plugin_pipe_amqp_sleeper_start(chptr);
-	}
-#endif
 
 	channels_list[index].rg.ptr += channels_list[index].bufsize;
 
@@ -452,6 +571,7 @@ reprocess:
 	/* let's protect the buffer we are going to write */
         ((struct ch_buf_hdr *)channels_list[index].rg.ptr)->seq = -1;
         ((struct ch_buf_hdr *)channels_list[index].rg.ptr)->num = 0;
+        ((struct ch_buf_hdr *)channels_list[index].rg.ptr)->core_pid = 0;
 
         /* rewind pointer */
         channels_list[index].bufptr = channels_list[index].buf;
@@ -474,12 +594,24 @@ reprocess:
      ensure we reload it for all plugins and prevent any
      timing issues with pointers to labels */
   if (reload_map_exec_plugins) {
+    memset(&req->ptm_c, 0, sizeof(struct ptm_complex)); 
+
     for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
       struct plugins_list_entry *p = channels_list[index].plugin;
 
       if (p->cfg.pre_tag_map && find_id_func) {
+        if (p->cfg.type_id == PLUGIN_ID_TEE) {
+          req->ptm_c.load_ptm_plugin = p->cfg.type_id;
+          req->ptm_c.load_ptm_res = FALSE;
+        }
+
         load_pre_tag_map(config.acct_type, p->cfg.pre_tag_map, &p->cfg.ptm, req, &p->cfg.ptm_alloc,
                          p->cfg.maps_entries, p->cfg.maps_row_len);
+
+        if (p->cfg.type_id == PLUGIN_ID_TEE) {
+          p->cfg.ptm_complex = req->ptm_c.load_ptm_res;
+          if (req->ptm_c.load_ptm_res) req->ptm_c.exec_ptm_dissect = TRUE;
+        }
       }
     }
   }
@@ -503,6 +635,8 @@ struct channels_list_entry *insert_pipe_channel(int plugin_type, struct configur
       chptr->agg_filter.table = cfg->bpfp_a_table;
       chptr->agg_filter.num = (int *) &cfg->bpfp_a_num; 
       chptr->bufsize = cfg->buffer_size;
+      chptr->buffer_immediate = cfg->buffer_immediate;
+      chptr->core_pid = getpid();
       chptr->tag = cfg->post_tag;
       chptr->tag2 = cfg->post_tag2;
       if (cfg->sampling_rate && plugin_type != PLUGIN_ID_SFPROBE) { /* sfprobe cares for itself */
@@ -736,17 +870,25 @@ void fill_pipe_buffer()
 
     ((struct ch_buf_hdr *)chptr->rg.ptr)->seq = chptr->hdr.seq;
     ((struct ch_buf_hdr *)chptr->rg.ptr)->num = chptr->hdr.num;
+    ((struct ch_buf_hdr *)chptr->rg.ptr)->core_pid = chptr->core_pid;
 
-    if (!chptr->plugin->cfg.pipe_amqp) {
+    if (chptr->plugin->cfg.pipe_amqp) {
+#ifdef WITH_RABBITMQ
+      p_amqp_publish_binary(&chptr->amqp_host, chptr->rg.ptr, chptr->bufsize);
+#endif
+    }
+    else if (chptr->plugin->cfg.pipe_kafka) {
+#ifdef WITH_KAFKA
+      p_kafka_produce_data(&chptr->kafka_host, chptr->rg.ptr, chptr->bufsize);
+#endif
+    }
+    else {
       if (chptr->status->wakeup) {
         chptr->status->wakeup = chptr->request;
         if (write(chptr->pipe, &chptr->rg.ptr, CharPtrSz) != CharPtrSz)
-	  Log(LOG_WARNING, "WARN: Failed during write: %s\n", strerror(errno));
+	  Log(LOG_WARNING, "WARN ( %s/%s ): Failed during write: %s\n", chptr->plugin->cfg.name, chptr->plugin->cfg.type, strerror(errno));
       }
     }
-#ifdef WITH_RABBITMQ
-    else p_amqp_publish_binary(&chptr->amqp_host, chptr->rg.ptr, chptr->bufsize);
-#endif
   }
 }
 
@@ -754,14 +896,14 @@ int check_pipe_buffer_space(struct channels_list_entry *mychptr, struct pkt_vlen
 {
   int buf_space = 0;
 
-  if (!mychptr || !pvlen) return ERR;
+  if (!mychptr) return ERR;
 
   /* init to base of current element */
   buf_space = mychptr->bufend - mychptr->bufptr;
 
   /* subtract fixed part, current variable part and new var part (len) */
   buf_space -= mychptr->datasize;
-  buf_space -= pvlen->tot_len;
+  if (pvlen) buf_space -= pvlen->tot_len;
   buf_space -= len;
 
   /* return virdict. if positive fix sizes. if negative take care of triggering a reprocess */
@@ -862,32 +1004,48 @@ int pkt_extras_clean(void *pextras, int len)
   return PdataSz+PextrasSz;
 }
 
-#ifdef WITH_RABBITMQ
-char *plugin_pipe_amqp_compose_routing_key(char *name, char *type)
+void handle_plugin_pipe_dyn_strings(char *new, int newlen, char *old, struct plugins_list_entry *list)
 {
-  char *rk = NULL, sep[] = "-";
-  int len = 0;
+  int oldlen, ptr_len;
+  char core_proc_name[] = "$core_proc_name", plugin_name[] = "$plugin_name";
+  char plugin_type[] = "$plugin_type";
+  char *ptr_start, *ptr_end;
 
-  if (!name || !type) return;
+  if (!new || !old || !list) return;
 
-  len = strlen(name) + strlen(type) + 2;
+  oldlen = strlen(old);
+  if (oldlen <= newlen) strcpy(new, old);
+  else {
+    strncpy(new, old, newlen);
+    return;
+  }
 
-  rk = malloc(len);
-  memset(rk, 0, len);
+  replace_string(new, newlen, core_proc_name, list->cfg.proc_name);
+  replace_string(new, newlen, plugin_name, list->cfg.name);
+  replace_string(new, newlen, plugin_type, list->cfg.type);
+}
 
-  strncpy(rk, name, len);
-  strncat(rk, sep, (len-strlen(rk)));
-  strncat(rk, type, (len-strlen(rk)));
+char *plugin_pipe_compose_default_string(struct plugins_list_entry *list, char *default_rk)
+{
+  char *rk = NULL;
+
+  if (!list || !default_rk) return rk;
+
+  rk = malloc(SRVBUFLEN);
+  memset(rk, 0, SRVBUFLEN);
+
+  handle_plugin_pipe_dyn_strings(rk, SRVBUFLEN, default_rk, list);
 
   return rk;
 }
 
+#ifdef WITH_RABBITMQ
 void plugin_pipe_amqp_init_host(struct p_amqp_host *amqp_host, struct plugins_list_entry *list)
 {
   int ret;
 
   if (amqp_host) {
-    char *amqp_rk = plugin_pipe_amqp_compose_routing_key(list->cfg.name, list->cfg.type);
+    char *amqp_rk = plugin_pipe_compose_default_string(list, "$core_proc_name-$plugin_name-$plugin_type");
 
     p_amqp_init_host(amqp_host);
 
@@ -905,7 +1063,7 @@ void plugin_pipe_amqp_init_host(struct p_amqp_host *amqp_host, struct plugins_li
     p_amqp_set_host(amqp_host, list->cfg.pipe_amqp_host);
     p_amqp_set_vhost(amqp_host, list->cfg.pipe_amqp_vhost);
     p_amqp_set_routing_key(amqp_host, list->cfg.pipe_amqp_routing_key);
-    p_amqp_set_retry_interval(amqp_host, list->cfg.pipe_amqp_retry);
+    P_broker_timers_set_retry_interval(&amqp_host->btimers, list->cfg.pipe_amqp_retry);
 
     p_amqp_set_frame_max(amqp_host, list->cfg.buffer_size);
     p_amqp_set_exchange_type(amqp_host, default_amqp_exchange_type);
@@ -918,7 +1076,7 @@ struct plugin_pipe_amqp_sleeper *plugin_pipe_amqp_sleeper_define(struct p_amqp_h
   struct plugin_pipe_amqp_sleeper *pas;
   int size = sizeof(struct plugin_pipe_amqp_sleeper);
 
-  if (!amqp_host || !flag) return;
+  if (!amqp_host || !flag) return NULL;
 
   pas = malloc(size);
 
@@ -951,7 +1109,7 @@ void plugin_pipe_amqp_sleeper_publish_func(struct plugin_pipe_amqp_sleeper *pas)
   if (!pas || !pas->amqp_host || !pas->plugin || !pas->do_reconnect) return;
 
 sleep_again:
-  sleep(p_amqp_get_retry_interval(pas->amqp_host));
+  sleep(P_broker_timers_get_retry_interval(&pas->amqp_host->btimers));
 
   plugin_pipe_amqp_init_host(pas->amqp_host, pas->plugin);
   ret = p_amqp_connect_to_publish(pas->amqp_host);
@@ -973,7 +1131,8 @@ void plugin_pipe_amqp_sleeper_start(struct channels_list_entry *chptr)
     assert(chptr->amqp_host_sleep);
 
     pas = plugin_pipe_amqp_sleeper_define(&chptr->amqp_host, &chptr->amqp_host_reconnect, chptr->plugin);
-    send_to_pool((thread_pool_t *) chptr->amqp_host_sleep, plugin_pipe_amqp_sleeper_publish_func, pas);
+    if (pas) send_to_pool((thread_pool_t *) chptr->amqp_host_sleep, plugin_pipe_amqp_sleeper_publish_func, pas);
+    else Log(LOG_ERR, "ERROR ( %s/%s ): plugin_pipe_amqp_sleeper_start(): sleeper define failed\n", chptr->plugin->cfg.name, chptr->plugin->cfg.type);
   }
 #endif
 }
@@ -983,6 +1142,7 @@ void plugin_pipe_amqp_sleeper_stop(struct channels_list_entry *chptr)
 #if defined ENABLE_THREADS
   if (chptr && chptr->amqp_host_reconnect) {
     deallocate_thread_pool((thread_pool_t **) &chptr->amqp_host_sleep);
+    chptr->amqp_host_sleep = NULL;
     chptr->amqp_host_reconnect = FALSE;
   }
 #endif
@@ -994,28 +1154,93 @@ int plugin_pipe_amqp_connect_to_consume(struct p_amqp_host *amqp_host, struct pl
   p_amqp_connect_to_consume(amqp_host);
   return p_amqp_get_sockfd(amqp_host);
 }
-
-int plugin_pipe_amqp_set_poll_timeout(struct p_amqp_host *amqp_host, int pipe_fd)
-{
-  if (pipe_fd == ERR) return (p_amqp_get_retry_interval(amqp_host) * 1000);
-  else return AMQP_LONGLONG_RETRY;
-}
-
-int plugin_pipe_amqp_calc_poll_timeout_diff(struct p_amqp_host *amqp_host, time_t now)
-{
-  int amqp_timeout;
-
-  amqp_timeout = (((p_amqp_get_last_fail(amqp_host) + p_amqp_get_retry_interval(amqp_host)) - now) * 1000);
-  assert(amqp_timeout >= 0);
-
-  return amqp_timeout;
-}
 #endif
 
+#if defined WITH_KAFKA
+int plugin_pipe_kafka_init_host(struct p_kafka_host *kafka_host, struct plugins_list_entry *list, int is_prod)
+{
+  int ret = SUCCESS;
+
+  if (kafka_host && list && !validate_truefalse(is_prod)) {
+    char *topic = plugin_pipe_compose_default_string(list, "pmacct.$core_proc_name-$plugin_name-$plugin_type");
+
+    p_kafka_init_host(kafka_host);
+
+    if (is_prod) ret = p_kafka_connect_to_produce(kafka_host);
+    else ret = p_kafka_connect_to_consume(kafka_host);
+
+    if (!list->cfg.pipe_kafka_broker_host) list->cfg.pipe_kafka_broker_host = default_kafka_broker_host;
+    if (!list->cfg.pipe_kafka_broker_port) list->cfg.pipe_kafka_broker_port = default_kafka_broker_port;
+    if (!list->cfg.pipe_kafka_topic) list->cfg.pipe_kafka_topic = topic;
+    if (!list->cfg.pipe_kafka_retry) list->cfg.pipe_kafka_retry = PM_KAFKA_DEFAULT_RETRY;
+
+    p_kafka_set_broker(kafka_host, list->cfg.pipe_kafka_broker_host, list->cfg.pipe_kafka_broker_port);
+    p_kafka_set_topic(kafka_host, list->cfg.pipe_kafka_topic);
+    p_kafka_set_partition(kafka_host, list->cfg.pipe_kafka_partition);
+    p_kafka_set_key(kafka_host, list->cfg.pipe_kafka_partition_key, list->cfg.pipe_kafka_partition_keylen);
+    p_kafka_set_content_type(kafka_host, PM_KAFKA_CNT_TYPE_BIN);
+    P_broker_timers_set_retry_interval(&kafka_host->btimers, list->cfg.pipe_kafka_retry);
+  }
+  else return ERR;
+
+  return ret;
+}
+
+int plugin_pipe_kafka_connect_to_consume(struct p_kafka_host *kafka_host, struct plugins_list_entry *plugin_data)
+{
+  int ret = SUCCESS;
+
+  if (kafka_host && plugin_data) {
+    ret = plugin_pipe_kafka_init_host(kafka_host, plugin_data, FALSE);
+    if (!ret) ret = p_kafka_manage_consumer(kafka_host, TRUE);
+  }
+  else return ERR;
+
+  return ret;
+}
+#endif 
+
+int plugin_pipe_set_retry_timeout(struct p_broker_timers *btimers, int pipe_fd)
+{
+  if (pipe_fd == ERR) return (P_broker_timers_get_retry_interval(btimers) * 1000);
+  else return LONGLONG_RETRY;
+}
+
+int plugin_pipe_calc_retry_timeout_diff(struct p_broker_timers *btimers, time_t now)
+{
+  int timeout;
+
+  timeout = (((P_broker_timers_get_last_fail(btimers) + P_broker_timers_get_retry_interval(btimers)) - now) * 1000);
+  assert(timeout >= 0);
+
+  return timeout;
+}
+
 void plugin_pipe_amqp_compile_check()
 {
 #ifndef WITH_RABBITMQ
-    Log(LOG_ERR, "ERROR ( %s/%s ): 'plugin_pipe_amqp' requires compiling with --enable-rabbitmq. Exiting ..\n", config.name, config.type);
-    exit_plugin(1);
+  Log(LOG_ERR, "ERROR ( %s/%s ): 'plugin_pipe_amqp' requires compiling with --enable-rabbitmq. Exiting ..\n", config.name, config.type);
+  exit_plugin(1);
 #endif
 }
+
+void plugin_pipe_kafka_compile_check()
+{
+#ifndef WITH_KAFKA
+  Log(LOG_ERR, "ERROR ( %s/%s ): 'plugin_pipe_kafka' requires compiling with --enable-kafka. Exiting ..\n", config.name, config.type);
+  exit_plugin(1);
+#endif
+}
+
+void plugin_pipe_check(struct configuration *cfg)
+{
+  if (!cfg->pipe_amqp && !cfg->pipe_kafka) cfg->pipe_homegrown = TRUE;
+
+  if (cfg->pipe_amqp && cfg->pipe_kafka) {
+    Log(LOG_WARNING, "WARN ( %s/%s ): 'plugin_pipe_amqp' and 'plugin_pipe_kafka' are mutual exclusive: disabling both.\n", cfg->name, cfg->type);
+
+    cfg->pipe_amqp = FALSE;
+    cfg->pipe_kafka = FALSE;
+    cfg->pipe_homegrown = TRUE;
+  }
+}
