@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -19,9 +19,9 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#ifdef WITH_RABBITMQ 
-#include "amqp_common.h"
-#endif
+#define __PLUGIN_COMMON_EXPORT
+#include "plugin_common.h"
+#undef  __PLUGIN_COMMON_EXPORT
 
 #define DEFAULT_CHBUFLEN 4096
 #define DEFAULT_PIPE_SIZE 65535
@@ -29,7 +29,7 @@
 #define WARNING_PIPE_SIZE 16384000 /* 16 Mb */
 #define MAX_FAILS 5 
 #define MAX_SEQNUM 65536 
-#define MAX_RG_COUNT_ERR 3
+#define MAX_RG_COUNT_ERR 3 
 
 struct channels_list_entry;
 typedef void (*pkt_handler) (struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -43,13 +43,16 @@ struct ring {
 };
 
 struct ch_buf_hdr {
+  pid_t core_pid;
+  u_int64_t len;
   u_int32_t seq;
   u_int32_t num;
 };
 
 struct ch_status {
-  u_int8_t wakeup;	/* plugin is polling */ 
+  u_int8_t wakeup;		/* plugin is polling */ 
   u_int32_t backlog;
+  u_int64_t last_buf_off;	/* offset of last committed buffer */
 };
 
 struct sampling {
@@ -81,24 +84,35 @@ struct plugins_list_entry {
   struct plugins_list_entry *next;
 };
 
+#ifdef WITH_RABBITMQ 
+#include "amqp_common.h"
+#endif
+
+#ifdef WITH_KAFKA
+#include "kafka_common.h"
+#endif
+
 struct channels_list_entry {
   pm_cfgreg_t aggregation;
   pm_cfgreg_t aggregation_2;
   u_int64_t buf;	/* buffer base */
   u_int64_t bufptr;	/* buffer current */
-  u_int64_t bufend;	/* buffer end; max 4Gb */
+  u_int64_t bufend;	/* buffer end */
   struct ring rg;	
   struct ch_buf_hdr hdr;
   struct ch_status *status;
   ring_cleaner clean_func;
   u_int8_t request;					/* does the plugin support on-request wakeup ? */
   u_int8_t reprocess;					/* do we need to jump back for packet reprocessing ? */
+  u_int8_t already_reprocessed;				/* loop avoidance for packet reprocessing */
   int datasize;
-  int bufsize;		
+  u_int64_t bufsize;		
   int var_size;
+  int buffer_immediate;
   int same_aggregate;
   pkt_handler phandler[N_PRIMITIVES];
   int pipe;
+  pid_t core_pid;
   pm_id_t tag;						/* post-tagging tag */
   pm_id_t tag2;						/* post-tagging tag2 */
   struct pretag_filter tag_filter; 			/* filter aggregates basing on their tag */
@@ -113,6 +127,13 @@ struct channels_list_entry {
   int amqp_host_reconnect;				/* flag need to reconnect to RabbitMQ server */ 
   void *amqp_host_sleep;				/* pointer to the sleep thread (in case of reconnection) */
 #endif
+#ifdef WITH_KAFKA
+  struct p_kafka_host kafka_host;
+/* XXX Kafka:
+  int kafka_host_reconnect;				// flag need to reconnect to Kafka server
+  void *kafka_host_sleep;				// pointer to the sleep thread (in case of reconnection)
+*/
+#endif
 };
 
 #ifdef WITH_RABBITMQ
@@ -155,7 +176,6 @@ EXT void evaluate_sampling(struct sampling *, pm_counter_t *, pm_counter_t *, pm
 EXT pm_counter_t take_simple_random_skip(pm_counter_t);
 EXT pm_counter_t take_simple_systematic_skip(pm_counter_t);
 #if defined WITH_RABBITMQ
-EXT char *plugin_pipe_amqp_compose_routing_key(char *, char *);
 EXT void plugin_pipe_amqp_init_host(struct p_amqp_host *, struct plugins_list_entry *);
 EXT struct plugin_pipe_amqp_sleeper *plugin_pipe_amqp_sleeper_define(struct p_amqp_host *, int *, struct plugins_list_entry *);
 EXT void plugin_pipe_amqp_sleeper_free(struct plugin_pipe_amqp_sleeper **);
@@ -163,10 +183,19 @@ EXT void plugin_pipe_amqp_sleeper_publish_func(struct plugin_pipe_amqp_sleeper *
 EXT void plugin_pipe_amqp_sleeper_start(struct channels_list_entry *);
 EXT void plugin_pipe_amqp_sleeper_stop(struct channels_list_entry *);
 EXT int plugin_pipe_amqp_connect_to_consume(struct p_amqp_host *, struct plugins_list_entry *);
-EXT int plugin_pipe_amqp_set_poll_timeout(struct p_amqp_host *, int);
-EXT int plugin_pipe_amqp_calc_poll_timeout_diff(struct p_amqp_host *, time_t);
+#endif
+#if defined WITH_KAFKA
+EXT int plugin_pipe_kafka_init_host(struct p_kafka_host *, struct plugins_list_entry *, int);
+EXT int plugin_pipe_kafka_connect_to_consume(struct p_kafka_host *, struct plugins_list_entry *);
 #endif
 EXT void plugin_pipe_amqp_compile_check();
+EXT void plugin_pipe_kafka_compile_check();
+EXT void plugin_pipe_check(struct configuration *);
+EXT int plugin_pipe_set_retry_timeout(struct p_broker_timers *, int);
+EXT int plugin_pipe_calc_retry_timeout_diff(struct p_broker_timers *, time_t);
+
+EXT void handle_plugin_pipe_dyn_strings(char *, int, char *, struct plugins_list_entry *);
+EXT char *plugin_pipe_compose_default_string(struct plugins_list_entry *, char *);
 #undef EXT
 
 #if (defined __PLUGIN_HOOKS_C)
@@ -199,4 +228,8 @@ EXT void mongodb_plugin(int, struct configuration *, void *);
 #ifdef WITH_RABBITMQ
 EXT void amqp_plugin(int, struct configuration *, void *);
 #endif
+
+#ifdef WITH_KAFKA
+EXT void kafka_plugin(int, struct configuration *, void *);
+#endif
 #undef EXT
