@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -31,78 +31,88 @@
 #define PLUGIN_ID_TEE		8
 #define PLUGIN_ID_MONGODB	9
 #define PLUGIN_ID_AMQP		10
+#define PLUGIN_ID_KAFKA		11
 #define PLUGIN_ID_UNKNOWN       -1
 
 /* vars */
-int protocols_number;
+#if (!defined __PMACCTD_C) && (!defined __NFACCTD_C) && (!defined __SFACCTD_C) && (!defined __UACCTD_C) && (!defined __PMTELEMETRYD_C) && (!defined __PMACCT_CLIENT_C) && (!defined __PMBGPD_C) && (!defined __PMBMPD_C)
+#define EXT extern
+#else
+#define EXT
+#endif
+EXT int protocols_number;
+#undef EXT
 
 /* structures */
 static const struct _primitives_matrix_struct _primitives_matrix[] = {
-  /* primitive, pmacctd, uacctd, nfacctd, sfacctd */
-  {"L2", 1, 1, 1, 1, ""}, 
-  {"cos", 1, 0, 1, 1, "Ethernet CoS, 802.1P"},
-  {"etype", 1, 0, 1, 1, "Ethernet Ethertype"},
-  {"src_mac", 1, 1, 1, 1, "Source MAC address"},
-  {"dst_mac", 1, 1, 1, 1, "Destination MAC address"},
-  {"vlan", 1, 1, 1, 1, "Ethernet VLAN, 802.1Q"},
-  {"L3", 1, 1, 1, 1, ""}, 
-  {"src_host", 1, 1, 1, 1, "Source IPv4/IPv6 address"}, 
-  {"dst_host", 1, 1, 1, 1, "Destination IPv4/IPv6 address"},
-  {"src_mask", 1, 1, 1, 1, "Source network mask"},
-  {"dst_mask", 1, 1, 1, 1, "Destination network mask"},
-  {"src_net", 1, 1, 1, 1, "Source IPv4/IPv6 prefix"},
-  {"dst_net", 1, 1, 1, 1, "Destination IPv4/IPv6 prefix"},
-  {"proto", 1, 1, 1, 1, "IP protocol"},
-  {"tos", 1, 1, 1, 1, "IP ToS"},
-  {"L4", 1, 1, 1, 1, ""}, 
-  {"src_port", 1, 1, 1, 1, "Source TCP/UDP port"},
-  {"dst_port", 1, 1, 1, 1, "Destination TCP/UDP port"},
-  {"tcpflags", 1, 1, 1, 1, "TCP flags"},
-  {"BGP", 1, 1, 1, 1, ""}, 
-  {"src_as", 1, 1, 1, 1, "Source ASN"},
-  {"dst_as", 1, 1, 1, 1, "Destination ASN"},
-  {"as_path", 1, 1, 1, 1, "AS PATH"},
-  {"std_comm", 1, 1, 1, 1, "Standard Communities"},
-  {"ext_comm", 1, 1, 1, 1, "Extended Communities"},
-  {"local_pref", 1, 1, 1, 1, "Local Preference"},
-  {"med", 1, 1, 1, 1, "Multi-Exit Discriminator"},
-  {"src_as_path", 1, 1, 1, 1, "Source AS PATH (via reverse BGP lookup)"},
-  {"src_std_comm", 1, 1, 1, 1, "Source Standard Communities (via reverse BGP lookup)"},
-  {"src_ext_comm", 1, 1, 1, 1, "Source Extended Communities (via reverse BGP lookup)"},
-  {"src_local_pref", 1, 1, 1, 1, "Source Local Preference (by default via reverse BGP lookup)"},
-  {"src_med", 1, 1, 1, 1, "Source MED (by default via reverse BGP lookup)"},
-  {"mpls_vpn_rd", 0, 0, 1, 1, "MPLS VPN Route Distinguisher"},
-  {"peer_src_as", 1, 1, 1, 1, "Source peer ASN (by default via reverse BGP lookup)"},
-  {"peer_dst_as", 1, 1, 1, 1, "Destination peer ASN"},
-  {"peer_dst_ip", 1, 1, 1, 1, "BGP next-hop"},
-  {"NAT", 1, 1, 1, 1, ""}, 
-  {"fw_event", 0, 0, 1, 0, "Firewall event ID"},
-  {"nat_event", 0, 0, 1, 0, "NAT event ID"},
-  {"post_nat_src_host", 0, 0, 1, 0, "Source IPv4/IPv6 address after NAT translation"},
-  {"post_nat_dst_host", 0, 0, 1, 0, "Destination IPv4/IPv6 address after NAT translation"},
-  {"post_nat_src_port", 0, 0, 1, 0, "Source TCP/UDP port after NAT translation"},
-  {"post_nat_dst_port", 0, 0, 1, 0, "Destination TCP/UDP port after NAT translation"},
-  {"MPLS", 1, 1, 1, 1, ""}, 
-  {"mpls_label_bottom", 1, 1, 1, 0, "Bottom MPLS label"},
-  {"mpls_label_top", 1, 1, 1, 0, "Top MPLS label"},
-  {"mpls_stack_depth", 1, 1, 1, 0, "MPLS stack depth"},
-  {"MISC", 1, 1, 1, 1, ""}, 
-  {"class", 1, 1, 1, 1, "L7 protocol classification"},
-  {"flows", 0, 0, 1, 0, "IP flows"},
-  {"src_host_country", 1, 1, 1, 1, "Source IP address GeoIP resolution: country"},
-  {"dst_host_country", 1, 1, 1, 1, "Destination IP address GeoIP resolution: country"},
-  {"in_iface", 0, 1, 1, 1, "Input interface, SNMP ifIndex"}, 
-  {"out_iface", 0, 1, 1, 1, "Output interface, SNMP ifIndex"}, 
-  {"peer_src_ip", 0, 0, 1, 1, "IP address or identificator of telemetry exporting device"},
-  {"pkt_len_distrib", 1, 1, 1, 1, "Packet length distribution"},
-  {"sampling_rate", 1, 1, 1, 1, "Sampling rate"},
-  {"tag", 1, 1, 1, 1, "Numeric tag, ie. as result of pre_tag_map evaluation"},
-  {"tag2", 1, 1, 1, 1, "Numeric tag, ie. as result of pre_tag_map evaluation"},
-  {"label", 1, 1, 1, 1, "String label, ie. as result of pre_tag_map evaluation"},
-  {"TIME", 1, 1, 1, 1, ""}, 
-  {"timestamp_start", 1, 1, 1, 1, "Observation time or flow start time"},
-  {"timestamp_end", 0, 0, 1, 0, "Flow end time"},
-  {"", 0, 0, 0, 0, ""}
+  /* primitive, pmacctd, uacctd, nfacctd, sfacctd, pmtelemetryd, pmbgpd, pmbmpd */
+  {"L2", 1, 1, 1, 1, 0, 0, 0, ""}, 
+  {"cos", 1, 0, 1, 1, 0, 0, 0, "Ethernet CoS, 802.1P"},
+  {"etype", 1, 0, 1, 1, 0, 0, 0, "Ethernet Ethertype"},
+  {"src_mac", 1, 1, 1, 1, 0, 0, 0, "Source MAC address"},
+  {"dst_mac", 1, 1, 1, 1, 0, 0, 0, "Destination MAC address"},
+  {"vlan", 1, 1, 1, 1, 0, 0, 0, "Ethernet VLAN, 802.1Q"},
+  {"L3", 1, 1, 1, 1, 0, 0, 0, ""}, 
+  {"src_host", 1, 1, 1, 1, 0, 0, 0, "Source IPv4/IPv6 address"}, 
+  {"dst_host", 1, 1, 1, 1, 0, 0, 0, "Destination IPv4/IPv6 address"},
+  {"src_mask", 1, 1, 1, 1, 0, 0, 0, "Source network mask"},
+  {"dst_mask", 1, 1, 1, 1, 0, 0, 0, "Destination network mask"},
+  {"src_net", 1, 1, 1, 1, 0, 0, 0, "Source IPv4/IPv6 prefix"},
+  {"dst_net", 1, 1, 1, 1, 0, 0, 0, "Destination IPv4/IPv6 prefix"},
+  {"proto", 1, 1, 1, 1, 0, 0, 0, "IP protocol"},
+  {"tos", 1, 1, 1, 1, 0, 0, 0, "IP ToS"},
+  {"L4", 1, 1, 1, 1, 0, 0, 0, ""}, 
+  {"src_port", 1, 1, 1, 1, 0, 0, 0, "Source TCP/UDP port"},
+  {"dst_port", 1, 1, 1, 1, 0, 0, 0, "Destination TCP/UDP port"},
+  {"tcpflags", 1, 1, 1, 1, 0, 0, 0, "TCP flags"},
+  {"BGP", 1, 1, 1, 1, 0, 0, 0, ""}, 
+  {"src_as", 1, 1, 1, 1, 0, 0, 0, "Source ASN"},
+  {"dst_as", 1, 1, 1, 1, 0, 0, 0, "Destination ASN"},
+  {"as_path", 1, 1, 1, 1, 0, 0, 0, "AS PATH"},
+  {"std_comm", 1, 1, 1, 1, 0, 0, 0, "Standard Communities"},
+  {"ext_comm", 1, 1, 1, 1, 0, 0, 0, "Extended Communities"},
+  {"local_pref", 1, 1, 1, 1, 0, 0, 0, "Local Preference"},
+  {"med", 1, 1, 1, 1, 0, 0, 0, "Multi-Exit Discriminator"},
+  {"src_as_path", 1, 1, 1, 1, 0, 0, 0, "Source AS PATH (via reverse BGP lookup)"},
+  {"src_std_comm", 1, 1, 1, 1, 0, 0, 0, "Source Standard Communities (via reverse BGP lookup)"},
+  {"src_ext_comm", 1, 1, 1, 1, 0, 0, 0, "Source Extended Communities (via reverse BGP lookup)"},
+  {"src_local_pref", 1, 1, 1, 1, 0, 0, 0, "Source Local Preference (by default via reverse BGP lookup)"},
+  {"src_med", 1, 1, 1, 1, 0, 0, 0, "Source MED (by default via reverse BGP lookup)"},
+  {"mpls_vpn_rd", 0, 0, 1, 1, 0, 0, 0, "MPLS VPN Route Distinguisher"},
+  {"peer_src_as", 1, 1, 1, 1, 0, 0, 0, "Source peer ASN (by default via reverse BGP lookup)"},
+  {"peer_dst_as", 1, 1, 1, 1, 0, 0, 0, "Destination peer ASN"},
+  {"peer_dst_ip", 1, 1, 1, 1, 0, 0, 0, "BGP next-hop"},
+  {"NAT", 1, 1, 1, 1, 0, 0, 0, ""}, 
+  {"fw_event", 0, 0, 1, 0, 0, 0, 0, "Firewall event ID"},
+  {"nat_event", 0, 0, 1, 0, 0, 0, 0, "NAT event ID"},
+  {"post_nat_src_host", 0, 0, 1, 0, 0, 0, 0, "Source IPv4/IPv6 address after NAT translation"},
+  {"post_nat_dst_host", 0, 0, 1, 0, 0, 0, 0, "Destination IPv4/IPv6 address after NAT translation"},
+  {"post_nat_src_port", 0, 0, 1, 0, 0, 0, 0, "Source TCP/UDP port after NAT translation"},
+  {"post_nat_dst_port", 0, 0, 1, 0, 0, 0, 0, "Destination TCP/UDP port after NAT translation"},
+  {"MPLS", 1, 1, 1, 1, 0, 0, 0, ""}, 
+  {"mpls_label_bottom", 1, 1, 1, 0, 0, 0, 0, "Bottom MPLS label"},
+  {"mpls_label_top", 1, 1, 1, 0, 0, 0, 0, "Top MPLS label"},
+  {"mpls_stack_depth", 1, 1, 1, 0, 0, 0, 0, "MPLS stack depth"},
+  {"MISC", 1, 1, 1, 1, 0, 0, 0, ""}, 
+  {"class", 1, 1, 1, 1, 0, 0, 0, "L7 protocol classification"},
+  {"flows", 0, 0, 1, 0, 0, 0, 0, "IP flows"},
+  {"src_host_country", 1, 1, 1, 1, 0, 0, 0, "Source IP address GeoIP resolution: country"},
+  {"dst_host_country", 1, 1, 1, 1, 0, 0, 0, "Destination IP address GeoIP resolution: country"},
+  {"in_iface", 0, 1, 1, 1, 0, 0, 0, "Input interface, SNMP ifIndex"}, 
+  {"out_iface", 0, 1, 1, 1, 0, 0, 0, "Output interface, SNMP ifIndex"}, 
+  {"peer_src_ip", 0, 0, 1, 1, 0, 0, 0, "IP address or identificator of telemetry exporting device"},
+  {"pkt_len_distrib", 1, 1, 1, 1, 0, 0, 0, "Packet length distribution"},
+  {"sampling_rate", 1, 1, 1, 1, 0, 0, 0, "Sampling rate"},
+  {"tag", 1, 1, 1, 1, 0, 0, 0, "Numeric tag, ie. as result of pre_tag_map evaluation"},
+  {"tag2", 1, 1, 1, 1, 0, 0, 0, "Numeric tag, ie. as result of pre_tag_map evaluation"},
+  {"label", 1, 1, 1, 1, 0, 0, 0, "String label, ie. as result of pre_tag_map evaluation"},
+  {"export_proto_seqno", 0, 0, 1, 1, 0, 0, 0, "Export protocol (ie. NetFlow) sequence number"},
+  {"export_proto_version", 0, 0, 1, 1, 0, 0, 0, "Export protocol (ie. NetFlow) version"},
+  {"TIME", 1, 1, 1, 1, 0, 0, 0, ""}, 
+  {"timestamp_start", 0, 0, 1, 0, 0, 0, 0, "Flow start time or observation time at the exporter"},
+  {"timestamp_end", 0, 0, 1, 0, 0, 0, 0, "Flow end time"},
+  {"timestamp_arrival", 1, 1, 1, 1, 0, 0, 0, "Observation time at the collector"},
+  {"", 0, 0, 0, 0, 0, 0, 0, ""}
 };
 
 static const struct _protocols_struct _protocols[] = {
@@ -298,6 +308,7 @@ static struct _devices_struct _devices[] = {
 #ifdef __CFG_C
 static const struct _dictionary_line dictionary[] = {
   {"debug", cfg_key_debug},
+  {"debug_internal_msg", cfg_key_debug_internal_msg},
   {"syslog", cfg_key_syslog},
   {"logfile", cfg_key_logfile},
   {"pidfile", cfg_key_pidfile},
@@ -319,6 +330,7 @@ static const struct _dictionary_line dictionary[] = {
   {"plugins", NULL},
   {"plugin_pipe_size", cfg_key_plugin_pipe_size},
   {"plugin_pipe_backlog", cfg_key_plugin_pipe_backlog},
+  {"plugin_pipe_check_core_pid", cfg_key_plugin_pipe_check_core_pid},
   {"plugin_pipe_amqp", cfg_key_plugin_pipe_amqp},
   {"plugin_pipe_amqp_user", cfg_key_plugin_pipe_amqp_user},
   {"plugin_pipe_amqp_passwd", cfg_key_plugin_pipe_amqp_passwd},
@@ -327,6 +339,12 @@ static const struct _dictionary_line dictionary[] = {
   {"plugin_pipe_amqp_vhost", cfg_key_plugin_pipe_amqp_vhost},
   {"plugin_pipe_amqp_routing_key", cfg_key_plugin_pipe_amqp_routing_key},
   {"plugin_pipe_amqp_retry", cfg_key_plugin_pipe_amqp_retry},
+  {"plugin_pipe_kafka", cfg_key_plugin_pipe_kafka},
+  {"plugin_pipe_kafka_broker_host", cfg_key_plugin_pipe_kafka_broker_host},
+  {"plugin_pipe_kafka_broker_port", cfg_key_plugin_pipe_kafka_broker_port},
+  {"plugin_pipe_kafka_topic", cfg_key_plugin_pipe_kafka_topic},
+  {"plugin_pipe_kafka_partition", cfg_key_plugin_pipe_kafka_partition},
+  {"plugin_pipe_kafka_retry", cfg_key_plugin_pipe_kafka_retry},
   {"plugin_buffer_size", cfg_key_plugin_buffer_size},
   {"interface", cfg_key_interface},
   {"interface_wait", cfg_key_interface_wait},
@@ -341,7 +359,7 @@ static const struct _dictionary_line dictionary[] = {
   {"networks_cache_entries", cfg_key_networks_cache_entries},
   {"ports_file", cfg_key_ports_file},
   {"timestamps_secs", cfg_key_timestamps_secs},
-  {"timestamps_since_epoch", cfg_key_sql_history_since_epoch},
+  {"timestamps_since_epoch", cfg_key_timestamps_since_epoch},
   {"imt_path", cfg_key_imt_path},
   {"imt_passwd", cfg_key_imt_passwd},
   {"imt_buckets", cfg_key_imt_buckets},
@@ -363,9 +381,8 @@ static const struct _dictionary_line dictionary[] = {
   {"sql_history", cfg_key_sql_history},
   {"sql_history_offset", cfg_key_sql_history_offset},
   {"sql_history_roundoff", cfg_key_sql_history_roundoff},
-  {"sql_history_since_epoch", cfg_key_sql_history_since_epoch},
+  {"sql_history_since_epoch", cfg_key_timestamps_since_epoch}, // legacy
   {"sql_recovery_backup_host", cfg_key_sql_recovery_backup_host},
-  {"sql_recovery_logfile", cfg_key_sql_recovery_logfile},
   {"sql_delimiter", cfg_key_sql_delimiter},
   {"sql_max_writers", cfg_key_sql_max_writers},
   {"sql_trigger_exec", cfg_key_sql_trigger_exec},
@@ -386,6 +403,7 @@ static const struct _dictionary_line dictionary[] = {
   {"print_output", cfg_key_print_output},
   {"print_output_file", cfg_key_print_output_file},
   {"print_output_file_append", cfg_key_print_output_file_append},
+  {"print_output_lock_file", cfg_key_print_output_lock_file},
   {"print_output_separator", cfg_key_print_output_separator},
   {"print_latest_file", cfg_key_print_latest_file},
   {"print_num_protos", cfg_key_num_protos},
@@ -416,6 +434,8 @@ static const struct _dictionary_line dictionary[] = {
   {"mongo_preprocess_type", cfg_key_sql_preprocess_type},
   {"mongo_startup_delay", cfg_key_sql_startup_delay},
   {"mongo_num_protos", cfg_key_num_protos},
+  {"avro_buffer_size", cfg_key_avro_buffer_size},
+  {"avro_schema_output_file", cfg_key_avro_schema_output_file},
   {"amqp_refresh_time", cfg_key_sql_refresh_time},
   {"amqp_history", cfg_key_sql_history},
   {"amqp_history_offset", cfg_key_sql_history_offset},
@@ -439,6 +459,27 @@ static const struct _dictionary_line dictionary[] = {
   {"amqp_multi_values", cfg_key_sql_multi_values},
   {"amqp_num_protos", cfg_key_num_protos},
   {"amqp_vhost", cfg_key_amqp_vhost},
+  {"amqp_markers", cfg_key_print_markers},
+  {"amqp_output", cfg_key_message_broker_output},
+  {"kafka_refresh_time", cfg_key_sql_refresh_time},
+  {"kafka_history", cfg_key_sql_history},
+  {"kafka_history_offset", cfg_key_sql_history_offset},
+  {"kafka_history_roundoff", cfg_key_sql_history_roundoff},
+  {"kafka_broker_host", cfg_key_sql_host},
+  {"kafka_broker_port", cfg_key_kafka_broker_port},
+  {"kafka_topic", cfg_key_sql_table},
+  {"kafka_topic_rr", cfg_key_amqp_routing_key_rr},
+  {"kafka_partition", cfg_key_kafka_partition},
+  {"kafka_partition_key", cfg_key_kafka_partition_key},
+  {"kafka_cache_entries", cfg_key_print_cache_entries},
+  {"kafka_max_writers", cfg_key_sql_max_writers},
+  {"kafka_preprocess", cfg_key_sql_preprocess},
+  {"kafka_preprocess_type", cfg_key_sql_preprocess_type},
+  {"kafka_startup_delay", cfg_key_sql_startup_delay},
+  {"kafka_multi_values", cfg_key_sql_multi_values},
+  {"kafka_num_protos", cfg_key_num_protos},
+  {"kafka_markers", cfg_key_print_markers},
+  {"kafka_output", cfg_key_message_broker_output},
   {"nfacctd_proc_name", cfg_key_proc_name},
   {"nfacctd_port", cfg_key_nfacctd_port},
   {"nfacctd_ip", cfg_key_nfacctd_ip},
@@ -446,6 +487,7 @@ static const struct _dictionary_line dictionary[] = {
   {"nfacctd_time_secs", cfg_key_nfacctd_time_secs},
   {"nfacctd_time_new", cfg_key_nfacctd_time_new},
   {"nfacctd_as_new", cfg_key_nfacctd_as_new},
+  {"nfacctd_as", cfg_key_nfacctd_as_new},
   {"nfacctd_net", cfg_key_nfacctd_net},
   {"nfacctd_mcast_groups", cfg_key_nfacctd_mcast_groups},
   {"nfacctd_peer_as", cfg_key_nfprobe_peer_as},
@@ -468,6 +510,7 @@ static const struct _dictionary_line dictionary[] = {
   {"pmacctd_pipe_size", cfg_key_nfacctd_pipe_size},
   {"pmacctd_stitching", cfg_key_nfacctd_stitching},
   {"pmacctd_renormalize", cfg_key_sfacctd_renormalize},
+  {"pmacctd_nonroot", cfg_key_pmacctd_nonroot},
   {"uacctd_proc_name", cfg_key_proc_name},
   {"uacctd_force_frag_handling", cfg_key_pmacctd_force_frag_handling},
   {"uacctd_frag_buffer_size", cfg_key_pmacctd_frag_buffer_size},
@@ -479,6 +522,58 @@ static const struct _dictionary_line dictionary[] = {
   {"uacctd_ext_sampling_rate", cfg_key_pmacctd_ext_sampling_rate},
   {"uacctd_stitching", cfg_key_nfacctd_stitching},
   {"uacctd_renormalize", cfg_key_sfacctd_renormalize},
+  {"telemetry_daemon", cfg_key_telemetry_daemon},
+  {"telemetry_daemon_port_tcp", cfg_key_telemetry_port_tcp},
+  {"telemetry_daemon_port_udp", cfg_key_telemetry_port_udp},
+  {"telemetry_daemon_ip", cfg_key_telemetry_ip},
+  {"telemetry_daemon_decoder", cfg_key_telemetry_decoder},
+  {"telemetry_daemon_max_peers", cfg_key_telemetry_max_peers},
+  {"telemetry_daemon_udp_timeout", cfg_key_telemetry_udp_timeout},
+  {"telemetry_daemon_allow_file", cfg_key_telemetry_allow_file},
+  {"telemetry_daemon_pipe_size", cfg_key_telemetry_pipe_size},
+  {"telemetry_daemon_ipprec", cfg_key_telemetry_ip_precedence},
+  {"telemetry_daemon_msglog_output", cfg_key_telemetry_msglog_output},
+  {"telemetry_daemon_msglog_file", cfg_key_telemetry_msglog_file},
+  {"telemetry_daemon_msglog_amqp_host", cfg_key_telemetry_msglog_amqp_host},
+  {"telemetry_daemon_msglog_amqp_vhost", cfg_key_telemetry_msglog_amqp_vhost},
+  {"telemetry_daemon_msglog_amqp_user", cfg_key_telemetry_msglog_amqp_user},
+  {"telemetry_daemon_msglog_amqp_passwd", cfg_key_telemetry_msglog_amqp_passwd},
+  {"telemetry_daemon_msglog_amqp_exchange", cfg_key_telemetry_msglog_amqp_exchange},
+  {"telemetry_daemon_msglog_amqp_exchange_type", cfg_key_telemetry_msglog_amqp_exchange_type},
+  {"telemetry_daemon_msglog_amqp_routing_key", cfg_key_telemetry_msglog_amqp_routing_key},
+  {"telemetry_daemon_msglog_amqp_routing_key_rr", cfg_key_telemetry_msglog_amqp_routing_key_rr},
+  {"telemetry_daemon_msglog_amqp_persistent_msg", cfg_key_telemetry_msglog_amqp_persistent_msg},
+  {"telemetry_daemon_msglog_amqp_frame_max", cfg_key_telemetry_msglog_amqp_frame_max},
+  {"telemetry_daemon_msglog_amqp_heartbeat_interval", cfg_key_telemetry_msglog_amqp_heartbeat_interval},
+  {"telemetry_daemon_msglog_amqp_retry", cfg_key_telemetry_msglog_amqp_retry},
+  {"telemetry_daemon_msglog_kafka_broker_host", cfg_key_telemetry_msglog_kafka_broker_host},
+  {"telemetry_daemon_msglog_kafka_broker_port", cfg_key_telemetry_msglog_kafka_broker_port},
+  {"telemetry_daemon_msglog_kafka_topic", cfg_key_telemetry_msglog_kafka_topic},
+  {"telemetry_daemon_msglog_kafka_topic_rr", cfg_key_telemetry_msglog_kafka_topic_rr},
+  {"telemetry_daemon_msglog_kafka_partition", cfg_key_telemetry_msglog_kafka_partition},
+  {"telemetry_daemon_msglog_kafka_partition_key", cfg_key_telemetry_msglog_kafka_partition_key},
+  {"telemetry_daemon_msglog_kafka_retry", cfg_key_telemetry_msglog_kafka_retry},
+  {"telemetry_dump_output", cfg_key_telemetry_dump_output},
+  {"telemetry_dump_file", cfg_key_telemetry_dump_file},
+  {"telemetry_dump_latest_file", cfg_key_telemetry_dump_latest_file},
+  {"telemetry_dump_refresh_time", cfg_key_telemetry_dump_refresh_time},
+  {"telemetry_dump_amqp_host", cfg_key_telemetry_dump_amqp_host},
+  {"telemetry_dump_amqp_vhost", cfg_key_telemetry_dump_amqp_vhost},
+  {"telemetry_dump_amqp_user", cfg_key_telemetry_dump_amqp_user},
+  {"telemetry_dump_amqp_passwd", cfg_key_telemetry_dump_amqp_passwd},
+  {"telemetry_dump_amqp_exchange", cfg_key_telemetry_dump_amqp_exchange},
+  {"telemetry_dump_amqp_exchange_type", cfg_key_telemetry_dump_amqp_exchange_type},
+  {"telemetry_dump_amqp_routing_key", cfg_key_telemetry_dump_amqp_routing_key},
+  {"telemetry_dump_amqp_routing_key_rr", cfg_key_telemetry_dump_amqp_routing_key_rr},
+  {"telemetry_dump_amqp_persistent_msg", cfg_key_telemetry_dump_amqp_persistent_msg},
+  {"telemetry_dump_amqp_frame_max", cfg_key_telemetry_dump_amqp_frame_max},
+  {"telemetry_dump_amqp_heartbeat_interval", cfg_key_telemetry_dump_amqp_heartbeat_interval},
+  {"telemetry_dump_kafka_broker_host", cfg_key_telemetry_dump_kafka_broker_host},
+  {"telemetry_dump_kafka_broker_port", cfg_key_telemetry_dump_kafka_broker_port},
+  {"telemetry_dump_kafka_topic", cfg_key_telemetry_dump_kafka_topic},
+  {"telemetry_dump_kafka_topic_rr", cfg_key_telemetry_dump_kafka_topic_rr},
+  {"telemetry_dump_kafka_partition", cfg_key_telemetry_dump_kafka_partition},
+  {"telemetry_dump_kafka_partition_key", cfg_key_telemetry_dump_kafka_partition_key},
   {"pcap_savefile", cfg_key_pcap_savefile},
   {"refresh_maps", cfg_key_maps_refresh}, // legacy
   {"maps_refresh", cfg_key_maps_refresh},
@@ -499,6 +594,7 @@ static const struct _dictionary_line dictionary[] = {
   {"sfacctd_ip", cfg_key_nfacctd_ip},
   {"sfacctd_allow_file", cfg_key_nfacctd_allow_file},
   {"sfacctd_as_new", cfg_key_nfacctd_as_new},
+  {"sfacctd_as", cfg_key_nfacctd_as_new},
   {"sfacctd_net", cfg_key_nfacctd_net},
   {"sfacctd_peer_as", cfg_key_nfprobe_peer_as},
   {"sfacctd_pipe_size", cfg_key_nfacctd_pipe_size},
@@ -509,6 +605,23 @@ static const struct _dictionary_line dictionary[] = {
   {"sfacctd_ext_sampling_rate", cfg_key_pmacctd_ext_sampling_rate},
   {"sfacctd_counter_output", cfg_key_sfacctd_counter_output},
   {"sfacctd_counter_file", cfg_key_sfacctd_counter_file},
+  {"sfacctd_counter_amqp_host", cfg_key_sfacctd_counter_amqp_host},
+  {"sfacctd_counter_amqp_vhost", cfg_key_sfacctd_counter_amqp_vhost},
+  {"sfacctd_counter_amqp_user", cfg_key_sfacctd_counter_amqp_user},
+  {"sfacctd_counter_amqp_passwd", cfg_key_sfacctd_counter_amqp_passwd},
+  {"sfacctd_counter_amqp_exchange", cfg_key_sfacctd_counter_amqp_exchange},
+  {"sfacctd_counter_amqp_exchange_type", cfg_key_sfacctd_counter_amqp_exchange_type},
+  {"sfacctd_counter_amqp_routing_key", cfg_key_sfacctd_counter_amqp_routing_key},
+  {"sfacctd_counter_amqp_persistent_msg", cfg_key_sfacctd_counter_amqp_persistent_msg},
+  {"sfacctd_counter_amqp_frame_max", cfg_key_sfacctd_counter_amqp_frame_max},
+  {"sfacctd_counter_amqp_heartbeat_interval", cfg_key_sfacctd_counter_amqp_heartbeat_interval},
+  {"sfacctd_counter_amqp_retry", cfg_key_sfacctd_counter_amqp_retry},
+  {"sfacctd_counter_kafka_broker_host", cfg_key_sfacctd_counter_kafka_broker_host},
+  {"sfacctd_counter_kafka_broker_port", cfg_key_sfacctd_counter_kafka_broker_port},
+  {"sfacctd_counter_kafka_topic", cfg_key_sfacctd_counter_kafka_topic},
+  {"sfacctd_counter_kafka_partition", cfg_key_sfacctd_counter_kafka_partition},
+  {"sfacctd_counter_kafka_partition_key", cfg_key_sfacctd_counter_kafka_partition_key},
+  {"sfacctd_counter_kafka_retry", cfg_key_sfacctd_counter_kafka_retry},
   {"classifiers", cfg_key_classifiers},
   {"classifier_tentatives", cfg_key_classifier_tentatives},
   {"classifier_table_num", cfg_key_classifier_table_num},
@@ -559,6 +672,13 @@ static const struct _dictionary_line dictionary[] = {
   {"bgp_daemon_msglog_amqp_frame_max", cfg_key_nfacctd_bgp_msglog_amqp_frame_max},
   {"bgp_daemon_msglog_amqp_heartbeat_interval", cfg_key_nfacctd_bgp_msglog_amqp_heartbeat_interval},
   {"bgp_daemon_msglog_amqp_retry", cfg_key_nfacctd_bgp_msglog_amqp_retry},
+  {"bgp_daemon_msglog_kafka_broker_host", cfg_key_nfacctd_bgp_msglog_kafka_broker_host},
+  {"bgp_daemon_msglog_kafka_broker_port", cfg_key_nfacctd_bgp_msglog_kafka_broker_port},
+  {"bgp_daemon_msglog_kafka_topic", cfg_key_nfacctd_bgp_msglog_kafka_topic},
+  {"bgp_daemon_msglog_kafka_topic_rr", cfg_key_nfacctd_bgp_msglog_kafka_topic_rr},
+  {"bgp_daemon_msglog_kafka_partition", cfg_key_nfacctd_bgp_msglog_kafka_partition},
+  {"bgp_daemon_msglog_kafka_partition_key", cfg_key_nfacctd_bgp_msglog_kafka_partition_key},
+  {"bgp_daemon_msglog_kafka_retry", cfg_key_nfacctd_bgp_msglog_kafka_retry},
   {"bgp_daemon_allow_file", cfg_key_nfacctd_bgp_allow_file},
   {"bgp_daemon_ipprec", cfg_key_nfacctd_bgp_ip_precedence},
   {"bgp_daemon_md5_file", cfg_key_nfacctd_bgp_md5_file},
@@ -581,6 +701,7 @@ static const struct _dictionary_line dictionary[] = {
   {"bgp_agent_map", cfg_key_nfacctd_bgp_to_agent_map},
   {"bgp_follow_default", cfg_key_nfacctd_bgp_follow_default},
   {"bgp_follow_nexthop", cfg_key_nfacctd_bgp_follow_nexthop},
+  {"bgp_follow_nexthop_external", cfg_key_nfacctd_bgp_follow_nexthop_external},
   {"bgp_neighbors_file", cfg_key_nfacctd_bgp_neighbors_file},
   {"bgp_table_peer_buckets", cfg_key_nfacctd_bgp_table_peer_buckets},
   {"bgp_table_per_peer_buckets", cfg_key_nfacctd_bgp_table_per_peer_buckets},
@@ -588,6 +709,7 @@ static const struct _dictionary_line dictionary[] = {
   {"bgp_table_per_peer_hash", cfg_key_nfacctd_bgp_table_per_peer_hash},
   {"bgp_table_dump_output", cfg_key_nfacctd_bgp_table_dump_output},
   {"bgp_table_dump_file", cfg_key_nfacctd_bgp_table_dump_file},
+  {"bgp_table_dump_latest_file", cfg_key_nfacctd_bgp_table_dump_latest_file},
   {"bgp_table_dump_refresh_time", cfg_key_nfacctd_bgp_table_dump_refresh_time},
   {"bgp_table_dump_amqp_host", cfg_key_nfacctd_bgp_table_dump_amqp_host},
   {"bgp_table_dump_amqp_vhost", cfg_key_nfacctd_bgp_table_dump_amqp_vhost},
@@ -600,6 +722,12 @@ static const struct _dictionary_line dictionary[] = {
   {"bgp_table_dump_amqp_persistent_msg", cfg_key_nfacctd_bgp_table_dump_amqp_persistent_msg},
   {"bgp_table_dump_amqp_frame_max", cfg_key_nfacctd_bgp_table_dump_amqp_frame_max},
   {"bgp_table_dump_amqp_heartbeat_interval", cfg_key_nfacctd_bgp_table_dump_amqp_heartbeat_interval},
+  {"bgp_table_dump_kafka_broker_host", cfg_key_nfacctd_bgp_table_dump_kafka_broker_host},
+  {"bgp_table_dump_kafka_broker_port", cfg_key_nfacctd_bgp_table_dump_kafka_broker_port},
+  {"bgp_table_dump_kafka_topic", cfg_key_nfacctd_bgp_table_dump_kafka_topic},
+  {"bgp_table_dump_kafka_topic_rr", cfg_key_nfacctd_bgp_table_dump_kafka_topic_rr},
+  {"bgp_table_dump_kafka_partition", cfg_key_nfacctd_bgp_table_dump_kafka_partition},
+  {"bgp_table_dump_kafka_partition_key", cfg_key_nfacctd_bgp_table_dump_kafka_partition_key},
   {"bmp_daemon", cfg_key_nfacctd_bmp},
   {"bmp_daemon_ip", cfg_key_nfacctd_bmp_ip},
   {"bmp_daemon_port", cfg_key_nfacctd_bmp_port},
@@ -623,13 +751,20 @@ static const struct _dictionary_line dictionary[] = {
   {"bmp_daemon_msglog_amqp_frame_max", cfg_key_nfacctd_bmp_msglog_amqp_frame_max},
   {"bmp_daemon_msglog_amqp_heartbeat_interval", cfg_key_nfacctd_bmp_msglog_amqp_heartbeat_interval},
   {"bmp_daemon_msglog_amqp_retry", cfg_key_nfacctd_bmp_msglog_amqp_retry},
-  {"bmp_neighbors_file", cfg_key_nfacctd_bmp_neighbors_file},
+  {"bmp_daemon_msglog_kafka_broker_host", cfg_key_nfacctd_bmp_msglog_kafka_broker_host},
+  {"bmp_daemon_msglog_kafka_broker_port", cfg_key_nfacctd_bmp_msglog_kafka_broker_port},
+  {"bmp_daemon_msglog_kafka_topic", cfg_key_nfacctd_bmp_msglog_kafka_topic},
+  {"bmp_daemon_msglog_kafka_topic_rr", cfg_key_nfacctd_bmp_msglog_kafka_topic_rr},
+  {"bmp_daemon_msglog_kafka_partition", cfg_key_nfacctd_bmp_msglog_kafka_partition},
+  {"bmp_daemon_msglog_kafka_partition_key", cfg_key_nfacctd_bmp_msglog_kafka_partition_key},
+  {"bmp_daemon_msglog_kafka_retry", cfg_key_nfacctd_bmp_msglog_kafka_retry},
   {"bmp_table_peer_buckets", cfg_key_nfacctd_bmp_table_peer_buckets},
   {"bmp_table_per_peer_buckets", cfg_key_nfacctd_bmp_table_per_peer_buckets},
   {"bmp_table_attr_hash_buckets", cfg_key_nfacctd_bmp_table_attr_hash_buckets},
   {"bmp_table_per_peer_hash", cfg_key_nfacctd_bmp_table_per_peer_hash},
   {"bmp_dump_output", cfg_key_nfacctd_bmp_dump_output},
   {"bmp_dump_file", cfg_key_nfacctd_bmp_dump_file},
+  {"bmp_dump_latest_file", cfg_key_nfacctd_bmp_dump_latest_file},
   {"bmp_dump_refresh_time", cfg_key_nfacctd_bmp_dump_refresh_time},
   {"bmp_dump_amqp_host", cfg_key_nfacctd_bmp_dump_amqp_host},
   {"bmp_dump_amqp_vhost", cfg_key_nfacctd_bmp_dump_amqp_vhost},
@@ -642,6 +777,12 @@ static const struct _dictionary_line dictionary[] = {
   {"bmp_dump_amqp_persistent_msg", cfg_key_nfacctd_bmp_dump_amqp_persistent_msg},
   {"bmp_dump_amqp_frame_max", cfg_key_nfacctd_bmp_dump_amqp_frame_max},
   {"bmp_dump_amqp_heartbeat_interval", cfg_key_nfacctd_bmp_dump_amqp_heartbeat_interval},
+  {"bmp_dump_kafka_broker_host", cfg_key_nfacctd_bmp_dump_kafka_broker_host},
+  {"bmp_dump_kafka_broker_port", cfg_key_nfacctd_bmp_dump_kafka_broker_port},
+  {"bmp_dump_kafka_topic", cfg_key_nfacctd_bmp_dump_kafka_topic},
+  {"bmp_dump_kafka_topic_rr", cfg_key_nfacctd_bmp_dump_kafka_topic_rr},
+  {"bmp_dump_kafka_partition", cfg_key_nfacctd_bmp_dump_kafka_partition},
+  {"bmp_dump_kafka_partition_key", cfg_key_nfacctd_bmp_dump_kafka_partition_key},
   {"flow_to_rd_map", cfg_key_nfacctd_flow_to_rd_map},
   {"isis_daemon", cfg_key_nfacctd_isis},
   {"isis_daemon_ip", cfg_key_nfacctd_isis_ip},
@@ -663,9 +804,12 @@ static const struct _dictionary_line dictionary[] = {
 #endif
   {"uacctd_group", cfg_key_uacctd_group},
   {"uacctd_nl_size", cfg_key_uacctd_nl_size},
+  {"uacctd_threshold", cfg_key_uacctd_threshold},
   {"tunnel_0", cfg_key_tunnel_0},
   {"pkt_len_distrib_bins", cfg_key_pkt_len_distrib_bins},
   {"tmp_net_own_field", cfg_key_tmp_net_own_field},
+  {"tmp_asa_bi_flow", cfg_key_tmp_asa_bi_flow},
+  {"tmp_comms_same_field", cfg_key_tmp_comms_same_field},
   {"", NULL},
 };
 
@@ -689,6 +833,9 @@ static struct plugin_type_entry plugin_types_list[] = {
 #endif
 #ifdef WITH_RABBITMQ
   {PLUGIN_ID_AMQP,	"amqp",		amqp_plugin},
+#endif
+#ifdef WITH_KAFKA
+  {PLUGIN_ID_KAFKA,     "kafka",        kafka_plugin},
 #endif
   {PLUGIN_ID_TEE,	"tee",		tee_plugin},
   {PLUGIN_ID_UNKNOWN,	"",		NULL},
