@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -52,7 +52,6 @@ EXT int PT_map_local_pref_handler(char *, struct id_entry *, char *, struct plug
 EXT int PT_map_src_comms_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 EXT int PT_map_comms_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 EXT int PT_map_mpls_vpn_rd_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
-EXT int PT_map_mpls_pw_id_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 EXT int PT_map_src_mac_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 EXT int PT_map_dst_mac_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 EXT int PT_map_vlan_id_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
@@ -65,36 +64,34 @@ EXT int PT_map_stack_handler(char *, struct id_entry *, char *, struct plugin_re
 EXT pm_id_t PT_stack_sum(pm_id_t, pm_id_t);
 EXT pm_id_t PT_stack_logical_or(pm_id_t, pm_id_t);
 
-EXT int PT_map_index_entries_ip_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_input_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_output_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_bgp_nexthop_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_src_as_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_dst_as_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_peer_src_as_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_peer_dst_as_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_mpls_vpn_rd_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_mpls_pw_id_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_mpls_label_bottom_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_src_mac_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_dst_mac_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_vlan_id_handler(struct id_entry *, void *); 
-EXT int PT_map_index_entries_cvlan_id_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_ip_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_input_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_output_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_bgp_nexthop_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_src_as_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_dst_as_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_peer_src_as_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_peer_dst_as_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_mpls_vpn_rd_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_mpls_pw_id_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_mpls_label_bottom_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_src_mac_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_dst_mac_handler(struct id_entry *, void *); 
-EXT int PT_map_index_fdata_vlan_id_handler(struct id_entry *, void *);
-EXT int PT_map_index_fdata_cvlan_id_handler(struct id_entry *, void *);
+EXT int PT_map_index_entries_ip_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_input_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_output_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_bgp_nexthop_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_src_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_dst_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_peer_src_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_peer_dst_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_mpls_vpn_rd_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_mpls_label_bottom_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_src_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_dst_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_vlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_entries_cvlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_ip_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_input_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_output_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_bgp_nexthop_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_src_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_dst_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_peer_src_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_peer_dst_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_mpls_vpn_rd_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_mpls_label_bottom_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_src_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_dst_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+EXT int PT_map_index_fdata_vlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *);
+EXT int PT_map_index_fdata_cvlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *);
 
 /* BPAS_*: bgp_peer_as_src map specific handlers */
 EXT int BPAS_map_bgp_nexthop_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
@@ -142,7 +139,6 @@ EXT int pretag_sample_type_handler(struct packet_ptrs *, void *, void *);
 EXT int pretag_sampling_rate_handler(struct packet_ptrs *, void *, void *);
 EXT int pretag_direction_handler(struct packet_ptrs *, void *, void *);
 EXT int pretag_mpls_vpn_rd_handler(struct packet_ptrs *, void *, void *);
-EXT int pretag_mpls_pw_id_handler(struct packet_ptrs *, void *, void *);
 EXT int pretag_src_mac_handler(struct packet_ptrs *, void *, void *);
 EXT int pretag_dst_mac_handler(struct packet_ptrs *, void *, void *);
 EXT int pretag_vlan_id_handler(struct packet_ptrs *, void *, void *);
@@ -162,7 +158,7 @@ EXT int SF_pretag_src_mac_handler(struct packet_ptrs *, void *, void *);
 EXT int SF_pretag_dst_mac_handler(struct packet_ptrs *, void *, void *);
 EXT int SF_pretag_vlan_id_handler(struct packet_ptrs *, void *, void *);
 EXT int SF_pretag_sample_type_handler(struct packet_ptrs *, void *, void *);
-EXT int SF_pretag_mpls_pw_id_handler(struct packet_ptrs *, void *, void *);
+EXT int SF_pretag_direction_handler(struct packet_ptrs *, void *, void *);
 
 EXT int PM_pretag_src_as_handler(struct packet_ptrs *, void *, void *);
 EXT int PM_pretag_dst_as_handler(struct packet_ptrs *, void *, void *);
