@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -75,6 +75,7 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
   unsigned int idx;
   struct pkt_data dummy;
   struct pkt_bgp_primitives dummy_pbgp;
+  struct pkt_legacy_bgp_primitives dummy_plbgp;
   struct pkt_nat_primitives dummy_pnat;
   struct pkt_mpls_primitives dummy_pmpls;
   char *dummy_pcust = NULL, *custbuf = NULL;
@@ -91,6 +92,7 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
 
   memset(&dummy, 0, sizeof(struct pkt_data));
   memset(&dummy_pbgp, 0, sizeof(struct pkt_bgp_primitives));
+  memset(&dummy_plbgp, 0, sizeof(struct pkt_legacy_bgp_primitives));
   memset(&dummy_pnat, 0, sizeof(struct pkt_nat_primitives));
   memset(&dummy_pmpls, 0, sizeof(struct pkt_mpls_primitives));
   memset(dummy_pcust, 0, config.cpptrs.len); 
@@ -117,6 +119,11 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
     offset += sizeof(struct pkt_bgp_primitives);
   }
   else q->extras.off_pkt_bgp_primitives = 0;
+  if (extras->off_pkt_lbgp_primitives) {
+    q->extras.off_pkt_lbgp_primitives = offset;
+    offset += sizeof(struct pkt_legacy_bgp_primitives);
+  }
+  else q->extras.off_pkt_lbgp_primitives = 0;
   if (extras->off_pkt_nat_primitives) {
     q->extras.off_pkt_nat_primitives = offset;
     offset += sizeof(struct pkt_nat_primitives);
@@ -158,15 +165,18 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
       if (!test_zero_elem(acc_elem)) {
 	enQueue_elem(sd, &rb, acc_elem, PdataSz, datasize);
 
-	/* XXX: to be optimized ? */
-	if (extras->off_pkt_bgp_primitives) {
-	  if (acc_elem->cbgp) {
-	    struct pkt_bgp_primitives tmp_pbgp;
+        if (extras->off_pkt_bgp_primitives && acc_elem->pbgp) {
+          enQueue_elem(sd, &rb, acc_elem->pbgp, PbgpSz, datasize - extras->off_pkt_bgp_primitives);
+        }
 
-	    cache_to_pkt_bgp_primitives(&tmp_pbgp, acc_elem->cbgp);
-	    enQueue_elem(sd, &rb, &tmp_pbgp, PbgpSz, datasize - extras->off_pkt_bgp_primitives);
-	  }
-	}
+        if (extras->off_pkt_lbgp_primitives) {
+          if (acc_elem->clbgp) {
+            struct pkt_legacy_bgp_primitives tmp_plbgp;
+
+            cache_to_pkt_legacy_bgp_primitives(&tmp_plbgp, acc_elem->clbgp);
+            enQueue_elem(sd, &rb, &tmp_plbgp, PlbgpSz, datasize - extras->off_pkt_lbgp_primitives);
+          }
+        }
 
         if (extras->off_pkt_nat_primitives && acc_elem->pnat) {
           enQueue_elem(sd, &rb, acc_elem->pnat, PnatSz, datasize - extras->off_pkt_nat_primitives);
@@ -235,6 +245,7 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
 	memcpy(&pd_dummy.primitives, &request.data, sizeof(struct pkt_primitives));
 	prim_ptrs.data = &pd_dummy;
 	prim_ptrs.pbgp = &request.pbgp;
+	prim_ptrs.plbgp = &request.plbgp;
 	prim_ptrs.pnat = &request.pnat;
 	prim_ptrs.pmpls = &request.pmpls;
 	prim_ptrs.pcust = request.pcust;
@@ -245,15 +256,18 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
 	  if (!test_zero_elem(acc_elem)) {
 	    enQueue_elem(sd, &rb, acc_elem, PdataSz, datasize);
 
-	    /* XXX: to be optimized ? */
-	    if (extras->off_pkt_bgp_primitives) {
-	      if (acc_elem->cbgp) {
-		struct pkt_bgp_primitives tmp_pbgp;
+            if (extras->off_pkt_bgp_primitives && acc_elem->pbgp) {
+              enQueue_elem(sd, &rb, acc_elem->pbgp, PbgpSz, datasize - extras->off_pkt_bgp_primitives);
+            }
 
-		cache_to_pkt_bgp_primitives(&tmp_pbgp, acc_elem->cbgp);
-		enQueue_elem(sd, &rb, &tmp_pbgp, PbgpSz, datasize - extras->off_pkt_bgp_primitives);
-	      }
-	    }
+            if (extras->off_pkt_lbgp_primitives) {
+              if (acc_elem->clbgp) {
+                struct pkt_legacy_bgp_primitives tmp_plbgp;
+
+                cache_to_pkt_legacy_bgp_primitives(&tmp_plbgp, acc_elem->clbgp);
+                enQueue_elem(sd, &rb, &tmp_plbgp, PlbgpSz, datasize - extras->off_pkt_lbgp_primitives);
+              }
+            }
 
             if (extras->off_pkt_nat_primitives && acc_elem->pnat) {
               enQueue_elem(sd, &rb, acc_elem->pnat, PnatSz, datasize - extras->off_pkt_nat_primitives);
@@ -283,6 +297,9 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
 	      if (extras->off_pkt_bgp_primitives)
 		enQueue_elem(sd, &rb, &dummy_pbgp, PbgpSz, datasize - extras->off_pkt_bgp_primitives);
 
+              if (extras->off_pkt_lbgp_primitives)
+                enQueue_elem(sd, &rb, &dummy_plbgp, PlbgpSz, datasize - extras->off_pkt_lbgp_primitives);
+
 	      if (extras->off_pkt_nat_primitives)
 		enQueue_elem(sd, &rb, &dummy_pnat, PnatSz, datasize - extras->off_pkt_nat_primitives);
 
@@ -304,6 +321,9 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
 	    if (extras->off_pkt_bgp_primitives)
 	      enQueue_elem(sd, &rb, &dummy_pbgp, PbgpSz, datasize - extras->off_pkt_bgp_primitives);
 
+	    if (extras->off_pkt_lbgp_primitives)
+	      enQueue_elem(sd, &rb, &dummy_plbgp, PlbgpSz, datasize - extras->off_pkt_lbgp_primitives);
+
 	    if (extras->off_pkt_nat_primitives)
 	      enQueue_elem(sd, &rb, &dummy_pnat, PnatSz, datasize - extras->off_pkt_nat_primitives);
 
@@ -321,6 +341,7 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
       else {
         struct pkt_primitives tbuf;  
 	struct pkt_bgp_primitives bbuf;
+	struct pkt_legacy_bgp_primitives lbbuf;
 	struct pkt_nat_primitives nbuf;
 	struct pkt_mpls_primitives mbuf;
 	struct pkt_data abuf;
@@ -332,23 +353,28 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
           if (!following_chain) acc_elem = (struct acc *) elem;
 	  if (!test_zero_elem(acc_elem)) {
 	    /* XXX: support for custom and vlen primitives */
-	    mask_elem(&tbuf, &bbuf, &nbuf, &mbuf, acc_elem, request.what_to_count, request.what_to_count_2, extras); 
+	    mask_elem(&tbuf, &bbuf, &lbbuf, &nbuf, &mbuf, acc_elem, request.what_to_count, request.what_to_count_2, extras); 
             if (!memcmp(&tbuf, &request.data, sizeof(struct pkt_primitives)) &&
 		!memcmp(&bbuf, &request.pbgp, sizeof(struct pkt_bgp_primitives)) &&
+		!memcmp(&lbbuf, &request.plbgp, sizeof(struct pkt_legacy_bgp_primitives)) &&
 		!memcmp(&nbuf, &request.pnat, sizeof(struct pkt_nat_primitives)) &&
 		!memcmp(&mbuf, &request.pmpls, sizeof(struct pkt_mpls_primitives))) {
 	      if (q->type & WANT_COUNTER) Accumulate_Counters(&abuf, acc_elem); 
 	      else {
 		enQueue_elem(sd, &rb, acc_elem, PdataSz, datasize); /* q->type == WANT_MATCH */
 
-		if (extras->off_pkt_bgp_primitives) {
-		  if (acc_elem->cbgp) {
-		    struct pkt_bgp_primitives tmp_pbgp;
+                if (extras->off_pkt_bgp_primitives && acc_elem->pbgp) {
+                  enQueue_elem(sd, &rb, acc_elem->pbgp, PbgpSz, datasize - extras->off_pkt_bgp_primitives);
+                }
 
-		    cache_to_pkt_bgp_primitives(&tmp_pbgp, acc_elem->cbgp);
-		    enQueue_elem(sd, &rb, &tmp_pbgp, PbgpSz, datasize - extras->off_pkt_bgp_primitives);
-		  }
-		}
+                if (extras->off_pkt_lbgp_primitives) {
+                  if (acc_elem->clbgp) {
+                    struct pkt_legacy_bgp_primitives tmp_plbgp;
+
+                    cache_to_pkt_legacy_bgp_primitives(&tmp_plbgp, acc_elem->clbgp);
+                    enQueue_elem(sd, &rb, &tmp_plbgp, PlbgpSz, datasize - extras->off_pkt_lbgp_primitives);
+                  }
+                }
 
                 if (extras->off_pkt_nat_primitives && acc_elem->pnat) {
                   enQueue_elem(sd, &rb, acc_elem->pnat, PnatSz, datasize - extras->off_pkt_nat_primitives);
@@ -453,20 +479,22 @@ void process_query_data(int sd, unsigned char *buf, int len, struct extra_primit
   if (custbuf) free(custbuf);
 }
 
-void mask_elem(struct pkt_primitives *d1, struct pkt_bgp_primitives *d2, struct pkt_nat_primitives *d3,
-		struct pkt_mpls_primitives *d4, struct acc *src, pm_cfgreg_t w, pm_cfgreg_t w2,
-		struct extra_primitives *extras)
+void mask_elem(struct pkt_primitives *d1, struct pkt_bgp_primitives *d2, struct pkt_legacy_bgp_primitives *d5,
+		struct pkt_nat_primitives *d3, struct pkt_mpls_primitives *d4, struct acc *src, pm_cfgreg_t w,
+		pm_cfgreg_t w2, struct extra_primitives *extras)
 {
   struct pkt_primitives *s1 = &src->primitives;
-  struct pkt_bgp_primitives tmp_pbgp;
-  struct pkt_bgp_primitives *s2 = &tmp_pbgp;
+  struct pkt_bgp_primitives *s2 = src->pbgp;
+  struct pkt_legacy_bgp_primitives tmp_plbgp;
+  struct pkt_legacy_bgp_primitives *s5 = &tmp_plbgp;
   struct pkt_nat_primitives *s3 = src->pnat;
   struct pkt_mpls_primitives *s4 = src->pmpls;
 
-  cache_to_pkt_bgp_primitives(s2, src->cbgp);
+  cache_to_pkt_legacy_bgp_primitives(s5, src->clbgp);
 
   memset(d1, 0, sizeof(struct pkt_primitives));
   memset(d2, 0, sizeof(struct pkt_bgp_primitives));
+  memset(d5, 0, sizeof(struct pkt_legacy_bgp_primitives));
   memset(d3, 0, sizeof(struct pkt_nat_primitives));
   memset(d4, 0, sizeof(struct pkt_mpls_primitives));
 
@@ -494,6 +522,8 @@ void mask_elem(struct pkt_primitives *d1, struct pkt_bgp_primitives *d2, struct
   if (w & COUNT_TAG) d1->tag = s1->tag; 
   if (w & COUNT_TAG2) d1->tag2 = s1->tag2; 
   if (w & COUNT_CLASS) d1->class = s1->class; 
+  if (w2 & COUNT_EXPORT_PROTO_SEQNO) memcpy(&d1->export_proto_seqno, &s1->export_proto_seqno, sizeof(d1->export_proto_seqno));
+  if (w2 & COUNT_EXPORT_PROTO_VERSION) memcpy(&d1->export_proto_version, &s1->export_proto_version, sizeof(d1->export_proto_version));
 
 #if defined (WITH_GEOIP) || defined (WITH_GEOIPV2)
   if (w2 & COUNT_SRC_HOST_COUNTRY) memcpy(&d1->src_ip_country, &s1->src_ip_country, sizeof(d1->src_ip_country)); 
@@ -503,12 +533,6 @@ void mask_elem(struct pkt_primitives *d1, struct pkt_bgp_primitives *d2, struct
   if (w2 & COUNT_PKT_LEN_DISTRIB) d1->pkt_len_distrib = s1->pkt_len_distrib; 
 
   if (extras->off_pkt_bgp_primitives && s2) {
-    if (w & COUNT_STD_COMM) strlcpy(d2->std_comms, s2->std_comms, MAX_BGP_STD_COMMS); 
-    if (w & COUNT_SRC_STD_COMM) strlcpy(d2->src_std_comms, s2->src_std_comms, MAX_BGP_STD_COMMS); 
-    if (w & COUNT_EXT_COMM) strlcpy(d2->ext_comms, s2->ext_comms, MAX_BGP_EXT_COMMS); 
-    if (w & COUNT_SRC_EXT_COMM) strlcpy(d2->src_ext_comms, s2->src_ext_comms, MAX_BGP_EXT_COMMS); 
-    if (w & COUNT_AS_PATH) strlcpy(d2->as_path, s2->as_path, MAX_BGP_ASPATH);
-    if (w & COUNT_SRC_AS_PATH) strlcpy(d2->src_as_path, s2->src_as_path, MAX_BGP_ASPATH);
     if (w & COUNT_LOCAL_PREF) d2->local_pref = s2->local_pref;
     if (w & COUNT_SRC_LOCAL_PREF) d2->src_local_pref = s2->src_local_pref;
     if (w & COUNT_MED) d2->med = s2->med;
@@ -520,6 +544,15 @@ void mask_elem(struct pkt_primitives *d1, struct pkt_bgp_primitives *d2, struct
     if (w & COUNT_MPLS_VPN_RD) memcpy(&d2->mpls_vpn_rd, &s2->mpls_vpn_rd, sizeof(rd_t)); 
   }
 
+  if (extras->off_pkt_lbgp_primitives && s5) {
+    if (w & COUNT_STD_COMM) strlcpy(d5->std_comms, s5->std_comms, MAX_BGP_STD_COMMS); 
+    if (w & COUNT_EXT_COMM) strlcpy(d5->ext_comms, s5->ext_comms, MAX_BGP_EXT_COMMS); 
+    if (w & COUNT_AS_PATH) strlcpy(d5->as_path, s5->as_path, MAX_BGP_ASPATH);
+    if (w & COUNT_SRC_STD_COMM) strlcpy(d5->src_std_comms, s5->src_std_comms, MAX_BGP_STD_COMMS); 
+    if (w & COUNT_SRC_EXT_COMM) strlcpy(d5->src_ext_comms, s5->src_ext_comms, MAX_BGP_EXT_COMMS); 
+    if (w & COUNT_SRC_AS_PATH) strlcpy(d5->src_as_path, s5->src_as_path, MAX_BGP_ASPATH);
+  }
+
   if (extras->off_pkt_nat_primitives && s3) {
     if (w2 & COUNT_POST_NAT_SRC_HOST) memcpy(&d3->post_nat_src_ip, &s3->post_nat_src_ip, sizeof(d3->post_nat_src_ip));
     if (w2 & COUNT_POST_NAT_DST_HOST) memcpy(&d3->post_nat_src_ip, &s3->post_nat_dst_ip, sizeof(d3->post_nat_dst_ip));
@@ -528,6 +561,7 @@ void mask_elem(struct pkt_primitives *d1, struct pkt_bgp_primitives *d2, struct
     if (w2 & COUNT_NAT_EVENT) d3->nat_event = s3->nat_event;
     if (w2 & COUNT_TIMESTAMP_START) memcpy(&d3->timestamp_start, &s3->timestamp_start, sizeof(struct timeval));
     if (w2 & COUNT_TIMESTAMP_END) memcpy(&d3->timestamp_end, &s3->timestamp_end, sizeof(struct timeval));
+    if (w2 & COUNT_TIMESTAMP_ARRIVAL) memcpy(&d3->timestamp_arrival, &s3->timestamp_arrival, sizeof(struct timeval));
   }
 
   if (extras->off_pkt_mpls_primitives && s4) {
