@@ -38,6 +38,9 @@ char pmacctd_globstr[] = "pmacctd\0";
 char nfacctd_globstr[] = "nfacctd\0";
 char sfacctd_globstr[] = "sfacctd\0";
 char uacctd_globstr[] = "uacctd\0";
+char pmtele_globstr[] = "pmtelemetryd\0";
+char pmbgpd_globstr[] = "pmbgpd\0";
+char pmbmpd_globstr[] = "pmbgmd\0";
 
 /*
  * NEWSTR -- Create a copy of a C string
@@ -137,6 +140,9 @@ initsetproctitle(argc, argv, envp)
 	else if (config.acct_type == ACCT_NF) __progname = nfacctd_globstr;
 	else if (config.acct_type == ACCT_SF) __progname = sfacctd_globstr;
 	else if (config.acct_type == ACCT_UL) __progname = uacctd_globstr;
+	else if (config.acct_type == ACCT_PMTELE) __progname = pmtele_globstr;
+	else if (config.acct_type == ACCT_PMBGP) __progname = pmbgpd_globstr;
+	else if (config.acct_type == ACCT_PMBMP) __progname = pmbmpd_globstr;
 #endif
 }
 
@@ -161,6 +167,7 @@ setproctitle(fmt, va_alist)
 	union pstun pst;
 #  endif /* SPT_TYPE == SPT_PSTAT */
 
+	memset(buf, 0, SPT_BUFSIZE);
 	p = buf;
 	va_start(ap, fmt);
 	vsnprintf(p, SPACELEFT(buf), fmt, ap);
@@ -207,7 +214,7 @@ pm_setproctitle(fmt, va_alist)
 #endif /* __STDC__ */
 {
   char buf[SPT_BUFSIZE];
-  char prefix[10];
+  char prefix[16];
   va_list ap;
 
   memset(prefix, 0, sizeof(prefix));
@@ -217,6 +224,9 @@ pm_setproctitle(fmt, va_alist)
   else if (config.acct_type == ACCT_PM) strcpy(prefix, pmacctd_globstr);
   else if (config.acct_type == ACCT_NF) strcpy(prefix, nfacctd_globstr);
   else if (config.acct_type == ACCT_SF) strcpy(prefix, sfacctd_globstr);
+  else if (config.acct_type == ACCT_PMTELE) strcpy(prefix, pmtele_globstr);
+  else if (config.acct_type == ACCT_PMBGP) strcpy(prefix, pmbgpd_globstr);
+  else if (config.acct_type == ACCT_PMBMP) strcpy(prefix, pmbmpd_globstr);
 
   va_start(ap, fmt);
   vsnprintf(buf, sizeof(buf), fmt, ap);
