@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /*
@@ -33,23 +33,84 @@
 #include "ip_flow.h"
 #include "net_aggr.h"
 #include "thread_pool.h"
+#include "classifier.h"
+#include "bgp/bgp.h"
+#include "isis/isis.h"
+#include <netinet/ip.h>
+#include <libnfnetlink/libnfnetlink.h>
+#include <libnetfilter_log/libnetfilter_log.h>
 
 /* variables to be exported away */
-int debug;
-struct configuration config; /* global configuration */ 
-struct plugins_list_entry *plugins_list = NULL; /* linked list of each plugin configuration */ 
 struct channels_list_entry channels_list[MAX_N_PLUGINS]; /* communication channels: core <-> plugins */
-int have_num_memory_pools; /* global getopt() stuff */
-pid_t failed_plugins[MAX_N_PLUGINS]; /* plugins failed during startup phase */
-u_char dummy_tlhdr[16];
-
-#ifdef ENABLE_ULOG
 
 /* Functions */
+static int nflog_incoming(struct nflog_g_handle *gh, struct nfgenmsg *nfmsg,
+                          struct nflog_data *nfa, void *p)
+{
+  static char jumbo_container[10000];
+  struct timeval tv = {};
+  struct pcap_pkthdr hdr;
+  char *pkt = NULL;
+  ssize_t pkt_len = nflog_get_payload(nfa, &pkt);
+  ssize_t mac_len = nflog_get_msg_packet_hwhdrlen(nfa);
+  struct pcap_callback_data *cb_data = p;
+
+  /* Check we can handle this packet */
+  switch (nfmsg->nfgen_family) {
+  case AF_INET: break;
+#ifdef ENABLE_IPV6
+  case AF_INET6: break;
+#endif
+  default: return 0;
+  }
+
+  if (pkt_len == -1)
+    return -1;
+  nflog_get_timestamp(nfa, &tv);
+  hdr.ts = tv;
+  hdr.caplen = MIN(pkt_len, config.snaplen);
+  hdr.len = pkt_len;
+
+  cb_data->ifindex_in = nflog_get_physindev(nfa);
+  if (cb_data->ifindex_in == 0)
+    cb_data->ifindex_in = nflog_get_indev(nfa);
+
+  cb_data->ifindex_out = nflog_get_physoutdev(nfa);
+  if (cb_data->ifindex_out == 0)
+    cb_data->ifindex_out = nflog_get_outdev(nfa);
+
+#if defined (HAVE_L2)
+  if (mac_len) {
+    memcpy(jumbo_container, nflog_get_msg_packet_hwhdr(nfa), mac_len);
+    memcpy(jumbo_container + mac_len, pkt, hdr.caplen);
+    hdr.caplen += mac_len;
+    hdr.len += mac_len;
+  } else {
+    memset(jumbo_container, 0, ETHER_HDRLEN);
+    memcpy(jumbo_container+ETHER_HDRLEN, pkt, hdr.caplen);
+    hdr.caplen += ETHER_HDRLEN;
+    hdr.len += ETHER_HDRLEN;
+
+    switch (nfmsg->nfgen_family) {
+    case AF_INET:
+      ((struct eth_header *)jumbo_container)->ether_type = ntohs(ETHERTYPE_IP);
+      break;
+    case AF_INET6:
+      ((struct eth_header *)jumbo_container)->ether_type = ntohs(ETHERTYPE_IPV6);
+      break;
+    }
+  }
+
+  pcap_cb((u_char *) cb_data, &hdr, jumbo_container);
+#else
+  pcap_cb((u_char *) cb_data, &hdr, pkt);
+#endif
+}
+
 void usage_daemon(char *prog_name)
 {
   printf("%s (%s)\n", UACCTD_USAGE_HEADER, PMACCT_BUILD);
-  printf("Usage: %s [ -D | -d ] [ -g ULOG group ] [ -c primitive [ , ... ] ] [ -P plugin [ , ... ] ]\n", prog_name);
+  printf("Usage: %s [ -D | -d ] [ -g NFLOG group ] [ -c primitive [ , ... ] ] [ -P plugin [ , ... ] ]\n", prog_name);
   printf("       %s [ -f config_file ]\n", prog_name);
   printf("       %s [ -h ]\n", prog_name);
   printf("\nGeneral options:\n");
@@ -59,27 +120,27 @@ void usage_daemon(char *prog_name)
   printf("  -a  \tPrint list of supported aggregation primitives\n");
   printf("  -c  \tAggregation method, see full list of primitives with -a (DEFAULT: src_host)\n");
   printf("  -D  \tDaemonize\n"); 
-  printf("  -n  \tPath to a file containing Network definitions\n");
-  printf("  -o  \tPath to a file containing Port definitions\n");
-  printf("  -P  \t[ memory | print | mysql | pgsql | sqlite3 | mongodb | nfprobe | sfprobe ] \n\tActivate plugin\n"); 
+  printf("  -n  \tPath to a file containing networks and/or ASNs definitions\n");
+  printf("  -t  \tPath to a file containing ports definitions\n");
+  printf("  -P  \t[ memory | print | mysql | pgsql | sqlite3 | mongodb | amqp | kafka | nfprobe | sfprobe ] \n\tActivate plugin\n"); 
   printf("  -d  \tEnable debug\n");
   printf("  -S  \t[ auth | mail | daemon | kern | user | local[0-7] ] \n\tLog to the specified syslog facility\n");
   printf("  -F  \tWrite Core Process PID into the specified file\n");
   printf("  -R  \tRenormalize sampled data\n");
-  printf("  -g  \tNetlink ULOG group\n");
-  printf("  -L  \tNetlink socket read buffer size\n");
+  printf("  -g  \tNetlink NFLOG group\n");
+  printf("  -L  \tSnapshot length\n");
   printf("  -u  \tLeave IP protocols in numerical format\n");
   printf("\nMemory plugin (-P memory) options:\n");
   printf("  -p  \tSocket for client-server communication (DEFAULT: /tmp/collect.pipe)\n");
   printf("  -b  \tNumber of buckets\n");
   printf("  -m  \tNumber of memory pools\n");
   printf("  -s  \tMemory pool size\n");
-  printf("\nPostgreSQL (-P pgsql)/MySQL (-P mysql)/SQLite (-P sqlite3) plugin options:\n");
-  printf("  -r  \tRefresh time (in seconds)\n");
-  printf("  -v  \t[ 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 ] \n\tTable version\n");
   printf("\nPrint plugin (-P print) plugin options:\n");
   printf("  -r  \tRefresh time (in seconds)\n");
-  printf("  -O  \t[ formatted | csv | json ] \n\tOutput format\n");
+  printf("  -O  \t[ formatted | csv | json | avro ] \n\tOutput format\n");
+  printf("  -o  \tPath to output file\n");
+  printf("  -A  \tAppend output (applies to -o)\n");
+  printf("  -E  \tCSV format serparator (applies to -O csv, DEFAULT: ',')\n");
   printf("\n");
   printf("  See QUICKSTART or visit http://wiki.pmacct.net/ for examples.\n");
   printf("\n");
@@ -89,16 +150,12 @@ void usage_daemon(char *prog_name)
 
 int main(int argc,char **argv, char **envp)
 {
-  bpf_u_int32 localnet, netmask;  /* pcap library stuff */
-  struct bpf_program filter;
   struct pcap_device device;
-  char errbuf[PCAP_ERRBUF_SIZE];
   int index, logf, ret;
 
   struct plugins_list_entry *list;
   struct plugin_requests req;
   char config_file[SRVBUFLEN];
-  int psize = ULOG_BUFLEN;
 
   struct id_table bpas_table;
   struct id_table blp_table;
@@ -110,23 +167,14 @@ int main(int argc,char **argv, char **envp)
   /* getopt() stuff */
   extern char *optarg;
   extern int optind, opterr, optopt;
-  int errflag, cp; 
+  int errflag, cp;
 
-  /* ULOG stuff */
-  int ulog_fd, one = 1;
-  struct nlmsghdr *nlh;
-  struct sockaddr_nl nls;
-  ulog_packet_msg_t *ulog_pkt;
+  /* NFLOG stuff */
+  struct nflog_handle *nfh = NULL;
+  struct nflog_g_handle *nfgh = NULL;
+  int one = 1;
   ssize_t len = 0;
-  socklen_t alen;
-  unsigned char *ulog_buffer;
-  struct pcap_pkthdr hdr;
-  struct timeval tv;
-
-  char jumbo_container[10000];
-  u_int8_t mac_len;
-
-
+  unsigned char *nflog_buffer;
 
 #if defined ENABLE_IPV6
   struct sockaddr_storage client;
@@ -142,7 +190,6 @@ int main(int argc,char **argv, char **envp)
   compute_once();
 
   /* a bunch of default definitions */ 
-  have_num_memory_pools = FALSE;
   reload_map = FALSE;
   reload_geoipv2_file = FALSE;
   bpas_map_allocated = FALSE;
@@ -153,6 +200,7 @@ int main(int argc,char **argv, char **envp)
   sampling_map_caching = FALSE;
   custom_primitives_allocated = FALSE;
   find_id_func = PM_find_id;
+  plugins_list = NULL;
 
   errflag = 0;
 
@@ -181,7 +229,8 @@ int main(int argc,char **argv, char **envp)
 
   /* getting commandline values */
   while (!errflag && ((cp = getopt(argc, argv, ARGS_UACCTD)) != -1)) {
-    cfg_cmdline[rows] = malloc(SRVBUFLEN);
+    if (!cfg_cmdline[rows]) cfg_cmdline[rows] = malloc(SRVBUFLEN);
+    memset(cfg_cmdline[rows], 0, SRVBUFLEN);
     switch (cp) {
     case 'P':
       strlcpy(cfg_cmdline[rows], "plugins: ", SRVBUFLEN);
@@ -202,7 +251,7 @@ int main(int argc,char **argv, char **envp)
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
       rows++;
       break;
-    case 'o':
+    case 't':
       strlcpy(cfg_cmdline[rows], "ports_file: ", SRVBUFLEN);
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
       rows++;
@@ -212,12 +261,29 @@ int main(int argc,char **argv, char **envp)
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
       rows++;
       break;
+    case 'o':
+      strlcpy(cfg_cmdline[rows], "print_output_file: ", SRVBUFLEN);
+      strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
+      rows++;
+      break;
+    case 'A':
+      strlcpy(cfg_cmdline[rows], "print_output_file_append: ", SRVBUFLEN);
+      strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
+      rows++;
+      break;
+    case 'E':
+      strlcpy(cfg_cmdline[rows], "print_output_separator: ", SRVBUFLEN);
+      strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
+      rows++;
+      break;
     case 'u':
       strlcpy(cfg_cmdline[rows], "print_num_protos: true", SRVBUFLEN);
       rows++;
       break;
     case 'f':
       strlcpy(config_file, optarg, sizeof(config_file));
+      free(cfg_cmdline[rows]);
+      cfg_cmdline[rows] = NULL;
       break;
     case 'F':
       strlcpy(cfg_cmdline[rows], "pidfile: ", SRVBUFLEN);
@@ -237,7 +303,6 @@ int main(int argc,char **argv, char **envp)
     case 'm':
       strlcpy(cfg_cmdline[rows], "imt_mem_pools_number: ", SRVBUFLEN);
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
-      have_num_memory_pools = TRUE;
       rows++;
       break;
     case 'p':
@@ -250,11 +315,6 @@ int main(int argc,char **argv, char **envp)
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
       rows++;
       break;
-    case 'v':
-      strlcpy(cfg_cmdline[rows], "sql_table_version: ", SRVBUFLEN);
-      strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
-      rows++;
-      break;
     case 's':
       strlcpy(cfg_cmdline[rows], "imt_mem_pools_size: ", SRVBUFLEN);
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
@@ -324,8 +384,9 @@ int main(int argc,char **argv, char **envp)
 
   if (config.files_umask) umask(config.files_umask);
 
-  if (!config.snaplen) config.snaplen = psize;
-  if (!config.uacctd_nl_size) config.uacctd_nl_size = psize;
+  if (!config.snaplen) config.snaplen = DEFAULT_SNAPLEN;
+  if (!config.uacctd_nl_size) config.uacctd_nl_size = DEFAULT_NFLOG_BUFLEN;
+  if (!config.uacctd_threshold) config.uacctd_threshold = DEFAULT_NFLOG_THRESHOLD;
 
   /* Let's check whether we need superuser privileges */
   if (getuid() != 0) {
@@ -335,10 +396,10 @@ int main(int argc,char **argv, char **envp)
   }
 
   if (!config.uacctd_group) {
-    config.uacctd_group = DEFAULT_ULOG_GROUP;
+    config.uacctd_group = DEFAULT_NFLOG_GROUP;
     list = plugins_list;
     while (list) {
-      list->cfg.uacctd_group = DEFAULT_ULOG_GROUP;
+      list->cfg.uacctd_group = DEFAULT_NFLOG_GROUP;
       list = list->next;
     }
   }
@@ -346,11 +407,12 @@ int main(int argc,char **argv, char **envp)
   if (config.daemon) {
     list = plugins_list;
     while (list) {
-      if (!strcmp(list->type.string, "print")) printf("INFO ( %s/core ): Daemonizing. Hmm, bye bye screen.\n", config.name);
+      if (!strcmp(list->type.string, "print") && !list->cfg.print_output_file)
+        printf("INFO ( %s/%s ): Daemonizing. Bye bye screen.\n", list->name, list->type.string);
       list = list->next;
     }
     if (debug || config.debug)
-      printf("WARN ( %s/core ): debug is enabled; forking in background. Console logging will get lost.\n", config.name); 
+      printf("WARN ( %s/core ): debug is enabled; forking in background. Logging to standard error (stderr) will get lost.\n", config.name); 
     daemonize();
   }
 
@@ -359,7 +421,7 @@ int main(int argc,char **argv, char **envp)
     logf = parse_log_facility(config.syslog);
     if (logf == ERR) {
       config.syslog = NULL;
-      printf("WARN ( %s/core ): specified syslog facility is not supported; logging to console.\n", config.name);
+      printf("WARN ( %s/core ): specified syslog facility is not supported. Logging to standard error (stderr).\n", config.name);
     }
     else openlog(NULL, LOG_PID, logf);
     Log(LOG_INFO, "INFO ( %s/core ): Start logging ...\n", config.name);
@@ -367,7 +429,7 @@ int main(int argc,char **argv, char **envp)
 
   if (config.logfile)
   {
-    config.logfile_fd = open_logfile(config.logfile, "a");
+    config.logfile_fd = open_output_file(config.logfile, "a", FALSE);
     list = plugins_list;
     while (list) {
       list->cfg.logfile_fd = config.logfile_fd ;
@@ -383,6 +445,9 @@ int main(int argc,char **argv, char **envp)
     else Log(LOG_INFO, "INFO ( %s/core ): proc_priority set to %d\n", config.name, getpriority(PRIO_PROCESS, 0));
   }
 
+  Log(LOG_INFO, "INFO ( %s/core ): %s (%s)\n", config.name, UACCTD_USAGE_HEADER, PMACCT_BUILD);
+  Log(LOG_INFO, "INFO ( %s/core ): %s\n", config.name, PMACCT_COMPILE_ARGS);
+
   if (strlen(config_file)) {
     char canonical_path[PATH_MAX], *canonical_path_ptr;
 
@@ -396,15 +461,19 @@ int main(int argc,char **argv, char **envp)
   while (list) {
     if (list->type.id != PLUGIN_ID_CORE) {
       /* applies to all plugins */
+      plugin_pipe_check(&list->cfg);
+
       if (config.classifiers_path && (list->cfg.sampling_rate || config.ext_sampling_rate)) {
         Log(LOG_ERR, "ERROR ( %s/core ): Packet sampling and classification are mutual exclusive.\n", config.name);
         exit(1);
       }
+
       if (list->cfg.sampling_rate && config.ext_sampling_rate) {
         Log(LOG_ERR, "ERROR ( %s/core ): Internal packet sampling and external packet sampling are mutual exclusive.\n", config.name);
         exit(1);
       }
 
+      /* applies to specific plugins */
       if (list->type.id == PLUGIN_ID_TEE) {
         Log(LOG_ERR, "ERROR ( %s/core ): 'tee' plugin not supported in 'uacctd'.\n", config.name);
         exit(1);
@@ -480,7 +549,7 @@ int main(int argc,char **argv, char **envp)
            we cancel the sampling information from the probe plugin */
         if (config.sfacctd_renormalize && list->cfg.ext_sampling_rate) list->cfg.ext_sampling_rate = 0;
 
-	if (psize < 128) psize = config.snaplen = 128; /* SFL_DEFAULT_HEADER_SIZE */
+	if (config.snaplen < 128) config.snaplen = 128; /* SFL_DEFAULT_HEADER_SIZE */
 	list->cfg.what_to_count = COUNT_PAYLOAD;
 	list->cfg.what_to_count_2 = 0;
 	if (list->cfg.classifiers_path) {
@@ -525,7 +594,7 @@ int main(int argc,char **argv, char **envp)
       else {
         if (list->cfg.what_to_count_2 & (COUNT_POST_NAT_SRC_HOST|COUNT_POST_NAT_DST_HOST|
                         COUNT_POST_NAT_SRC_PORT|COUNT_POST_NAT_DST_PORT|COUNT_NAT_EVENT|
-                        COUNT_TIMESTAMP_START|COUNT_TIMESTAMP_END))
+                        COUNT_TIMESTAMP_START|COUNT_TIMESTAMP_END|COUNT_TIMESTAMP_ARRIVAL))
           list->cfg.data_type |= PIPE_TYPE_NAT;
 
         if (list->cfg.what_to_count_2 & (COUNT_MPLS_LABEL_TOP|COUNT_MPLS_LABEL_BOTTOM|
@@ -550,8 +619,8 @@ int main(int argc,char **argv, char **envp)
 	  Log(LOG_WARNING, "WARN ( %s/%s ): defaulting to SRC HOST aggregation.\n", list->name, list->type.string);
 	  list->cfg.what_to_count |= COUNT_SRC_HOST;
 	}
-        if ((list->cfg.what_to_count & COUNT_SRC_HOST) && (list->cfg.what_to_count & COUNT_SRC_NET) ||
-            (list->cfg.what_to_count & COUNT_DST_HOST) && (list->cfg.what_to_count & COUNT_DST_NET)) {
+        if (((list->cfg.what_to_count & COUNT_SRC_HOST) && (list->cfg.what_to_count & COUNT_SRC_NET)) ||
+            ((list->cfg.what_to_count & COUNT_DST_HOST) && (list->cfg.what_to_count & COUNT_DST_NET))) {
           if (!list->cfg.tmp_net_own_field) {
             Log(LOG_ERR, "ERROR ( %s/%s ): src_host, src_net and dst_host, dst_net are mutually exclusive: set tmp_net_own_field to true. Exiting...\n\n", list->name, list->type.string);
             exit(1);
@@ -592,6 +661,7 @@ int main(int argc,char **argv, char **envp)
 	  exit(1);
 	}
 
+	list->cfg.type_id = list->type.id;
 	bgp_config_checks(&list->cfg);
 
 	list->cfg.what_to_count |= COUNT_COUNTERS;
@@ -647,46 +717,83 @@ int main(int argc,char **argv, char **envp)
   signal(SIGUSR2, reload_maps); /* sets to true the reload_maps flag */
   signal(SIGPIPE, SIG_IGN); /* we want to exit gracefully when a pipe is broken */
 
-  ulog_fd = socket(PF_NETLINK, SOCK_RAW, NETLINK_NFLOG);
-  if (ulog_fd == -1) {
-    Log(LOG_ERR, "ERROR ( %s/core ): Failed to create Netlink ULOG socket\n", config.name);
+  nfh = nflog_open();
+  if (nfh == NULL) {
+    Log(LOG_ERR, "ERROR ( %s/core ): Failed to create Netlink NFLOG socket\n", config.name);
+    nflog_close(nfh);
     exit_all(1);
   }
 
-  Log(LOG_INFO, "INFO ( %s/core ): Successfully connected Netlink ULOG socket\n", config.name);
+  Log(LOG_INFO, "INFO ( %s/core ): Successfully connected Netlink NFLOG socket\n", config.name);
+
+  /* Bind to IPv4 (and IPv6) */
+  if (nflog_unbind_pf(nfh, AF_INET) < 0) {
+    Log(LOG_ERR, "ERROR ( %s/core ): Failed to unbind Netlink NFLOG socket from IPv4\n", config.name);
+    nflog_close(nfh);
+    exit_all(1);
+  }
+  if (nflog_bind_pf(nfh, AF_INET) < 0) {
+    Log(LOG_ERR, "ERROR ( %s/core ): Failed to bind Netlink NFLOG socket from IPv4\n", config.name);
+    nflog_close(nfh);
+    exit_all(1);
+  }
+#if defined ENABLE_IPV6
+  if (nflog_unbind_pf(nfh, AF_INET6) < 0) {
+    Log(LOG_ERR, "ERROR ( %s/core ): Failed to unbind Netlink NFLOG socket from IPv6\n", config.name);
+    nflog_close(nfh);
+    exit_all(1);
+  }
+  if (nflog_bind_pf(nfh, AF_INET6) < 0) {
+    Log(LOG_ERR, "ERROR ( %s/core ): Failed to bind Netlink NFLOG socket from IPv6\n", config.name);
+    nflog_close(nfh);
+    exit_all(1);
+  }
+#endif
+
+  /* Bind to group */
+  if ((nfgh = nflog_bind_group(nfh, config.uacctd_group)) == NULL) {
+    Log(LOG_ERR, "ERROR ( %s/core ): Failed to join NFLOG group %d\n", config.name, config.uacctd_group);
+    nflog_close(nfh);
+    exit_all(1);
+  }
+
+  /* Set snaplen */
+  if (nflog_set_mode(nfgh, NFULNL_COPY_PACKET, config.snaplen) < 0) {
+    Log(LOG_ERR, "ERROR ( %s/core ): Failed to set snaplen to %d\n", config.name, config.snaplen);
+    nflog_unbind_group(nfgh);
+    nflog_close(nfh);
+    exit_all(1);
+  }
+
+  /* Set threshold */
+  if (nflog_set_qthresh(nfgh, config.uacctd_threshold) < 0) {
+    Log(LOG_ERR, "ERROR ( %s/core ): Failed to set threshold to %d\n", config.name, config.uacctd_threshold);
+    nflog_unbind_group(nfgh);
+    nflog_close(nfh);
+    exit_all(1);
+  }
+
+  /* Set buffer size */
+  if (nflog_set_nlbufsiz(nfgh, config.uacctd_nl_size) < 0) {
+    Log(LOG_ERR, "ERROR ( %s/core ): Failed to set receive buffer size to %d\n", config.name, config.uacctd_nl_size);
+    nflog_unbind_group(nfgh);
+    nflog_close(nfh);
+    exit_all(1);
+  }
 
   /* Turn off netlink errors from overrun. */
-  if (setsockopt(ulog_fd, SOL_NETLINK, NETLINK_NO_ENOBUFS, &one, sizeof(one)))
+  if (setsockopt(nflog_fd(nfh), SOL_NETLINK, NETLINK_NO_ENOBUFS, &one, sizeof(one)))
     Log(LOG_ERR, "ERROR ( %s/core ): Failed to turn off netlink ENOBUFS\n", config.name);
 
-  if (config.uacctd_nl_size > ULOG_BUFLEN) {
-    /* If configured buffer size is larger than default 4KB */
-    if (setsockopt(ulog_fd, SOL_SOCKET, SO_RCVBUF, &config.uacctd_nl_size, sizeof(config.uacctd_nl_size)))
-      Log(LOG_ERR, "ERROR ( %s/core ): Failed to set Netlink receive buffer size\n", config.name);
-    else
-      Log(LOG_INFO, "INFO ( %s/core ): Netlink receive buffer size set to %u\n", config.name, config.uacctd_nl_size);
-  }
-
-  ulog_buffer = malloc(config.snaplen);
-  if (ulog_buffer == NULL) {
-    Log(LOG_ERR, "ERROR ( %s/core ): ULOG buffer malloc() failed\n", config.name);
-    close(ulog_fd);
+  nflog_callback_register(nfgh, &nflog_incoming, &cb_data);
+  nflog_buffer = malloc(config.uacctd_nl_size);
+  if (nflog_buffer == NULL) {
+    Log(LOG_ERR, "ERROR ( %s/core ): NFLOG buffer malloc() failed\n", config.name);
+    nflog_unbind_group(nfgh);
+    nflog_close(nfh);
     exit_all(1);
   }
 
-  memset(&nls, 0, sizeof(nls));
-  nls.nl_family = AF_NETLINK;
-  nls.nl_pid = getpid();
-  nls.nl_groups = config.uacctd_group;
-  alen = sizeof(nls);
-
-  if (bind(ulog_fd, (struct sockaddr *) &nls, sizeof(nls))) {
-    Log(LOG_ERR, "ERROR ( %s/core ): bind() to Netlink ULOG socket failed\n", config.name);
-    close(ulog_fd);
-    exit_all(1);
-  }
-  Log(LOG_INFO, "INFO ( %s/core ): Netlink ULOG: binding to group %u\n", config.name, config.uacctd_group);
-
 #if defined ENABLE_THREADS
   /* starting the ISIS threa */
   if (config.nfacctd_isis) {
@@ -709,7 +816,7 @@ int main(int argc,char **argv, char **envp)
 	cb_data.bpas_table = (u_char *) &bpas_table;
       }
       else {
-        Log(LOG_ERR, "ERROR: bgp_peer_as_src_type set to 'map' but no map defined. Exiting.\n");
+        Log(LOG_ERR, "ERROR ( %s/core ): bgp_peer_as_src_type set to 'map' but no map defined. Exiting.\n", config.name);
         exit(1);
       }
     }
@@ -721,7 +828,7 @@ int main(int argc,char **argv, char **envp)
         cb_data.blp_table = (u_char *) &blp_table;
       }
       else {
-        Log(LOG_ERR, "ERROR: bgp_src_local_pref_type set to 'map' but no map defined. Exiting.\n");
+        Log(LOG_ERR, "ERROR ( %s/core ): bgp_src_local_pref_type set to 'map' but no map defined. Exiting.\n", config.name);
         exit(1);
       }
     }
@@ -733,7 +840,7 @@ int main(int argc,char **argv, char **envp)
         cb_data.bmed_table = (u_char *) &bmed_table;
       }
       else {
-        Log(LOG_ERR, "ERROR: bgp_src_med_type set to 'map' but no map defined. Exiting.\n");
+        Log(LOG_ERR, "ERROR ( %s/core ): bgp_src_med_type set to 'map' but no map defined. Exiting.\n", config.name);
         exit(1);
       }
     }
@@ -790,8 +897,7 @@ int main(int argc,char **argv, char **envp)
 
   /* plugins glue: creation (until 093) */
   evaluate_packet_handlers();
-  if (!config.proc_name) pm_setproctitle("%s [%s]", "Core Process", "default");
-  else pm_setproctitle("%s [%s]", "Core Process", config.proc_name);
+  pm_setproctitle("%s [%s]", "Core Process", config.proc_name);
   if (config.pidfile) write_pid_file(config.pidfile);  
 
   /* signals to be handled only by pmacctd;
@@ -814,158 +920,8 @@ int main(int argc,char **argv, char **envp)
       }
     }
 
-    len = recvfrom(ulog_fd, ulog_buffer, config.snaplen, 0, (struct sockaddr*) &nls, &alen);
-
-    /*
-     * Read timeout or failure condition.
-     */
-    if (len < (int)sizeof(struct nlmsghdr)) continue;
-    if (alen != sizeof(nls)) continue;
-
-    nlh = (struct nlmsghdr*) ulog_buffer;
-    if ((nlh->nlmsg_flags & MSG_TRUNC) || ((size_t)len > config.snaplen)) continue;
-
-    gettimeofday(&tv, NULL);
-
-    while (NLMSG_OK(nlh, (size_t)len)) {
-      ulog_pkt = NLMSG_DATA(nlh);
-      hdr.ts = tv;
-      hdr.caplen = MIN(ulog_pkt->data_len, config.snaplen);
-      hdr.len = ulog_pkt->data_len;
-
-      if (strlen(ulog_pkt->indev_name) > 1) {
-	cb_data.ifindex_in = cache_ifindex(ulog_pkt->indev_name, tv.tv_sec);
-      }
-      else cb_data.ifindex_in = 0;
-
-      if (strlen(ulog_pkt->outdev_name) > 1) {
-	cb_data.ifindex_out = cache_ifindex(ulog_pkt->outdev_name, tv.tv_sec);
-      }
-      else cb_data.ifindex_out = 0;
-
-#if defined (HAVE_L2)
-      if (ulog_pkt->mac_len) {
-	memcpy(jumbo_container, ulog_pkt->mac, ulog_pkt->mac_len);
-	memcpy(jumbo_container+ulog_pkt->mac_len, ulog_pkt->payload, hdr.caplen);
-	// XXX
-	hdr.caplen += ulog_pkt->mac_len;
-	hdr.len += ulog_pkt->mac_len;
-      }
-      else {
-	memset(jumbo_container, 0, ETHER_HDRLEN);
-	memcpy(jumbo_container+ETHER_HDRLEN, ulog_pkt->payload, hdr.caplen);
-	hdr.caplen += ETHER_HDRLEN;
-	hdr.len += ETHER_HDRLEN;
-
-	switch (IP_V((struct my_iphdr *) ulog_pkt->payload)) {
-	case 4:
-	  ((struct eth_header *)jumbo_container)->ether_type = ntohs(ETHERTYPE_IP);
-	  break;
-	case 6:
-	  ((struct eth_header *)jumbo_container)->ether_type = ntohs(ETHERTYPE_IPV6);
-	  break;
-	}
-
-      }
-
-      pcap_cb((u_char *) &cb_data, &hdr, jumbo_container);
-#else
-      pcap_cb((u_char *) &cb_data, &hdr, ulog_pkt->payload);
-#endif
-
-      if (nlh->nlmsg_type == NLMSG_DONE || !(nlh->nlmsg_flags & NLM_F_MULTI)) {
-        /* Last part of the multilink message */
-        break;
-      }
-      nlh = NLMSG_NEXT(nlh, len);
-    }
+    len = recv(nflog_fd(nfh), nflog_buffer, config.uacctd_nl_size, 0);
+    if (len < 0) continue;
+    if (nflog_handle_packet(nfh, nflog_buffer, len) != 0) continue;
   }
 }
-
-unsigned int get_ifindex(char *device) 
-{
-  static int sock = -1;
-
-  if (sock < 0) {
-    sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
-    if (sock < 0) {
-      Log(LOG_ERR, "ERROR: Unable to open socket for ifindex");
-      return -1;
-    }
-  }
-  
-  struct ifreq req;
-  strcpy(req.ifr_name, device);
-  if (ioctl(sock, SIOCGIFINDEX, &req)) {
-    Log(LOG_ERR, "ERROR: Interface %s not found\n", device);
-    return -1;
-  }
-
-  return req.ifr_ifindex;
-}
-
-unsigned int hash_ifname(char *name)
-{
-  unsigned hash = 0;
-
-  while (*name)
-    hash = 33 * hash + *name++;
-
-  return (hash & IFCACHE_HASHSIZ-1);
-}
-
-/* Cache name to ifindex mapping */
-unsigned int cache_ifindex(char *device, unsigned long now)
-{
-  struct ifname_cache *ifc, **top;
-  unsigned int ifindex;
-
-  top = &hash_heads[hash_ifname(device)];
-  while ( (ifc = *top) != NULL) {
-    if (strncmp(device, ifc->name, IFNAMSIZ)) {
-      top = &ifc->next;
-      continue;
-    }
-
-    /* prune old entry to deal with hotplug */
-    if ((long)(now - ifc->tstamp) > IFCACHE_LIFETIME) {
-      *top = ifc->next;
-      free(ifc);
-      break;
-    }
-
-    return ifc->index;
-  }
-
-  ifindex = get_ifindex(device);
-  if (ifindex) {
-    ifc = malloc(sizeof(struct ifname_cache));
-    if (ifc) {
-      ifc->index = ifindex;
-      strncpy(ifc->name, device, IFNAMSIZ);
-      ifc->tstamp = now;
-      ifc->next = *top;
-      *top = ifc;
-    }
-  }
-
-  return ifindex;
-}
-
-#else
-
-int main(int argc,char **argv, char **envp)
-{
-  printf("WARN: uacctd (Linux NetFilter ULOG accounting) daemon is not active. This is enabled by --enable-ulog\n");
-}
-
-#endif
-
-/* Dummy objects here - ugly to see but well portable */
-void NF_find_id(struct id_table *t, struct packet_ptrs *pptrs, pm_id_t *tag, pm_id_t *tag2)
-{
-}
-
-void SF_find_id(struct id_table *t, struct packet_ptrs *pptrs, pm_id_t *tag, pm_id_t *tag2)
-{
-}
